Package: highdir
Title: Backend-Agnostic Figure Builder for 'highcharter' and 'ggplot2'
Version: 0.5.0
Authors@R: 
  person(
    given = "Yusman",
    family = "Kamaleri",
    email = "ybkamaleri@gmail.com",
    role = c("aut", "cre"),
    comment = c(ORCID = "0000-0001-5014-3665")
  )
Description: Provides a backend-agnostic 'API' for creating data visualizations
    using 'highcharter' (interactive) or 'ggplot2' (static). Figures are
    defined once via a specification object and can be rendered to either
    backend without modifying the calling code. Supports both declarative
    and layered workflows, flexible theming and colour palettes, optional
    'JavaScript' enhancements, and tools for exporting figures and interactive
    exploration via a 'shiny' app.
URL: https://folkehelsestats.github.io/highdir/,
        https://github.com/folkehelsestats/highdir
BugReports: https://github.com/folkehelsestats/highdir/issues
License: MIT + file LICENSE
Encoding: UTF-8
LazyData: true
Depends: R (>= 4.1.0)
Imports: highcharter (>= 0.9.4), ggplot2 (>= 3.4.0), htmlwidgets (>=
        1.6.0), jsonlite (>= 1.8.0), rlang (>= 1.1.0), viridis (>=
        0.6.0), shiny (>= 1.7.0), tools, utils, stats
Suggests: sf (>= 1.0.0), maps (>= 3.4.0), webshot2, bslib (>= 0.7.0),
        testthat (>= 3.0.0), withr (>= 2.5.0), data.table, knitr,
        rmarkdown, tibble
Config/testthat/edition: 3
VignetteBuilder: knitr
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-05-19 13:26:55 UTC; ykama
Author: Yusman Kamaleri [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-5014-3665>)
Maintainer: Yusman Kamaleri <ybkamaleri@gmail.com>
Repository: CRAN
Date/Publication: 2026-05-27 09:40:06 UTC
Built: R 4.5.2; ; 2026-05-27 11:23:58 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method("+",hd)
S3method("+",hd_gg_theme)
S3method(as.list,hd_opts)
S3method(as.list,hd_spec)
S3method(print,hd)
S3method(print,hd_geom)
S3method(print,hd_opts)
S3method(print,hd_spec)
export(geom_args)
export(get_palette)
export(gg_theme)
export(hd)
export(hd_add_js)
export(hd_app)
export(hd_geom_arearange)
export(hd_geom_column)
export(hd_geom_line)
export(hd_geom_pie)
export(hd_geom_ranked_bar)
export(hd_geom_scatter)
export(hd_geom_stacked_column)
export(hd_make)
export(hd_opts)
export(hd_save)
export(hd_set_theme)
export(hd_spec)
export(hd_theme)
export(list_backends)
export(list_geoms)
export(list_palettes)
export(register_backend)
export(register_geom)
export(register_palette)
importFrom(rlang,.data)
importFrom(stats,setNames)
importFrom(tools,file_ext)
importFrom(tools,file_path_sans_ext)
importFrom(utils,head)
importFrom(utils,modifyList)

<!-- README.md is generated from README.Rmd. Please edit that file -->

# highdir <a href="https://github.com/folkehelsestats/highdir"><img src="man/figures/logo.png" align="right" height="120" alt="highdir website" /></a>

[![R-CMD-check](https://github.com/folkehelsestats/highdir/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/folkehelsestats/highdir/actions/workflows/R-CMD-check.yaml)
[![Codecov test
coverage](https://codecov.io/gh/folkehelsestats/highdir/graph/badge.svg)](https://app.codecov.io/gh/folkehelsestats/highdir)
[![Lifecycle:
experimental](https://img.shields.io/badge/lifecycle-experimental-orange.svg)](https://lifecycle.r-lib.org/articles/stages.html#experimental)

**highdir** is an R package that provides a unified, backend-agnostic
API for building figures with either
[**highcharter**](https://jkunst.com/highcharter/) (interactive) or
[**ggplot2**](https://ggplot2.tidyverse.org/) (static).

The package provides two complementary yet fully interoperable APIs for
creating figures:

1.  **Declarative API** - define all components of a figure up front and
    render it in a single step.
2.  **Layered API** - incrementally compose a figure by adding layers,
    similar to the *grammar of graphics*.

Both APIs produce equivalent visual output and can target any supported
backend without changes to the calling code.

With the declarative API, a figure is specified once as an `hd_spec`
object and can later be rendered to different backends. Backend-specific
presentation options (such as interactivity or styling tweaks) can be
supplied separately via an `hc_opts` object prior to rendering.

The layered API supports an exploratory, iterative workflow. Figures are
built step by step using `+` similar to **ggplot2** style, making this
approach particularly well suited for interactive analysis and rapid
prototyping, while remaining backend-independent.

By default, **highdir** ships with colour palette, theme, and visual
identity of [The Norwegian Directorate of
Health](https://www.helsedirektoratet.no) (*Helsedirektoratet*). To
further enhance usability, a Shiny graphical user interfacef for
building and previewing figures is also included as part of the package.

------------------------------------------------------------------------

## Installation

``` r
# Install from GitHub
if(!require(remotes)) install.packages("remotes")
remotes::install_github("folkehelsestats/highdir")

# Install from development version (dev branch)
remotes::install_github("folkehelsestats/highdir@dev")
```

------------------------------------------------------------------------

## Get started

The simplest way to get started with *highdir* is by using the built-in
Shiny app. It shows also codes to demonstrate how to use the package
programmatically in R. Start the app with:

``` r
library(highdir)
hd_app()
```

The app is also available directly through ShinyApps.io at:
<https://bit.ly/highdir>

------------------------------------------------------------------------

## Supported geometries

| Name | highcharter type | ggplot2 equivalent | Extra args |
|:---|:---|:---|:---|
| `column` | column | `geom_col()` | — |
| `ranked_bar` | column | `geom_col()` | `vs`, `aim`, `char_scale`, `min_frac` |
| `line` | line / spline | `geom_line()` | `smooth`, `dot_size`, `line_symbols` |
| `scatter` | scatter | `geom_point()` | `dot_size` |
| `arearange` | arearange | `geom_ribbon()` | `ymin`, `ymax` |
| `pie` | pie | `geom_bar()`, `coord_polar()` | `inner_size` |
| `stacked_column` | column | `geom_bar()`, `facet_wrap()` | `stack`, `stacking` |

------------------------------------------------------------------------

To see complete list of extra arguments for specify geoms use
`geom_args()` function:

``` r
geom_args("ranked_bar")
geom_args("arearange")
```

## License

MIT © Kamaleri

# inst/app/modules/mod_figure.R
# ── Figure module ─────────────────────────────────────────────────────────────
#
# Responsibilities:
#   • Build hd_spec() and hd_opts() from reactive inputs
#   • Render highcharter figure  (hc_out)
#   • Render ggplot2 figure      (gg_out)
#   • Generate R code preview    (code_preview)
#
# geom_inputs_r is a reactive named list containing ALL optional_args AND
# required_args for the current geometry, assembled in server.R by reading
# the registry.  mod_figure passes this directly to hd_make() via do.call.
# There is no need to know which args belong to which geom here.

# ── UI ────────────────────────────────────────────────────────────────────────

#' @keywords internal
mod_figure_ui <- function(id) {
  ns <- shiny::NS(id)

  shiny::tabsetPanel(
    id   = "out_tabs",
    type = "tabs",

    shiny::tabPanel("Figure",
      shiny::br(),
      shiny::conditionalPanel(
        condition = "input.backend == 'highcharter'",
        highcharter::highchartOutput(ns("hc_out"), height = "520px")
      ),
      shiny::conditionalPanel(
        condition = "input.backend == 'ggplot2'",
        shiny::plotOutput(ns("gg_out"), height = "520px")
      )
    ),

    shiny::tabPanel("Data",
      shiny::br(),
      # "data-tbl_head" = module id "data" + output id "tbl_head"
      DT::DTOutput("data-tbl_head")
    ),

    shiny::tabPanel("R code",
      shiny::br(),
      shiny::verbatimTextOutput(ns("code_preview"))
    )
  )
}

# ── Server ────────────────────────────────────────────────────────────────────

#' @keywords internal
#' @param id            Module id.
#' @param run_r         Reactive integer — input$run click counter.
#' @param data_r        Named list from mod_data_server().
#' @param opts_r        Reactive list of hd_opts() arguments (no use_js).
#' @param use_js_r      Reactive logical — JS hover band (hd_make arg).
#' @param geom_r        Reactive string — selected geometry name.
#' @param backend_r     Reactive string — "highcharter" or "ggplot2".
#' @param geom_inputs_r Reactive named list — ALL optional + required args for
#'                      the current geometry, keyed by arg name as they appear
#'                      in optional_args / required_args in the registry.
#'                      Assembled in server.R from input$ values.
mod_figure_server <- function(id,
                              run_r,
                              data_r,
                              opts_r,
                              use_js_r,
                              geom_r,
                              backend_r,
                              geom_inputs_r) {

  shiny::moduleServer(id, function(input, output, session) {

    # ── hd_spec ───────────────────────────────────────────────────────────────
    the_spec <- shiny::eventReactive(run_r(), {
      shiny::req(data_r$dataset(), data_r$x(), data_r$y())
      hd_spec(
        data  = data_r$dataset(),
        x     = data_r$x(),
        y     = data_r$y(),
        group = if (nzchar(data_r$group() %||% "")) data_r$group() else NULL,
        n     = if (nzchar(data_r$n_col() %||% "")) data_r$n_col() else NULL
      )
    })

    # ── hd_opts ───────────────────────────────────────────────────────────────
    the_opts <- shiny::eventReactive(run_r(), {
      do.call(hd_opts, opts_r())
    })

    # ── Build extra args for hd_make() ──────────────────────────────────────
    # Merges two sources:
    #   • geom_inputs_r() — optional args read from top-level input$ in server.R
    #                        keyed by names(optional_args) from the registry
    #   • data_r$req_args() — required args (e.g. ymin/ymax) read inside the
    #                          data module with the correct "data-" namespace
    # .sanitise() converts empty strings → NULL and NA numerics → NULL so
    # hd_make() never receives a blank textInput value as a real argument.
    .sanitise <- function(args) {
      lapply(args, function(v) {
        if (is.null(v))                     return(NULL)
        if (is.character(v) && !nzchar(v))  return(NULL)
        if (is.numeric(v)   && is.na(v))    return(NULL)
        v
      })
    }

    .build_extra <- function() {
      # optional args from top-level geom option inputs (server.R)
      opt <- .sanitise(geom_inputs_r())
      # required args from the data module (correct namespace)
      req <- .sanitise(data_r$req_args())
      c(opt, req)
    }

    # ── Highcharter figure ────────────────────────────────────────────────────
    # .build_extra() merges optional + required geom args → hd_make() via ...
    # do.call() splices extra as named args, so hd_make receives e.g.:
    #   hd_make(spec, type, opts, backend, use_js, smooth=TRUE, dot_size=4,
    #           ymin="lo_col", ymax="hi_col")
    hc_fig <- shiny::eventReactive(run_r(), {
      shiny::req(backend_r() == "highcharter", the_spec())
      do.call(hd_make, c(
        list(spec    = the_spec(),
             type    = geom_r(),
             opts    = the_opts(),
             backend = "highcharter",
             use_js  = use_js_r()),
        .build_extra()   # ← optional + required geom args → hd_make(...)
      ))
    })

    # ── ggplot2 figure ────────────────────────────────────────────────────────
    # Same flow: .build_extra() → hd_make(...)
    gg_fig <- shiny::eventReactive(run_r(), {
      shiny::req(backend_r() == "ggplot2", the_spec())
      do.call(hd_make, c(
        list(spec    = the_spec(),
             type    = geom_r(),
             opts    = the_opts(),
             backend = "ggplot2"),
        .build_extra()   # ← optional + required geom args → hd_make(...)
      ))
    })

    output$hc_out <- highcharter::renderHighchart(hc_fig())
    output$gg_out <- shiny::renderPlot(gg_fig())

    # ── R code preview ────────────────────────────────────────────────────────
    # Renders live (not only on Draw) so the code box reflects the current UI.
    output$code_preview <- shiny::renderText({
      shiny::req(data_r$x(), data_r$y(), geom_r(), backend_r())

      o  <- opts_r()
      # Merge optional + required geom args for code preview (same as .build_extra())
      gi <- c(.sanitise(geom_inputs_r()), .sanitise(data_r$req_args()))

      # Helper: emit one "  name = value,\n" line, or "" if value is absent
      L <- function(nm, val, quote = TRUE) {
        if (is.null(val) || !nzchar(as.character(val) %||% "")) return("")
        v <- if (quote) paste0('"', val, '"') else as.character(val)
        paste0("  ", nm, " = ", v, ",\n")
      }

      # Build geom-specific extra arg lines from whatever is in gi
      extra_lines <- paste0(
        mapply(function(nm, val) {
          if (is.null(val)) return("")
          if (is.character(val)) L(nm, val)
          else                   L(nm, val, quote = FALSE)
        }, names(gi), gi),
        collapse = ""
      )

      paste0(
        "spec <- hd_spec(\n",
        "  data  = your_data,\n",
        L("x",     data_r$x()),
        L("y",     data_r$y()),
        L("group", data_r$group()),
        L("n",     data_r$n_col()),
        ")\n\n",
        "opts <- hd_opts(\n",
        L("title",    o$title),
        L("subtitle", o$subtitle),
        L("caption",  o$caption),
        print_opt("xlab", o$xlab),
        print_opt("ylab", o$ylab),
        print_val("ylim", o$ylim),
        L("yint", o$yint, quote = FALSE),
        L("ysuffix",  o$ysuffix),
        L("xtick_labels",  o$xtick_labels),
        L("decimals", o$decimals, quote = FALSE),
        L("description",  o$description),
        if (isTRUE(o$flip))
          paste0("  flip  = ", isTRUE(o$flip)),
        ")\n\n",
        "hd_make(\n",
        "  spec    = spec,\n",
        "  opts    = opts,\n",
        L("type",    geom_r()),
        L("backend", backend_r()),
        if (backend_r() == "highcharter")
          paste0("  use_js  = ", isTRUE(use_js_r()), ",\n") else "",
        extra_lines,
        ")"
      )
    })

    # Expose figures so server.R can wire download handlers
    list(hc_fig = hc_fig, gg_fig = gg_fig)
  })
}


print_opt <- function(name, val) {
  if (is.null(val)) {
    paste0("  ", name, " = NULL,\n")
  } else if (is.character(val)) {
    paste0("  ", name, ' = "', val, '",\n')
  }
}

print_val <- function(name, val){
  if (is.null(val)) return("")
  paste0("  ", name, " = ", deparse(val), ",\n")
}

# inst/app/modules/mod_data.R
# ── Data module ───────────────────────────────────────────────────────────────
#
# Responsibilities:
#   • File upload and parsing (via rio)
#   • Column-mapping dropdowns  (x, y, group, n_col)
#   • Required-arg dropdowns    (ymin / ymax for arearange)
#   • Data-preview table        (tbl_head)
#
# ── SPEED PRINCIPLE: updateSelectInput everywhere, renderUI nowhere ────────────
#
# renderUI() rebuilds DOM nodes from scratch on every trigger:
#   serialize HTML → transmit → browser destroys nodes → recreates → rebinds
#   = 300–800 ms perceived lag
#
# updateSelectInput() sends a tiny JSON patch (~200 bytes) that replaces
# only the <option> list in an existing node — no DOM destruction, no
# rebinding, < 30 ms.
#
# Applied consistently here:
#   x / y / group / n_col  — static in UI, updated via updateSelectInput
#   ymin / ymax            — static in UI, shown via conditionalPanel,
#                            updated via updateSelectInput
#

# ── UI ────────────────────────────────────────────────────────────────────────

#' @keywords internal
mod_data_ui <- function(id) {
  ns <- shiny::NS(id)

  shiny::tagList(

    shiny::div(class = "hd-label", "Data"),
    shiny::fileInput(
      ns("file"), NULL,
      accept      = c(".csv", ".xlsx", ".xls", ".rds", ".sav", ".dta", ".json"),
      placeholder = "CSV / XLSX / RDS \u2026",
      buttonLabel = shiny::icon("folder-open")
    ),

    shiny::tags$button(
      class         = "hd-toggle",
      `data-target` = paste0("#", ns("panel-spec")),
      shiny::span("Spec \u2014 variable mapping"),
      shiny::span(class = "hd-arrow", "\u25bc")
    ),

    shiny::div(
      id    = ns("panel-spec"),
      class = "hd-collapse",

      # ── Standard column-mapping inputs (always visible) ───────────────────
      # All declared statically — choices populated by updateSelectInput()
      # in observeEvent(cols()).  Zero lag on upload.
      shiny::selectInput(ns("x"),
                         label   = "X variable",
                         choices = character(0)),

      shiny::selectInput(ns("y"),
                         label   = "Y variable",
                         choices = character(0)),

      shiny::conditionalPanel(
        condition = "input.geom != 'pie'",
        shiny::selectInput(ns("group"),
                           label   = "Group variable",
                           choices = c("(none)" = ""))
      ),

      shiny::selectInput(ns("n_col"),
                         label   = "Count column",
                         choices = c("(none)" = "")),

      # ── Required-arg inputs — static, shown/hidden by conditionalPanel ────
      # arearange: ymin + ymax
      shiny::conditionalPanel(
        condition = "input.geom == 'arearange'",
        shiny::selectInput(ns("ymin"),
                           label   = "Column: ymin",
                           choices = character(0)),
        shiny::selectInput(ns("ymax"),
                           label   = "Column: ymax",
                           choices = character(0))
      )
    )
  )
}

# ── Server ────────────────────────────────────────────────────────────────────

#' @keywords internal
#' @param id     Module id.
#' @param geom_r Reactive string — currently selected geometry name.
mod_data_server <- function(id, geom_r) {

  shiny::moduleServer(id, function(input, output, session) {

    # ── Parse uploaded file ──────────────────────────────────────────────────
    dataset <- shiny::reactive({
      shiny::req(input$file)
      rio::import(input$file$datapath)
    })

    # ── Cache column names ────────────────────────────────────────────────────
    cols <- shiny::reactive({
      shiny::req(dataset())
      names(dataset())
    })

    # ── Populate ALL column dropdowns in one observer ─────────────────────────
    # Fires once when cols() becomes available (upload) and again if a new
    # file is uploaded.  Covers x/y/group/n_col AND the required-arg inputs
    # (ymin/ymax) in a single round-trip — no renderUI involved.
    shiny::observeEvent(cols(), {
      ch <- cols()
      # Standard mapping inputs
      shiny::updateSelectInput(session, "x",
        choices  = ch,
        selected = ch[1])
      shiny::updateSelectInput(session, "y",
        choices  = ch,
        selected = ch[min(2, length(ch))])
      shiny::updateSelectInput(session, "group",
        choices  = c("(none)" = "", ch),
        selected = "")
      shiny::updateSelectInput(session, "n_col",
        choices  = c("(none)" = "", ch),
        selected = "")

      # Required-arg inputs — updated here, shown/hidden by conditionalPanel
      # arearange: ymin defaults to 3rd col if available, ymax to 4th
      shiny::updateSelectInput(session, "ymin",
        choices  = ch,
        selected = ch[min(3, length(ch))])
      shiny::updateSelectInput(session, "ymax",
        choices  = ch,
        selected = ch[min(4, length(ch))])
    }, ignoreNULL = TRUE)

    # ── Data preview ──────────────────────────────────────────────────────────
    output$tbl_head <- DT::renderDT({
      shiny::req(dataset())
      dat <- dataset()
      if (nrow(dat) > 20L) dat <- dat[sample(seq_len(nrow(dat)), 15L), ]
      else                  dat <- utils::head(dat, 10L)
      DT::datatable(dat,
                    options  = list(pageLength = 10L, scrollX = TRUE),
                    rownames = FALSE)
    }, server = TRUE)

    # ── Return values to parent ───────────────────────────────────────────────
    list(
      dataset  = dataset,
      x        = shiny::reactive(input$x),
      y        = shiny::reactive(input$y),
      group    = shiny::reactive(input$group),
      n_col    = shiny::reactive(input$n_col),
      # req_args: named list of required-arg column selections for the current
      # geom.  Read from input$ directly — IDs match names(required_args)
      # because the static inputs above use ns("ymin"), ns("ymax") etc.
      # req_args reads required_args as a NAMED LIST (same structure as
      # optional_args).  names(ra) gives the arg names (e.g. 'ymin', 'ymax');
      # input[[a]] reads the current column selection from the namespaced
      # selectInput (ns('ymin'), ns('ymax')) declared statically above.
      req_args = shiny::reactive({
        ra <- highdir:::.get_geom(geom_r())$required_args
        if (length(ra) == 0L) return(list())
        arg_names <- names(ra)
        stats::setNames(
          lapply(arg_names, function(a) input[[a]]),
          arg_names
        )
      })
    )
  })
}

# inst/app/server.R
# ── highdir Shiny app — Server ────────────────────────────────────────────────
#
# Module call order:
#   1. mod_data_server   — upload + column mapping
#   2. mod_opts_server   — labels + style (returns opts_r, use_js_r)
#   3. mod_figure_server — figures + code preview (returns hc_fig, gg_fig)
#
# Download handlers are registered directly here against top-level output IDs
# that match the static downloadButtons declared in ui.R.  No module
# namespace, no renderUI delay, no reactiveVal bridge needed.
#
# Geom options are built STATICALLY by geom_opts_ui() in global.R at startup
# (one conditionalPanel per geom).  No renderUI — zero server round-trips.

server <- function(input, output, session) {

  # ── 1. Data ────────────────────────────────────────────────────────────────
  data_r <- mod_data_server(
    "data",
    geom_r = shiny::reactive(input$geom)
  )

  # ── 2. Opts ────────────────────────────────────────────────────────────────
  opts_m <- mod_opts_server("opts")

  # ── 3. Geom options — inputs are static (built by geom_opts_ui() in global.R)
  # No renderUI here.  All geom option inputs are in the DOM from page load,
  # wrapped in conditionalPanels keyed on input$geom.  geom_inputs_r() reads
  # them by name exactly as before — the input IDs are identical.

  # ── Collect geom optional-arg inputs → forwarded to hd_make() ────────────
  # Reads ONLY optional_args for the current geometry from top-level input$.
  # Input IDs match because geom_opts_ui() sets inputId = nm (the arg name
  # from names(optional_args)), so input[["smooth"]], input[["dot_size"]] etc. work.
  #
  # Required args (e.g. ymin/ymax for arearange) are NOT collected here.
  # They are rendered inside the data module under namespaced IDs ("data-ymin"),
  # so input[[nm]] at top-level would return NULL.  mod_figure_server() merges
  # required args via data_r$req_args() which reads them in the correct namespace.
  geom_inputs_r <- shiny::reactive({
    oa_names <- names(highdir:::.get_geom(input$geom)$optional_args)
    lapply(stats::setNames(oa_names, oa_names), function(nm) input[[nm]])
  })

  # ── 4. Figure ──────────────────────────────────────────────────────────────
  fig_m <- mod_figure_server(
    id            = "fig",
    run_r         = shiny::reactive(input$run),
    data_r        = data_r,
    opts_r        = opts_m$opts_r,
    use_js_r      = opts_m$use_js_r,
    geom_r        = shiny::reactive(input$geom),
    backend_r     = shiny::reactive(input$backend),
    geom_inputs_r = geom_inputs_r
  )

  # ── 5. Download handlers ────────────────────────────────────────────────────
  # Registered against the top-level IDs that exactly match the static
  # downloadButtons in ui.R.  No module namespace involved — simple and direct.
  dl_basename <- shiny::reactive({
    raw <- trimws(input$dl_filename %||% "")
    if (!nzchar(raw)) return(paste0("highdir-figure_", Sys.Date()))
    tools::file_path_sans_ext(raw)
  })

  .dl <- function(ext, fig_r) {
    shiny::downloadHandler(
      filename = function() paste0(dl_basename(), ".", ext),
      content  = function(file) {
        fig <- fig_r()
        shiny::req(!is.null(fig))
        hd_save(fig, file, type = ext)
      }
    )
  }

  output$dl_json   <- .dl("json", fig_m$hc_fig)
  output$dl_html   <- .dl("html", fig_m$hc_fig)
  output$dl_gg_png <- .dl("png",  fig_m$gg_fig)
  output$dl_gg_svg <- .dl("svg",  fig_m$gg_fig)
}

# inst/app/global.R
# ── Loaded by Shiny before ui.R and server.R ──────────────────────────────────
#
# Shiny's load order is:  global.R → ui.R → server.R
#
# Modules MUST be sourced here, not in server.R.
# ui.R calls mod_data_ui(), mod_opts_ui(), mod_figure_ui() at parse time —
# before server.R is ever read.  Sourcing modules inside server() is too late.

library(highdir)

# ── Source modules ─────────────────────────────────────────────────────────────
# local = TRUE keeps each module's internal helpers out of the global env.
source("modules/mod_data.R",    local = FALSE)
source("modules/mod_opts.R",    local = FALSE)
source("modules/mod_figure.R",  local = FALSE)

# ── Runtime-only app dependencies ─────────────────────────────────────────────
# rio and DT are used exclusively in inst/app/ and are intentionally NOT
# declared in DESCRIPTION (Suggests) to avoid R CMD check warnings about
# packages listed in Suggests but never called from R/*.R.
# They are checked here at app launch time instead.
.has_webshot2 <- requireNamespace("webshot2", quietly = TRUE)

if (!requireNamespace("rio", quietly = TRUE))
  stop(
    "The 'rio' package is required to run the highdir Shiny app.\n",
    "Install it with:  install.packages('rio')",
    call. = FALSE
  )

if (!requireNamespace("DT", quietly = TRUE))
  stop(
    "The 'DT' package is required to run the highdir Shiny app.\n",
    "Install it with:  install.packages('DT')",
    call. = FALSE
  )

# ── geom_opts_ui() ────────────────────────────────────────────────────────────
#
# Called once from ui.R at app startup (before any user interaction).
# Reads optional_args for every registered geometry from the highdir registry
# and returns a tagList of conditionalPanels — one per geom.
#
# Each panel is only visible when input$geom matches its geometry name, making
# the whole thing pure client-side.  No renderUI, no server round-trips.
#
# Widget type dispatch (mirrors the old renderUI logic so input IDs are
# identical — geom_inputs_r() in server.R reads them the same way):
#   logical   → checkboxInput   (inputId = nm)
#   numeric   → numericInput    (inputId = nm)
#   "level"   → selectInput     (special-cased: county / municipality)
#   character → textInput       (inputId = nm, value = default)
#   NULL      → textInput       (inputId = nm, value = "")
#
# The hd-toggle + hd-collapse pattern is identical to mod_opts_ui() and
# mod_data_ui() — same CSS classes, same app.js click handler, no new code.

.geom_widget <- function(nm, entry) {
  def <- entry$default

  # Helper text — short description shown below widget
  desc_short <- entry$desc %||% ""
  if (nchar(desc_short) > 80)
    desc_short <- paste0(substr(desc_short, 1, 80), "\u2026")
  helper <- shiny::tags$p(
    style = "font-size:10px; color:#8b949e; margin:-3px 0 5px;",
    desc_short
  )

  widget <- if (nm == "level") {
    shiny::selectInput(nm, nm,
      choices  = c("County" = "county", "Municipality" = "municipality"),
      selected = def %||% "county")

  } else if (is.logical(def) ||
             identical(def, TRUE) || identical(def, FALSE)) {
    shiny::checkboxInput(nm, nm, value = isTRUE(def))

  } else if (is.numeric(def)) {
    shiny::numericInput(nm, nm, value = def)

  } else {
    ph <- if (!is.null(def)) as.character(def) else nm
    shiny::textInput(nm, nm,
      value       = if (!is.null(def)) as.character(def) else "",
      placeholder = ph)
  }

  shiny::div(widget, helper)
}

geom_opts_ui <- function() {
  geom_names <- highdir::list_geoms()

  panels <- lapply(geom_names, function(g) {
    geom_def <- highdir:::.get_geom(g)
    oa       <- geom_def$optional_args

    # Panel id is stable and unique — used by app.js toggle handler
    panel_id  <- paste0("panel-geom-", g)
    toggle_id <- paste0("toggle-geom-", g)

    panel_body <- if (length(oa) == 0L) {
      shiny::tags$p(
        style = "font-size:11px; color:#8b949e; margin:2px 0 0;",
        "No extra options for this geometry."
      )
    } else {
      shiny::tagList(lapply(names(oa), function(nm) .geom_widget(nm, oa[[nm]])))
    }

    shiny::conditionalPanel(
      condition = paste0("input.geom == '", g, "'"),

      shiny::tags$button(
        id            = toggle_id,
        class         = "hd-toggle",
        `data-target` = paste0("#", panel_id),
        shiny::span(paste("Geom \u2014", g)),
        shiny::span(class = "hd-arrow", "\u25bc")
      ),

      shiny::div(
        id    = panel_id,
        class = "hd-collapse",
        panel_body
      )
    )
  })

  shiny::tagList(panels)
}

# inst/app/modules/mod_opts.R
# ── Opts module ───────────────────────────────────────────────────────────────
#
# Responsibilities:
#   • Labels  (title, subtitle, caption, xlab, ylab)
#   • Style   (colours, HC theme, JS hover toggle)
#
# Downloads have been moved OUT of this module entirely.
# Download buttons are declared statically in ui.R (no renderUI, no module
# namespace) and handlers are registered in server.R.  This eliminates:
#   • The server round-trip that caused the button delay
#   • The namespace mismatch that made buttons grey/inactive
#   • The reactiveVal bridge that was needed to pass figures back here

# ── UI ────────────────────────────────────────────────────────────────────────

#' @keywords internal
mod_opts_ui <- function(id) {
  ns <- shiny::NS(id)

  shiny::tagList(

    # Auto-resize behavior
    shiny::tags$script(sprintf("
      document.addEventListener('DOMContentLoaded', function() {
        const el = document.getElementById('%s');
        if (!el) return;

        const maxHeight = 200;

        function resize() {
          el.style.height = 'auto';
          el.style.height = Math.min(el.scrollHeight, maxHeight) + 'px';
        }

        el.addEventListener('input', resize);
        resize();
      });
    ", ns("description"))),

    
    shiny::tags$button(
      class         = "hd-toggle",
      `data-target` = paste0("#", ns("panel-opts")),
      shiny::span("Opts \u2014 labels & style"),
      shiny::span(class = "hd-arrow", "\u25bc")
    ),

    shiny::div(
      id    = ns("panel-opts"),
      class = "hd-collapse",

      # Use of textInput to be able to describe the functions
      shiny::div(class = "hd-label", style = "margin-top:4px;", "Labels"),
      shiny::textInput(ns("title"),    NULL, placeholder = "Title"),
      shiny::textInput(ns("subtitle"), NULL, placeholder = "Subtitle"),
      shiny::textInput(ns("caption"),  NULL, placeholder = "Caption"),

      shiny::textInput(ns("xlab"),     NULL, placeholder = "X-axis label"),
      shiny::textInput(ns("ylab"),     NULL, placeholder = "Y-axis label"),

      shiny::textInput(ns("ylim"), NULL, placeholder = "Y-axis limits eg. c(10, 80)"),
      shiny::uiOutput(ns("ylim_warn")),

      shiny::textInput(ns("yint"), NULL, placeholder = "Y-axis interval"),
      shiny::uiOutput(ns("yint_warn")),

      shiny::textInput(ns("ysuffix"),  NULL, placeholder = "Y-tick suffix: %, km, mg ..."),
      shiny::textInput(ns("xtick_labels"),  NULL, placeholder = "x-tick labels, if different than x col"),
      shiny::textInput(ns("decimals"), NULL, placeholder = "Decimals points else as.is eg. 2"),
      shiny::textInput(ns("description"), NULL, placeholder = "Figure description for screen readers"),
      

      shiny::div(class = "hd-label", "Style"),
      shiny::textInput(ns("colors"), NULL,
                       placeholder = "Colours: #025169, #7C145C, \u2026"),

      # flip belongs in opts (not geom_inputs) because base_fig reads opts$flip
      # for BOTH backends — ggplot2 via coord_flip(), highcharter via inverted.
      # It is shown for all geoms; for most it has no visible effect.
      shiny::checkboxInput(ns("flip"), "Flip axes", value = FALSE),

      # ggplot2 themes
      shiny::conditionalPanel(
        condition = "input.backend == 'ggplot2'",
        shiny::selectInput(ns("gg_theme"), NULL,
                           choices  = c("classic", "minimal", "bw",
                                        "light", "dark", "void",
                                        "grey"),
                           selected = "classic")
        ),

    # HC-only options — client-side conditionalPanel, no server cost
      shiny::conditionalPanel(
        condition = "input.backend == 'highcharter'",
        shiny::selectInput(ns("hc_theme"), NULL,
                           choices  = c("default", "smpl", "economist",
                                        "darkunica", "gridlight", "bloom",
                                        "flat", "flatdark", "ggplot2"),
                           selected = "default"),
        shiny::checkboxInput(ns("use_js"), "JS hover band", value = TRUE)
      )
    )
  )
}

# ── Server ────────────────────────────────────────────────────────────────────

#' @keywords internal
mod_opts_server <- function(id) {

  shiny::moduleServer(id, function(input, output, session) {

    ## -------- live validation messages --------
    output$ylim_warn <- shiny::renderUI({
      x <- input$ylim %||% ""
      if (!nzchar(x)) return(NULL)

      if (!is_valid_ylim(x)) {
        shiny::div(
          class = "text-danger small",
          "Invalid format. Use: c(min, max), e.g. c(10, 80)"
        )
      }
    })

    output$yint_warn <- shiny::renderUI({
      x <- input$yint %||% ""
      if (!nzchar(x)) return(NULL)

      if (!is_valid_yint(x)) {
        shiny::div(
          class = "text-danger small",
          "Must be a numeric value (e.g. 5 or 10)"
        )
      }
    })

    # --- colors
    parsed_colors <- shiny::reactive({
      raw <- trimws(input$colors %||% "")
      if (!nzchar(raw)) return(NULL)
      strsplit(raw, "\\s*,\\s*")[[1]]
    })

    list(
      opts_r = shiny::reactive(list(
        title    = parse_input(input$title),
        subtitle = parse_input(input$subtitle),
        caption  = parse_input(input$caption),

        xlab = input_labs(input$xlab),
        ylab = input_labs(input$ylab),

        ylim = parse_ylim(input$ylim),
        yint = parse_yint(input$yint, default = 10),

        ysuffix = parse_input(input$ysuffix),
        xtick_labels = parse_input(input$xtick_labels),

        decimals = parse_input(
          input$decimals,
          coerce  = coerce_numeric,
          default = NULL
        ),

        description = parse_input(input$description),

        colors   = parsed_colors(),
        hc_theme = input$hc_theme %||% NULL,
        gg_theme = input$gg_theme %||% NULL,
        flip     = isTRUE(input$flip)
      )),
      use_js_r = shiny::reactive(isTRUE(input$use_js))
    )
  })
}

# Helper -----------------------------------------------------------------------

parse_input <- function(
  x,
  coerce       = identity,
  default      = NULL,
  empty_is     = NULL,
  allow_null   = TRUE,
  sentinel     = NULL
) {
  x <- x %||% ""

  # Hide value explicitly requested
  if (allow_null && grepl("^\\s*NULL\\s*$", x, ignore.case = TRUE))
    return(NULL)

  # Empty input
  if (!nzchar(x))
    return(empty_is)

  # Sentinel (e.g. " ")
  if (!is.null(sentinel) && identical(x, sentinel))
    return(sentinel)

  # Try coercion
  out <- tryCatch(coerce(x), error = function(e) default)

  if (is.null(out) || (is.atomic(out) && anyNA(out)))
    default
  else
    out
}

# logic
coerce_label <- function(x) x

coerce_numeric <- function(x)
  suppressWarnings(as.numeric(x))

coerce_ylim <- function(x) {
  x <- gsub("\\s+", "", x)
  if (!grepl("^c\\(-?[0-9.]+,-?[0-9.]+\\)$", x))
    stop("Invalid ylim")

  y <- eval(parse(text = x))
  if (!is.numeric(y) || length(y) != 2)
    stop("ylim must be length 2")

  y
}

# Validator
is_valid_ylim <- function(x) {
  !is.null(parse_ylim(x))
}

is_valid_yint <- function(x) {
  val <- suppressWarnings(as.numeric(x))
  nzchar(x) && !is.na(val)
}


# Functions
input_labs <- function(x) {
  parse_input(
    x,
    coerce     = coerce_label,
    empty_is   = " ",
    sentinel   = " ",
    allow_null = TRUE
  )
}

parse_ylim <- function(x) {
  parse_input(
    x,
    coerce     = coerce_ylim,
    empty_is   = NULL,
    allow_null = TRUE
  )
}

parse_yint <- function(x, default = 10) {
  parse_input(
    x,
    coerce     = coerce_numeric,
    default    = default,
    empty_is   = default,
    allow_null = FALSE
  )
}

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

library(highdir)

## ----setup, eval = FALSE------------------------------------------------------
# 
# library(highdir)
# 
# # Create dataset
# df <- data.frame(
#   age  = rep(c("18-24", "25-34", "35-44", "45-54"), each = 2),
#   sex  = rep(c("Male", "Female"), 4),
#   pct  = c(42, 38, 55, 61, 48, 52, 60, 57),
#   n    = c(120, 115, 200, 210, 180, 175, 160, 155)
# )
# 
# # Figure data specification
# spec <- hd_spec(df, x = "age", y = "pct", group = "sex", n = "n")
# 
# # Specify figure presentation
# opts <- hd_opts(title = "Tall om Report",
#                 subtitle = "Source: Example data",
#                 caption  = "Tall om helse",
#                 ylim = c(0, 80))
# 
# hd_make(spec, "column", opts)                       # highcharter (default)
# hd_make(spec, "column", opts, backend = "ggplot2")  # static ggplot2
# hd_make(spec, "line",   opts, smooth = TRUE)        # smooth spline
# hd_make(spec, "pie",    opts)                       # pie / donut
# 
# # Disable JavaScript (for static HTML export)
# hd_make(spec, "column", opts, use_js = FALSE)
# 
# # Save
# hd01 <- hd_make(spec, "column")
# hd_save(fig = hd01, file = "chart.html")
# 
# gg01 <- hd_make(spec, "column", backend = "ggplot2")
# hd_save(fig = gg01, file = "chart.png")

## ----gg, eval = FALSE---------------------------------------------------------
# 
# # Interactive
# hd(df, x = "age", y = "pct", group = "sex", n = "n") +
#   hd_geom_column() +
#   hd_opts(title = "Tall om Report",
#           subtitle = "Source: Example data",
#           caption  = "Tall om helse",
#           ylim = c(0, 80))
# 
# # Static ggplot2
# hd(df, x = "age", y = "pct", group = "sex", n = "n", backend = "ggplot2") +
#   hd_geom_column() +
#   hd_opts(title = "Tall om Report",
#           subtitle = "Source: Example data",
#           caption  = "Tall om helse",
#           ylim = c(0, 80))
# 

## ----geom---------------------------------------------------------------------
list_geoms()

# inst/app/ui.R
# ── highdir Shiny app — UI ────────────────────────────────────────────────────
#
# Sidebar zones (top → bottom):
#   1. Title + logo             always visible
#   2. Geometry + Backend       always visible
#   3. [Spec ▾] collapsible     file upload + variable mapping
#   4. [Opts ▾] collapsible     labels & style
#   5. Geom options             renderUI — driven by input$geom (server-side)
#   6. Draw button              always visible
#   7. Download buttons         always visible (static, conditionalPanel)
#
# Download buttons are STATIC in the UI (not inside renderUI) so they appear
# immediately and their IDs are unambiguous — no module namespace issues.
# Handlers are registered directly in server.R.

# ── Head assets ───────────────────────────────────────────────────────────────

.head_assets <- shiny::tags$head(
  shiny::tags$meta(charset = "utf-8"),
  shiny::tags$link(rel = "preconnect", href = "https://fonts.googleapis.com"),
  shiny::tags$link(
    rel  = "stylesheet",
    href = "https://fonts.googleapis.com/css2?family=IBM+Plex+Mono:wght@400;600&family=IBM+Plex+Sans:wght@300;400;500;600&display=swap"
  ),
  shiny::tags$link(rel = "stylesheet", href = "styles.css"),
  shiny::tags$script(defer = NA, src = "app.js")
)

# ── Sidebar ───────────────────────────────────────────────────────────────────

.sidebar <- shiny::div(

  # Title + logo
  shiny::div(
    style = "display:flex; align-items:center; justify-content:space-between;",
    shiny::div(
      shiny::div(class = "hd-title",   "highdir"),
      shiny::div(class = "hd-tagline", "Create Figure")
    ),
    shiny::img(src = "logo.png", height = "40px",
               style = "margin-left:6px;", id = "app_logo")
  ),

  # Geometry + Backend
  shiny::div(class = "hd-label", "Figure"),
  shiny::selectInput("geom",    NULL, choices = list_geoms(),    selected = "column"),
  shiny::selectInput("backend", NULL, choices = list_backends(), selected = "highcharter"),

  # Spec collapsible — file upload + column mapping
  mod_data_ui("data"),

  # Opts collapsible — labels, style
  mod_opts_ui("opts"),

  # ── Geom options (static, generated from registry at startup) ────────────
  # geom_opts_ui() is defined in global.R.  It reads optional_args from the
  # registry once at app startup and produces one hd-toggle collapsible panel
  # per geometry, each wrapped in a conditionalPanel so only the panel for
  # the currently selected geom is visible.  Pure client-side — zero server
  # round-trips, no renderUI lag, no input flicker on geom switch.
  geom_opts_ui(),

  # ── Draw button ───────────────────────────────────────────────────────────
  shiny::actionButton("run", "Draw figure",
                      icon  = shiny::icon("palette"),
                      class = "btn-primary"),

  # ── Download buttons — STATIC, always in DOM ──────────────────────────────
  # Placing these here (not inside renderUI or a module) means:
  #   • They appear immediately — no server round-trip, no delay
  #   • Their IDs ("dl_json", "dl_html", etc.) are top-level — handlers
  #     registered as output$dl_json in server.R match without namespace issues
  #   • conditionalPanel hides/shows client-side — zero server cost
  #
  # Buttons are disabled via CSS class "disabled" until a figure exists;
  # server.R removes the class via shinyjs or we rely on downloadHandler's
  # own guard (req()) — the button is always clickable but produces nothing
  # until a figure has been drawn.
  shiny::div(class = "hd-label", "Download"),
  shiny::textInput("dl_filename", NULL,
                   placeholder = paste0("highdir-figure_", Sys.Date())),
  shiny::div(
    class = "hd-dl-row",
    # HC buttons — shown when backend == highcharter
    shiny::conditionalPanel(
      condition = "input.backend == 'highcharter'",
      shiny::downloadButton("dl_json", "JSON"),
      shiny::downloadButton("dl_html", "HTML")
    ),
    # ggplot2 buttons — shown when backend == ggplot2
    shiny::conditionalPanel(
      condition = "input.backend == 'ggplot2'",
      shiny::downloadButton("dl_gg_png", "PNG"),
      shiny::downloadButton("dl_gg_svg", "SVG")
    )
  )
)

# ── Root UI ───────────────────────────────────────────────────────────────────

ui <- shiny::fluidPage(
  .head_assets,
  shiny::sidebarLayout(
    shiny::sidebarPanel(.sidebar,          width = 3),
    shiny::mainPanel(mod_figure_ui("fig"), width = 9)
  )
)

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----args, eval = FALSE-------------------------------------------------------
# geom_args("line") #args in geom line
# geom_args("arearange")
# 
# # or
# highdir:::.get_geom("ranked_bar")$optional_args
# highdir:::.get_geom("arearange")$required_args

## ----eval = FALSE-------------------------------------------------------------
# # Set package-wide defaults for the session
# hd_set_theme(
#   hc_theme = "helsedirektoratet",
#   colors   = c("#025169", "#7C145C", "#C68803"),
#   font     = "Source Sans Pro"
# )
# 
# # All subsequent hd_make() calls use these settings automatically
# hd_make(spec, "column")

## ----eval = FALSE-------------------------------------------------------------
# fig <- hd_make(spec, "column")
# 
# # Inline JS
# fig <- hd_add_js(fig, code = "console.log('chart loaded');")
# 
# # From a .js file
# fig <- hd_add_js(fig, file = "path/to/my-plugin.js")
# 
# # From a bundled plugin (inst/js/<name>.js)
# fig <- hd_add_js(fig, plugin = "my-plugin")

## ----setup, include = FALSE---------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment  = "#>",
  eval     = FALSE   # set TRUE for local preview
)

## ----caption-vs-description---------------------------------------------------
# library(highdir)
# 
# opts <- hd_opts(
#   title       = "Alcohol consumption over time",
#   subtitle    = "Source: Norwegian Directorate of Health",
# 
#   # Visible below the chart — for citations and footnotes
#   caption     = "Data: Hdir annual report 2023. Adjusted for population age.",
# 
#   # Read by screen readers only — describes what the chart shows
#   description = paste(
#     "Line chart showing adjusted mean alcohol consumption in Norway from",
#     "2010 to 2023. Consumption peaked at 7.3 litres per capita in 2014 and",
#     "declined steadily to 5.3 litres by 2023."
#   )
# )

## ----data, eval = TRUE--------------------------------------------------------
library(highdir)

# Single series — annual adjusted mean with 95% CI
alcohol <- highdir::alco1   # columns: year, adj_mean, lower_95CI, upper_95CI

# Grouped by sex
alcohol2 <- highdir::alco2  # same columns + kjonn

## -----------------------------------------------------------------------------
# highdir:::.hd_accessibility(chart, description = opts$description)
# 

## ----hc-line-description, eval = TRUE-----------------------------------------
spec_line <- hd_spec(alcohol,
                     x    = "year",
                     y    = "adj_mean")

opts_line <- hd_opts(
  title       = "Alcohol consumption in Norway",
  subtitle    = "Source: Norwegian Directorate of Health",
  ylab        = "Litres per capita",
  ylim        = c(0, 40),
  description = paste(
    "Line chart showing adjusted mean alcohol consumption per capita in",
    "Norway from 2010 to 2023.",
    "Consumption peaked at approximately 7.3 litres in 2014 and declined",
    "to 5.3 litres by 2023, a reduction of 27 percent."
  )
)

# The description is embedded in the widget — invisible in the browser
# but announced by screen readers
# ------------------------------------------------------------------------------
# Layered approach
hd(spec_line) +
  hd_geom_line() +
  opts_line

# Declarative approach
hd_make(spec_line, "line", opts_line)

## ----hc-line-grouped, eval = TRUE---------------------------------------------
spec_line2 <- hd_spec(alcohol2,
                      x     = "year",
                      y     = "adj_mean",
                      group = "kjonn")

opts_line2 <- hd_opts(
  title       = "Alcohol consumption by sex",
  subtitle    = "Source: Norwegian Directorate of Health",
  ylab        = "Litres per capita",
  ylim        = c(0, 50),
  description = paste(
    "Line chart comparing adjusted mean alcohol consumption per capita",
    "between men and women in Norway, 2010 to 2023.",
    "Men consistently drink more than women throughout the period.",
    "Both groups show a declining trend from 2014 onwards."
  )
)

hd_make(spec_line2, "line", opts_line2)

## ----hc-arearange, eval=TRUE--------------------------------------------------
opts_ar <- hd_opts(
  title       = "Alcohol consumption with 95% confidence interval",
  subtitle    = "Source: Norwegian Directorate of Health",
  ylab        = "Litres per capita",
  ylim        = c(0, 40),
  description = paste(
    "Area range chart showing adjusted mean alcohol consumption in Norway",
    "with 95 percent confidence intervals, 2010 to 2023.",
    "The shaded band represents uncertainty around the estimate.",
    "The interval is narrowest in the most recent years."
  )
)

# Layered approach
hd(spec_line) +
  hd_geom_arearange(ymin = "lower_95CI", ymax = "upper_95CI") +
  opts_ar

# Declarative approach
hd_make(spec_line, "arearange", opts_ar,
        ymin = "lower_95CI", ymax = "upper_95CI")

## ----save-html----------------------------------------------------------------
# hc_fig <- hd_make(spec_line, "line", opts_line)
# 
# # Description is embedded — the saved HTML file is fully accessible
# hd_save(hc_fig, "alcohol_line.html")
# 
# # selfcontained = TRUE (the default) embeds all JS/CSS so the file works
# # without an internet connection — recommended for sharing
# hd_save(hc_fig, "alcohol_line.html", selfcontained = TRUE)

## ----save-json----------------------------------------------------------------
# # The JSON file contains:
# # {
# #   "title": { "text": "Alcohol consumption in Norway" },
# #   "accessibility": {
# #     "description": "Line chart showing adjusted mean alcohol..."
# #   },
# #   ...
# # }
# hd_save(hc_fig, "alcohol_line.json")

## ----gg-line-description------------------------------------------------------
# spec_line <- hd_spec(alcohol,
#                      x    = "year",
#                      y    = "adj_mean",
#                      ylab = "Litres per capita")
# 
# opts_gg <- hd_opts(
#   title       = "Alcohol consumption in Norway",
#   subtitle    = "Source: Norwegian Directorate of Health",
#   ylab        = "Litres per capita",
#   ylim        = c(0, 10),
#   description = paste(
#     "Line chart showing adjusted mean alcohol consumption per capita in",
#     "Norway from 2010 to 2023.",
#     "Consumption peaked at approximately 7.3 litres in 2014 and has",
#     "declined steadily since."
#   )
# )
# 
# gg_fig <- hd_make(spec_line, "line", opts_gg, backend = "ggplot2")
# 
# # SVG preserves the description as a <desc> element — recommended
# hd_save(gg_fig, "alcohol_line.svg")
# 
# # PNG does not carry alt text — only use for contexts where accessibility
# # is not required or where the alt text is set separately in the HTML
# hd_save(gg_fig, "alcohol_line.png")

## ----alcohol-line, fig.alt = "Line chart showing alcohol consumption declining from 7.3 to 5.3 litres per capita between 2014 and 2023."----
# hd_make(spec_line, "line", opts_gg, backend = "ggplot2")

## ----alcohol-hc, fig.alt = "Interactive line chart of alcohol consumption."----
# hd_make(spec_line, "line", opts_line)

## ----good-description---------------------------------------------------------
# # Too vague — does not help a non-sighted user understand the finding
# opts_vague <- hd_opts(
#   title       = "Alcohol consumption",
#   description = "A chart about alcohol."   # unhelpful
# )
# 
# # Too long — screen readers become tedious to listen to
# opts_too_long <- hd_opts(
#   title       = "Alcohol consumption",
#   description = paste(
#     "This chart shows data from the Norwegian Directorate of Health about",
#     "alcohol consumption measured in litres per capita per year adjusted for",
#     "the age distribution of the population from the year 2010 through 2023",
#     "for the total population of Norway showing an increase from 6.2 litres",
#     "in 2010 to a peak of 7.3 litres in 2014 followed by a sustained",
#     "reduction to 5.3 litres in the year 2023."   # too long
#   )
# )
# 
# # Well-written — chart type, subject, key finding, specific values
# opts_good <- hd_opts(
#   title       = "Alcohol consumption in Norway",
#   subtitle    = "Source: Norwegian Directorate of Health",
#   ylab        = "Litres per capita",
#   description = paste(
#     "Line chart of adjusted per capita alcohol consumption in Norway,",
#     "2010 to 2023. Consumption peaked at 7.3 litres in 2014 and fell to",
#     "5.3 litres by 2023."
#   )
# )
# 
# hd_make(spec_line, "line", opts_good)
# hd_make(spec_line, "line", opts_good, backend = "ggplot2")

## ----grouped-description------------------------------------------------------
# opts_grouped <- hd_opts(
#   title       = "Alcohol consumption by sex",
#   subtitle    = "Source: Norwegian Directorate of Health",
#   ylab        = "Litres per capita",
#   description = paste(
#     "Line chart comparing alcohol consumption between men and women in",
#     "Norway, 2010 to 2023. Men consume approximately twice as much as women",
#     "throughout the period. Both groups show a declining trend after 2014."
#   )
# )
# 
# hd_make(spec_line2, "line", opts_grouped)
# hd_make(spec_line2, "line", opts_grouped, backend = "ggplot2")

#
# Engine contract:
#   function(spec, geom, opts, geom_params, use_js, filename, ...)
#
# `geom_params` is a named list built by hd_make() that carries *all*
# geom-specific arguments (smooth, dot_size, line_symbols, ymin, ymax, ...).
# Passing them as an explicit list instead of bare `...` means:
#   1. The engine signature is stable regardless of how many geoms exist.
#   2. Nothing unexpected leaks into hc_add_series() causing tibble errors.

# -- highcharter engine --------------------------------------------------------

#' @keywords internal
highcharter_engine <- function(spec, geom, opts, geom_params,
                               use_js = TRUE, module = FALSE, ...) {

  chart <- base_fig(spec, opts, "highcharter")
  
  # -- Tooltip -----------------------------------------------------------------
  ysuffix <- opts$ysuffix %||% ""
  has_suffix <- nzchar(ysuffix)

  series_header <- paste0(
    '<span style="color:{series.color}">\u25CF</span> ',
    '<span style="color:black">{series.name}</span>: '
  )

  y_fmt <- if (has_suffix) paste0('{point.y}', ysuffix) else '{point.y}'

  ysuffix_fmt <- paste0(series_header, '<b>', y_fmt, '</b><br/>')

  tools_fmt <- if (is.null(spec$n) || !nzchar(spec$n)) {
    paste0('<b>', y_fmt, '</b><br/>')
  } else {
    paste0('<b>{point.', spec$n, '} (', y_fmt, ')</b><br/>')
  }

  point_fmt <- if (is.null(spec$n) || !nzchar(spec$n)) ysuffix_fmt else paste0(series_header, tools_fmt)

  # -- Chart -------------------------------------------------------------------
  chart <- chart |>
    highcharter::hc_tooltip(
      useHTML      = TRUE,
      shared       = TRUE,
      headerFormat =
        '<span style="font-size:14px;font-weight:bold;">{point.key}</span><br/>',
      pointFormat  = point_fmt
    ) |>
    highcharter::hc_credits(
      enabled = TRUE,
      text    = "Helsedirektoratet",
      href    = "https://www.helsedirektoratet.no/"
    ) |>
    highcharter::hc_legend(
      align         = "left",
      verticalAlign = "bottom",
      layout        = "horizontal",
      x             = 50, y = 0
    ) |>
    highcharter::hc_exporting(
      enabled       = TRUE,
      filename      = "highdir-figure",
      accessibility = list(enabled = TRUE)
    )

  # When to use FALSE:
  #   - embedding in apps where modules are loaded globally already via hc_function
  #   - unit tests where CDN access is unavailable
  #   - performance-critical contexts with many widgets on one page
  if (isTRUE(module)) {
    chart <- .hd_add_dep(chart, "plugins/accessibility.js")
  }

  # -- Series (geom renders here) ----------------------------------------------
  chart <- geom$highcharter_fun(chart, spec, opts, geom_params,
                                use_js = use_js, ...)

  # -- Theme (per-figure opts$hc_theme overrides session default) --------------
  # hd_theme() resolves name, colors, and font in one call -- same priority
  # chain as gg_theme() in ggplot_engine():
  #   explicit opts > getOption("highdir.*") > package default
  # Font is read from getOption("highdir.font") inside hd_theme() itself.
  chart <- chart |>
    highcharter::hc_add_theme(
      hd_theme(name   = opts$hc_theme,
               colors = opts$colors)
    )

  # -- Session-level JS plugins ------------------------------------------------
  for (plugin in getOption("highdir.js_plugins", default = character(0)))
    chart <- hd_add_js(chart, plugin = plugin)

  chart
}

#
# -- JS hover-band helpers -----------------------------------------------------

#' Hover-band point events (or NULL)
#'
#' When `use_js = TRUE` returns a Highcharts `point.events` list that draws a
#' translucent highlight band behind the hovered category.  When `FALSE`
#' returns `NULL` so the key is omitted entirely from the serialised config -
#' an empty `list()` would break shared tooltips.
#'
#' @param use_js    Logical.
#' @param band_color CSS colour string for the hover band.
#' @param half_width Half-width of the band in category units.
#' @keywords internal
point_events_or_null <- function(use_js,
                                 band_color = "rgba(204, 211, 255, 0.25)",
                                 half_width = 0.4) {
  if (!isTRUE(use_js)) return(NULL)
  list(events = list(
    mouseOver = htmlwidgets::JS(sprintf(
      "function(){
         var c=this.series.chart, i=this.x;
         c.xAxis[0].removePlotBand('hb');
         c.xAxis[0].addPlotBand({id:'hb',from:i-%s,to:i+%s,
           color:'%s',zIndex:0});
       }", half_width, half_width, band_color)),
    mouseOut = htmlwidgets::JS(
      "function(){ this.series.chart.xAxis[0].removePlotBand('hb'); }")
  ))
}

# -- Shared x-mapping (category index vs numeric) ------------------------------
## Highcharts draws axes in two fundamentally different ways:
## Categorical axis (e.g. "18-24", "Oslo", "Male") - Highcharts expects 0-based integer positions: 0, 1, 2, 3… and a separate categories array for the labels
## Numeric axis (e.g. 1990, 1995, 2000) - Highcharts reads the raw numbers directly as x-positions
##
## rlang::sym() converts it to a symbol and !! unquotes it so hcaes() sees y = pct rather than y = "pct"
##
## Example:
## xmap <- .hc_x_map(spec)
##
## hc_add_series(
##   data    = xmap$data[grp$rows, ],   # the (possibly modified) data frame
##   mapping = xmap$mapping             # the hcaes() with correct x reference
## )

#' @keywords internal
.hc_x_map <- function(spec) {
  if (!is.numeric(spec$data[[spec$x]])) {
    lvls          <- unique(spec$data[[spec$x]]) # Get unique group values in order of appearance i.e keep position as it's
    spec$data$x_index <- match(spec$data[[spec$x]], lvls) - 1L #provide base position to 0
    list(data    = spec$data,
         mapping = highcharter::hcaes(x = x_index,
                                      y = !!rlang::sym(spec$y)))
  } else {
    list(data    = spec$data,
         mapping = highcharter::hcaes(x = !!rlang::sym(spec$x),
                                      y = !!rlang::sym(spec$y)))
  }
}

# -- Group-splitting helper ----------------------------------------------------
## Draws each group when exists as a separate series
## Example:
## groups  <- .group_split(spec)
## palette <- resolve_colors(length(groups), opts$colors)
##
## for (i in seq_along(groups)) {
##   grp   <- groups[[i]]          # one descriptor
##   chart <- hc_add_series(
##     data  = data[grp$rows, ],   # only this group's rows
##     name  = grp$name,           # legend / tooltip label
##     color = palette[i]          # consistent colour per group
##   )
## }

#' @keywords internal
.group_split <- function(spec) {
  if (!is.null(spec$group)) {
    lvls <- unique(spec$data[[spec$group]]) # Get unique group values in order of appearance
    lapply(lvls, function(l)
      list(name = as.character(l),
           rows = spec$data[[spec$group]] == l))
  } else {
    list(list(name = spec$y, rows = rep(TRUE, nrow(spec$data)))) #Loop don't iterate with inner list
  }
}

# SCATTER
# ------------------------------------------------------------------------------
# Each geometry is a pair:
#   gg_<name>  -> returns a ggplot2 layer (or list of layers)
#   hc_<name>  -> adds series to a highchart object, returns the updated chart
#
# Calling convention (enforced by the registry):
#   gg_*:  function(spec, opts, geom_params, ...)
#   hc_*:  function(chart, spec, opts, geom_params, use_js, ...)
#
# geom_params is a named list carrying all geom-specific args so that the
# engine signature stays stable as new geoms are added.  Nothing leaks into
# hc_add_series() via bare `...`.

#' @keywords internal
gg_scatter <- function(spec, opts, geom_params) {
  sc   <- geom_params$single_colour
  size <- geom_params$dot_size %||% 4L

  if (!is.null(sc)) {
    list(ggplot2::geom_point(colour = sc, fill = sc,
                              size = size / 3, shape = 21))
  } else {
    list(ggplot2::geom_point(size = size / 3, shape = 21))
  }
}

#' @keywords internal
hc_scatter <- function(chart, spec, opts, geom_params, use_js = TRUE, ...) {
  groups   <- .group_split(spec)
  palette  <- resolve_colors(length(groups), opts$colors)
  point_ev <- point_events_or_null(use_js)
  mapping  <- highcharter::hcaes(x = !!rlang::sym(spec$x),
                                   y = !!rlang::sym(spec$y))

  for (i in seq_along(groups)) {
    grp  <- groups[[i]]
    args <- list(
      chart,
      data  = spec$data[grp$rows, ],
      type  = "scatter",
      name  = grp$name,
      mapping,
      color = palette[i]
    )
    if (!is.null(point_ev)) args$point <- point_ev
    chart <- do.call(highcharter::hc_add_series, args)
  }
  chart
}

# ------------------------------------------------------------------------------
# Public constructor for scatter geometry layer
# ------------------------------------------------------------------------------
#
#' Scatter Geometry Layer for hd Objects
#'
#' `hd_geom_scatter()` creates a scatter geometry layer that is added to an
#' [hd()] object via `+`. Use [geom_args()] to discover available arguments per
#' geometry, e.g. `geom_args("scatter")` lists `dot_size`.
#'
#' @param dot_size Numeric. Size of the points in the scatter plot. Default is 4.
#' @param ... Geometry-specific arguments forwarded to [hd_make()].
#' ' @return An S3 object of class `"hd_geom"` for use with `+.hd`.
#'
#' @examples
#' # Basic scatter plot - layered API
#' hd(mtcars, x = "wt", y = "mpg", backend = "ggplot2") +
#'  hd_geom_scatter() +
#'  hd_opts(title = "Scatter Plot of mtcars")
#'
#' # Basic scatter plot - declarative API
#' car <- hd_spec(mtcars, x = "wt", y = "mpg")
#' opt <- hd_opts(title = "Scatter Plot of mtcars")
#' hd_make(car, type = "scatter")
#'
#' @return An S3 object of class `"hd_geom"` for use with `+.hd`.
#' @export
hd_geom_scatter <- function(dot_size = 4, ...) {
  hd_geom("scatter", dot_size = dot_size, ...)
}

# COLUMN
# ------------------------------------------------------------------------------
# Each geometry is a pair:
#   gg_<name>  -> returns a ggplot2 layer (or list of layers)
#   hc_<name>  -> adds series to a highchart object, returns the updated chart
#
# Calling convention (enforced by the registry):
#   gg_*:  function(spec, opts, geom_params, ...)
#   hc_*:  function(chart, spec, opts, geom_params, use_js, ...)
#
# geom_params is a named list carrying all geom-specific args so that the
# engine signature stays stable as new geoms are added.  Nothing leaks into
# hc_add_series() via bare `...`.

#' @keywords internal
gg_column <- function(spec, opts, geom_params) {
  sc <- geom_params$single_colour

  list(
    if (!is.null(sc)) {
      # Single series: inject brand colour as fixed aesthetic
      ggplot2::geom_col(
        position = "dodge",
        fill = sc,
        colour = sc
      )
    } else {
      # Multi-series: no fixed colour - inherits from mapped aesthetic
      ggplot2::geom_col(position = "dodge")
    }
  )
}


#' @keywords internal
hc_column <- function(chart, spec, opts, geom_params, use_js = TRUE, ...) {
  groups   <- .group_split(spec)
  palette  <- resolve_colors(length(groups), opts$colors)
  point_ev <- point_events_or_null(use_js)
  xmap     <- .hc_x_map(spec)

  for (i in seq_along(groups)) {
    grp  <- groups[[i]]
    args <- list(
      chart,
      data   = xmap$data[grp$rows, ],
      type   = "column",
      name   = grp$name,
      xmap$mapping,
      color  = palette[i],
      states = list(hover = list(brightness = 0.2))
    )
    if (!is.null(point_ev)) args$point <- point_ev
    chart <- do.call(highcharter::hc_add_series, args)
  }
  chart
}

## -----------------------------------------------------------------------------
## Public constructor for column geometry layer.  See ?hd for usage.
## -----------------------------------------------------------------------------

#' Column Geometry Layer for hd Objects
#'
#' `hd_geom_column()` creates a column geometry layer that is added to an [hd()]
#' object via `+`.  The layer records the geometry type and any geometry-specific
#' arguments; rendering only happens when the `hd` object is printed.
#'
#' @param ... Geometry-specific arguments forwarded to [hd_make()].
#'
#' @return An S3 object of class `"hd_geom"` for use with `+.hd`.
#' @examples
#' 
#' survey <- data.frame(
#'   age_group = rep(c("18-24", "25-34", "35-44", "45-54", "55-64"), each = 2),
#'   kjonn       = rep(c("Male", "Female"), times = 5),
#'   pct       = c(42, 38, 55, 61, 48, 52, 60, 57, 65, 70),
#'   n         = c(120, 115, 200, 210, 180, 175, 160, 155, 140, 145)
#' )
#'
#' spec_col <- hd_spec(survey,
#'                     x     = "age_group",
#'                     y     = "pct",
#'                     group = "kjonn",
#'                     n     = "n")
#'
#' opts_col <- hd_opts(
#'   title    = "Alcohol use by age group and kjonn",
#'   subtitle = "Source: Norwegian Directorate of Health",
#'   ylim     = c(0, 100),
#'   yint     = 20,
#'   ylab     = "Percentage (%)"
#' )
#'
#' # Interactive (default)
#' hd_make(spec_col, "column", opts_col)
#'
#' # Static ggplot2
#' hd_make(spec_col, "column", opts_col, backend = "ggplot2")
#'
#' # Composable style
#' p <- hd(survey, x = "age_group", y = "pct", group = "kjonn")
#' p2 <- p + hd_geom_column()
#'
#' # More options
#' p2 + hd_opts(title = "Health survey", ylim = c(0, 100))
#'
#' # Pass an existing hd_spec
#' spec <- hd_spec(survey, x = "age_group", y = "pct", group = "kjonn", n = "n")
#'
#' hd(spec, backend = "ggplot2") +
#'  hd_geom_column() +
#'  hd_opts(title = "Health survey", ylim = c(0, 80))
#' 
#' @export
hd_geom_column <- function(...) {
  hd_geom("column", ...)
}

# -- Package-level option defaults ---------------------------------------------

# These are the default values for options used in the package. They will be set
# as falls back if the user has not set them in their R session. See .onLoad()
# in R/zzz.R for where these are set.

#' @keywords internal
.hd_defaults <- list(
  highdir.hc_theme   = "default",
  highdir.gg_theme   = "classic",
  highdir.colors     = NULL,
  highdir.font       = NULL,
  highdir.js_plugins = character(0),
  highdir.backend    = "highcharter"   # default engine for hd()
)

#' Standard health-survey data frame used across tests
survey_df <- data.frame(
  age = rep(c("18-24", "25-34", "35-44", "45-54"), each = 2),
  sex = rep(c("Male", "Female"), 4),
  pct = c(42, 38, 55, 61, 48, 52, 60, 57),
  lo  = c(37, 33, 50, 56, 43, 47, 55, 52),
  hi  = c(47, 43, 60, 66, 53, 57, 65, 62),
  n   = c(120, 115, 200, 210, 180, 175, 160, 155)
)

#' Standard pie data frame
pie_df <- data.frame(
  category = c("Aldri", "Sjelden", "Av og til", "Ofte"),
  value    = c(35, 25, 25, 15)
)

#' Use in colors and palettes testing
df2 <- data.frame(
  x   = rep(c("A","B","C"), 2),
  y   = c(1,2,3,4,5,6),
  grp = rep(c("Male","Female"), each = 3)
)

spec2 <- hd_spec(df2, "x", "y", group = "grp")

# Helper used in tests for colors and paletts for ggplot2
.extract_gg_fill_values <- function(fig) {
  idx <- which(vapply(fig$scales$scales,
                      function(s) "fill" %in% s$aesthetics,
                      logical(1)))
  if (!length(idx)) return(NULL)
  fig$scales$scales[[idx[1]]]$palette(
    length(fig$scales$scales[[idx[1]]]$range$range)
  )
}

# For stacked_column
olympics <- data.frame(
  Country   = c("Norway", "Norway", "Norway",
                "Germany", "Germany", "Germany",
                "United States", "United States", "United States",
                "Canada", "Canada", "Canada"),
  Continent = c("Europe", "Europe", "Europe",
                "Europe", "Europe", "Europe",
                "North America", "North America", "North America",
                "North America", "North America", "North America"),
  Medal     = rep(c("Gold", "Silver", "Bronze"), times = 4),
  Count     = c(148, 133, 124,
                102,  98,  65,
                113, 122,  95,
                 77,  72,  80)
)

twostack_df <- data.frame(
  x = c("2024", "2024", "2024", "2025", "2025", "2025"),
  y = c(10, 20, 15, 12, 18, 22),
  grp = c("g1", "g2", "g3", "g4", "g5", "g6"),
  stack = c("A", "A", "A", "B", "B", "B"),
  stringsAsFactors = FALSE
)

#
# All register_geom() calls live here so there is one place to see every
# geometry's full contract: required_args (must supply), optional_args
# (have defaults, may omit).
#
# The optional_args field is what eg. geom_args("line") prints for the user.
# Each entry is list(default = <value>, desc = "<short string>").
# The geom function itself applies the default via `geom_params$key %||%
# default` - the registry entry is informational only, not enforced.
#
# Rule of thumb for what goes in optional_args vs required_args:
#   required  - the geom cannot render at all without it (e.g. ymin/ymax)
#   optional  - the geom works fine with a built-in default (e.g. smooth)

#' @keywords internal
.onLoad <- function(libname, pkgname) {
  # Built-in colour palettes ------------------------------------------- -------
  register_palette("hdir", c(
    "#025169", "#0069E8",
    "#7C145C", "#C68803",
    "#047FA4", "#38A389",
    "#6996CE", "#366558",
    "#BF78DE", "#767676"
  ))

  # hdir2: two-colour palette for binary group comparisons
  ## register_palette("hdir2", c("rgba(49,101,117,1)", "rgba(138,41,77,1)")) ##
  register_palette("hdir2", c("#315975", "#8A294D"))

  # Backends ----------------------------------------------------------- -------
  register_backend("ggplot2",     ggplot_engine)
  register_backend("highcharter", highcharter_engine)

  # -- Geometries -------------------------------------------------------------
  # Geom optional args need to be defined here. They will be used in two different
  # places:
  # 1 - geom_args()
  # 2 - app output$ui_geom_opts() or geom_intpu_r object in Shiny server
  # --------------------------------------------------------------------------
  # .geom_registry_defs is defined in R/additional_args.R.
  # The gg_* and hc_* function objects are looked up here by name so that
  # the plain list in additional_args.R does not need to reference them
  # (function objects are not available at list-parse time in that file).
  .fn <- function(geom_name, suffix) {
    fn_name <- paste0(suffix, "_", geom_name)
    if (exists(fn_name, mode = "function", envir = parent.env(environment()),
               inherits = TRUE))
      get(fn_name, mode = "function", envir = parent.env(environment()),
          inherits = TRUE)
    else
      NULL
  }

  for (g in names(.geom_registry_defs)) {
    def <- .geom_registry_defs[[g]]
    register_geom(
      name            = g,
      ggplot_fun      = .fn(g, "gg"),
      highcharter_fun = .fn(g, "hc"),
      required_args   = def$required_args %||% character(),
      optional_args   = def$optional_args %||% list(),
      is_map_geom     = isTRUE(def$is_map_geom)
    )
  }

  # -- Option defaults ---------------------------------------------------------
  op     <- options()
  to_set <- .hd_defaults[!names(.hd_defaults) %in% names(op)]
  if (length(to_set)) options(to_set)

  invisible()
}


.onAttach <- function(libname = find.package("highdir"),
                      pkgname = "highdir") {
  if (stats::runif(1) <= 1 / 3) {
    packageStartupMessage("NB! Highcharts (www.highcharts.com) is a Highsoft software product which is")

    packageStartupMessage("not free for commercial and Governmental use")
  }
}

#' Save a Figure to Disk
#'
#' Exports a `highchart` or `ggplot` figure produced by [hd_make()] to a
#' file. The output format is inferred from the file extension unless `type`
#' is supplied explicitly.
#'
#' | Backend     | Supported formats                        |
#' |:----------- |:---------------------------------------- |
#' | highcharter | `html`, `json`                           |
#' | ggplot2     | `png`, `svg`, `pdf`, `jpeg`, `jpg`,      |
#' |             | `tiff`, `bmp`, `eps`                     |
#'
#' To export a highcharter figure as an image, either save as `html` and
#' screenshot in a browser, or re-render with `backend = "ggplot2"` in
#' [hd_make()] and save as `png`.
#'
#' @param fig           A `highchart` or `ggplot` object (output of
#'   [hd_make()]).
#' @param file          Character. Output path including extension.
#' @param type          `"auto"` (infer from extension) or an explicit format
#'   string. Default: `"auto"`.
#' @param width         Numeric. Width in inches (ggplot2). Default: `8`.
#' @param height        Numeric. Height in inches (ggplot2). Default: `5`.
#' @param dpi           Numeric. Raster resolution for ggplot2. Default: `300`.
#' @param selfcontained Logical. Embed all JS/CSS in the HTML file.
#'   Default: `TRUE`.
#' @param ...           Passed to [ggplot2::ggsave()] for ggplot2 figures.
#'
#' @return `file`, invisibly.
#' @export
hd_save <- function(fig,
                    file,
                    type          = "auto",
                    width         = 8,
                    height        = 5,
                    dpi           = 300,
                    selfcontained = TRUE,
                    ...) {

  type <- .resolve_type(type, file)

  if (inherits(fig, "highchart")) {
    .save_highchart(fig, file, type, width, height, selfcontained)
  } else if (inherits(fig, "ggplot")) {
    .save_ggplot(fig, file, type, width, height, dpi, ...)
  } else {
    stop(
      "`fig` must be a highchart or ggplot object.\n",
      "Pass the direct output of hd_make().",
      call. = FALSE
    )
  }

  invisible(file)
}

# -- Internals -----------------------------------------------------------------

#' @keywords internal
.resolve_type <- function(type, file) {
  if (!identical(type, "auto")) return(tolower(trimws(type)))
  ext <- tolower(tools::file_ext(file))
  if (!nzchar(ext))
    stop(
      "Cannot infer format from `file`: no file extension found.\n",
      "Add an extension e.g. 'chart.html' or 'chart.png'.",
      call. = FALSE
    )
  ext
}

#' @keywords internal
.sanitise_hc_config <- function(x) {
  # Recursively convert JS_EVAL objects to plain character strings.
  # highcharter stores raw JavaScript (formatters, callbacks) as JS_EVAL -
  # a character vector with class attribute "JS_EVAL". jsonlite has no
  # asJSON method for this class and errors without this conversion.
  # The JS string content is preserved; only the class wrapper is removed.
  if (inherits(x, "JS_EVAL")) return(as.character(x))
  if (is.list(x))              return(lapply(x, .sanitise_hc_config))
  x
}

#' @keywords internal
.save_highchart <- function(hc, file, type, width, height, selfcontained) {

  # -- Format gate -------------------------------------------------------------
  hc_formats <- c("html", "json")

  if (!type %in% hc_formats)
    stop(
      "Format '.", type, "' is not supported for highcharter figures.\n",
      "Supported formats: ", paste0(".", hc_formats, collapse = ", "), "\n",
      "To save as an image use backend = 'ggplot2' in hd_make(), ",
      "then call hd_save() with '.png', '.svg', or '.pdf'.",
      call. = FALSE
    )

  switch(type,

    html = {
      if (!requireNamespace("htmlwidgets", quietly = TRUE))
        stop(
          "HTML export requires the htmlwidgets package.\n",
          "Install: install.packages('htmlwidgets')",
          call. = FALSE
        )
      htmlwidgets::saveWidget(
        widget        = hc,
        file          = normalizePath(file, mustWork = FALSE),
        selfcontained = selfcontained
      )
    },

    json = {
      if (!requireNamespace("jsonlite", quietly = TRUE))
        stop(
          "JSON export requires the jsonlite package.\n",
          "Install: install.packages('jsonlite')",
          call. = FALSE
        )
      cfg <- hc$x$hc_opts
      if (is.null(cfg))
        stop(
          "Could not extract Highcharts config from widget.\n",
          "Ensure `fig` is the direct output of hd_make().",
          call. = FALSE
        )
      jsonlite::write_json(
        .sanitise_hc_config(cfg),
        path       = file,
        pretty     = TRUE,
        auto_unbox = TRUE
      )
    }
  )
}

#' @keywords internal
.save_ggplot <- function(p, file, type, width, height, dpi, ...) {

  gg_formats <- c("png", "svg", "pdf", "jpeg", "jpg", "tiff", "bmp", "eps")

  if (!type %in% gg_formats)
    stop(
      "Format '.", type, "' is not supported for ggplot2 figures.\n",
      "Supported formats: ", paste0(".", gg_formats, collapse = ", "),
      call. = FALSE
    )

  ggplot2::ggsave(
    filename = file,
    plot     = p,
    device   = type,
    width    = width,
    height   = height,
    dpi      = dpi,
    ...
  )
}

# tests/testthat/test-save.R

spec   <- hd_spec(data.frame(x = c("A","B"), y = c(1,2)), "x", "y")
fig_hc <- hd_make(spec, "column", hd_opts())
fig_gg <- hd_make(spec, "column", hd_opts(), backend = "ggplot2")

# ── HC: allowed ───────────────────────────────────────────────────────────────
test_that("hd_save: HC to HTML works", {
  f <- tempfile(fileext = ".html")
  withr::defer(unlink(f))
  expect_invisible(hd_save(fig_hc, f))
  expect_true(file.exists(f))
})

test_that("hd_save: HC to JSON works", {
  f <- tempfile(fileext = ".json")
  withr::defer(unlink(f))
  hd_save(fig_hc, f)
  expect_true(file.exists(f))
  expect_true(is.list(jsonlite::read_json(f)))
})

# ── HC: blocked — clear error pointing to ggplot2 path ───────────────────────
test_that("hd_save: HC to PNG errors with ggplot2 suggestion", {
  expect_error(
    hd_save(fig_hc, "chart.png"),
    regexp = "not supported for highcharter"
  )
})

test_that("hd_save: HC to PDF errors", {
  expect_error(hd_save(fig_hc, "chart.pdf"), "not supported for highcharter")
})

test_that("hd_save: HC to SVG errors", {
  expect_error(hd_save(fig_hc, "chart.svg"), "not supported for highcharter")
})

# ── ggplot2: allowed ──────────────────────────────────────────────────────────
test_that("hd_save: gg to PNG works", {
  f <- tempfile(fileext = ".png")
  withr::defer(unlink(f))
  hd_save(fig_gg, f)
  expect_true(file.exists(f))
})

test_that("hd_save: gg to PDF works", {
  f <- tempfile(fileext = ".pdf")
  withr::defer(unlink(f))
  hd_save(fig_gg, f)
  expect_true(file.exists(f))
})

test_that("hd_save: gg to SVG works", {
  skip_if_not_installed("svglite")
  f <- tempfile(fileext = ".svg")
  withr::defer(unlink(f))
  hd_save(fig_gg, f)
  expect_true(file.exists(f))
})

# ── Bad inputs ────────────────────────────────────────────────────────────────
test_that("hd_save: no extension errors", {
  expect_error(hd_save(fig_hc, "chart"), "no file extension")
})

test_that("hd_save: non-figure object errors", {
  expect_error(hd_save(list(a = 1), "chart.html"), "must be a highchart")
})

test_that("hd_save: returns file path invisibly", {
  f <- tempfile(fileext = ".html")
  withr::defer(unlink(f))
  result <- hd_save(fig_hc, f)
  expect_equal(result, f)
})

# ------------------------------------------------------------------------------
# PIE
# ------------------------------------------------------------------------------
# Each geometry is a pair:
#   gg_<name>  -> returns a ggplot2 layer (or list of layers)
#   hc_<name>  -> adds series to a highchart object, returns the updated chart
#
# Calling convention (enforced by the registry):
#   gg_*:  function(spec, opts, geom_params, ...)
#   hc_*:  function(chart, spec, opts, geom_params, use_js, ...)
#
# geom_params is a named list carrying all geom-specific args so that the
# engine signature stays stable as new geoms are added.  Nothing leaks into
# hc_add_series() via bare `...`.

# For a pie chart:
#   spec$x ->  slice label column
#   spec$y ->  slice value column
#   spec$group is ignored (a pie shows one series)
#
# ggplot2: geom_col() + coord_polar("y") - the classic polar-bar pie.
# highcharter: a single "pie" type series where each slice name comes from x.

#' @keywords internal
gg_pie <- function(spec, opts, geom_params, ...) {
  # ggplot2 pie = stacked bar in polar coordinates.
  # We re-map: fill = x (the label column), y = y (the value column).
  # The base canvas already has x/y mapped; we override with a coord_polar.
  list(
    ggplot2::aes(x = "", fill = .data[[spec$x]],
                 y = .data[[spec$y]]),
    ggplot2::geom_bar(stat = "identity", width = 1, colour = "white",
                      linewidth = 0.4),
    ggplot2::coord_polar(theta = "y"),
    ggplot2::labs(x = NULL, y = NULL),
    ggplot2::theme_void(),
    ggplot2::theme(legend.position = "right")
  )
}

## # Pie is always single-series from ggplot2's perspective but uses ##
## # fill to distinguish slices — single_colour is ignored here.     ##
## gg_pie <- function(spec, opts, geom_params) {                     ##
##   list(                                                           ##
##     ggplot2::geom_bar(                                            ##
##       ggplot2::aes(x    = "",                                     ##
##                    y    = .data[[spec$y]],                        ##
##                    fill = .data[[spec$x]]),                       ##
##       stat     = "identity",                                      ##
##       width    = 1,                                               ##
##       position = "stack"                                          ##
##     ),                                                            ##
##     ggplot2::coord_polar("y", start = 0),                         ##
##     ggplot2::theme_void()                                         ##
##   )                                                               ##
## }                                                                 ##


#' @keywords internal
hc_pie <- function(chart, spec, opts, geom_params, use_js = TRUE, ...) {
    inner_size <- geom_params$inner_size %||% "0%" # "50%" = donut
    df <- spec$data
    labels <- df[[spec$x]]
    values <- df[[spec$y]]
    palette <- resolve_colors(length(labels), opts$colors)

    # Build the data list Highcharts expects for a pie series
    pie_data <- lapply(seq_len(nrow(df)), function(i) {
        list(
            name = as.character(labels[i]),
            y = values[i],
            color = palette[i]
        )
    })

    chart |>
        highcharter::hc_add_series(
            type = "pie",
            name = spec$ylab,
            data = pie_data,
            innerSize = inner_size,
            dataLabels = list(
                enabled = TRUE,
                format  = "<b>{point.name}</b>: {point.percentage:.1f}%"
            )
        )
}


# ------------------------------------------------------------------------------
# Public constructor for pie geometry layer.  See ?hd for usage.
# ------------------------------------------------------------------------------
#' Pie Geometry Layer for hd Objects
#'
#' `hd_geom_pie()` creates a pie geometry layer that is added to an [hd()]
#' object via `+`.  The layer records the geometry type and any geometry-specific
#' arguments; rendering only happens when the `hd` object is printed.
#'
#' @param inner_size A string specifying the inner radius of the pie as a percentage
#'   of the total radius.  For example, "50%" creates a donut chart
#'   with a hole in the middle.  The default "0%" creates a standard pie chart.
#'   This argument is only applicable to the Highcharts backend; it is ignored
#'   by ggplot2 since it does not support donut charts.
#' @param ... Geometry-specific arguments forwarded to [hd_make()].
#' @return An S3 object of class `"hd_geom"` for use with `+.hd`.
#'
#' @examples
#' # Category share dataset (pie)
#' drinking_freq <- data.frame(
#'     category = c("Never", "Rarely", "Monthly", "Weekly", "Daily"),
#'     pct      = c(18, 25, 30, 20, 7)
#' )
#'
#' spec_pie <- hd_spec(drinking_freq,
#'     x    = "category",
#'     y    = "pct"
#' )
#'
#' opts_pie <- hd_opts(
#'     title = "Drinking frequency",
#'     subtitle = "Source: Norwegian Directorate of Health",
#'     ylab = "Share (%)"
#' )
#'
#' # Donut interactive
#' hd_make(spec_pie, "pie", opts_pie, inner_size = "50%")
#'
#' # Composable API style (ggplot2 ignores inner_size)
#' hd(drinking_freq, x = "category", y = "pct", backend = "ggplot2") +
#'     hd_geom_pie() +
#'     hd_opts(
#'         title = "Drinking frequency",
#'         subtitle = "Source: Norwegian Directorate of Health"
#'     )
#' 
#' @export
hd_geom_pie <- function(inner_size = "0%", ...) {
 hd_geom("pie", inner_size = inner_size, ...)
}

# R/figure-spec.R - Figure specification objects
#
# Two complementary constructors are provided:
#
#   hd_spec()  - *what* the data means  (x, y, group, n)
#   hd_opts()  - *how* it should look   (title, ylab, ylim, flip, colours, theme)
#
# Keeping them separate means:
#   * The same hd_spec can be rendered with different opts (e.g. EN vs NO
#     titles) without repeating data-mapping code.
#   * hd_opts objects are reusable across multiple specs.
#   * Validation errors are localised to the object they belong to.

# ------------------------------------------------------------------------------
# hd_spec  data mapping and structure
# ------------------------------------------------------------------------------
# 
#' Create a Figure Data Specification
#'
#' Defines the **data mapping** for a figure - which columns map to x, y,
#' group, and count - independently of any visual presentation choices.
#' Pass the result to [hd_make()] together with an optional [hd_opts()]
#' object.
#'
#' @param data   A `data.frame` containing all referenced columns.
#' @param x      Character. Column name for the x-axis variable.
#' @param y      Character. Column name for the y-axis variable (typically
#'   a percentage or count).
#' @param group  Character or `NULL`. Column used to split data into multiple
#'   series.
#' @param n      Character or `NULL`. Column of raw counts shown in
#'   highcharter tooltips alongside the y value.  Ignored by ggplot2.
#' @param colour Character or `NULL`. ggplot2 colour aesthetic column.
#'   Defaults to `group` when `NULL` and `group` is set.
#'
#' @return An S3 object of class `"hd_spec"`.
#'
#' @seealso [hd_opts()], [hd_make()]
#'
#' @examples
#' df <- data.frame(
#'   age = rep(c("18-24", "25-34", "35-44", "45-54"), each = 2),
#'   sex = rep(c("Male", "Female"), 4),
#'   pct = c(42, 38, 55, 61, 48, 52, 60, 57),
#'   n   = c(120, 115, 200, 210, 180, 175, 160, 155)
#' )
#'
#' spec <- hd_spec(df, x = "age", y = "pct", group = "sex", n = "n")
#' spec
#'
#' @export
hd_spec <- function(data,
                    x,
                    y,
                    group  = NULL,
                    n      = NULL,
                    colour = NULL
                    ) {

  data.table::setattr(data, "src_data", deparse1(substitute(data)))

  if (!is.data.frame(data))
    stop("`data` must be a data.frame.", call. = FALSE)

  check_columns(data, c(x, y, group, n, colour))

  structure(
    list(
      data   = as.data.frame(data),   # strip tibble/data.table subclasses
      x      = x,
      y      = y,
      group  = group,
      n      = n,
      colour = colour
    ),
    class = "hd_spec"
  )
}

# -- S3 methods ----------------------------------------------------------------

#' @export
print.hd_spec <- function(x, ...) {
  cat("<hd_spec>\n")
  cat("  data   :", get_src_data(x$data),  "\n")
  cat("  x      :", x$x,    "\n")
  cat("  y      :", x$y,    "\n")
  if (!is.null(x$group))  cat("  group  :", x$group,  "\n")
  if (!is.null(x$n))      cat("  n      :", x$n,      "\n")
  if (!is.null(x$colour)) cat("  colour :", x$colour, "\n")
  cat("  rows   :", nrow(x$data), "\n")
  invisible(x)
}

#' @export
as.list.hd_spec <- function(x, ...) {
  out      <- unclass(x)
  out$data <- sprintf("<data.frame [%d x %d]>", nrow(x$data), ncol(x$data))
  out
}


#' @keywords internal
get_src_data <- function(obj) {
  nm <- attr(obj, "src_data")
  return(if (is.null(nm)) "<?>" else nm)
}

spec <- hd_spec(survey_df, "age", "pct")
opts <- hd_opts()

test_that("validate_fig_inputs: passes for all valid inputs", {
  expect_invisible(
    validate_fig_inputs(spec, opts, "column", "highcharter", list()))
})

test_that("validate_fig_inputs: rejects non-hd_spec", {
  expect_error(
    validate_fig_inputs(list(), opts, "column", "highcharter", list()),
    "hd_spec")
})

test_that("validate_fig_inputs: rejects non-hd_opts", {
  expect_error(
    validate_fig_inputs(spec, list(), "column", "highcharter", list()),
    "hd_opts")
})

test_that("validate_fig_inputs: rejects unknown geometry", {
  expect_error(
    validate_fig_inputs(spec, opts, "violin", "highcharter", list()),
    "Unknown geometry")
})

test_that("validate_fig_inputs: rejects unknown backend", {
  expect_error(
    validate_fig_inputs(spec, opts, "column", "plotly", list()),
    "Unknown backend")
})

test_that("validate_fig_inputs: enforces required geometry args", {
  expect_error(
    validate_fig_inputs(spec, opts, "arearange", "highcharter", list()),
    "Missing required")
  expect_invisible(
    validate_fig_inputs(spec, opts, "arearange", "highcharter",
                        list(ymin = "lo", ymax = "hi")))
})

test_that("validate_fig_inputs: pie has no required args", {
  expect_invisible(
    validate_fig_inputs(spec, opts, "pie", "highcharter", list()))
})

# ------------------------------------------------------------------------------
# hd()  -- constructor
# ------------------------------------------------------------------------------

#' Initialise a Composable highdir Figure
#'
#' Creates an `hd` object that accumulates geometry and presentation layers
#' via `+`, then renders when printed.  This mirrors the way `ggplot2` builds
#' plots: data mapping is declared first; visual decisions are added
#' incrementally; nothing is rendered until the object hits the console (or an
#' explicit `print()` / `knit` call).
#'
#' `hd()` accepts either raw data columns **or** an existing [hd_spec()] object.
#' Passing an `hd_spec` is the recommended bridge for code that already
#' constructs specs separately (e.g. in Shiny or a reporting pipeline).
#'
#' @param data   A `data.frame` **or** an [hd_spec()] object.  When an
#'   `hd_spec` is supplied every other mapping argument (`x`, `y`, …) is
#'   ignored - the spec carries them already.
#' @param x      Character. Column name for the x-axis variable.
#'   Ignored when `data` is an `hd_spec`.
#' @param y      Character. Column name for the y-axis variable.
#'   Ignored when `data` is an `hd_spec`.
#' @param group  Character or `NULL`. Column used to split data into multiple
#'   series.  Ignored when `data` is an `hd_spec`.
#' @param n      Character or `NULL`. Column of raw counts shown in
#'   highcharter tooltips alongside the y value.
#'   Ignored when `data` is an `hd_spec`.
#' @param colour Character or `NULL`. ggplot2 colour aesthetic column.
#'   Defaults to `group` when `NULL` and `group` is set.
#'   Ignored when `data` is an `hd_spec`.
#' @param backend Character. Rendering engine - `"highcharter"` (default,
#'   interactive) or `"ggplot2"` (static), or any engine added with
#'   [register_backend()].  Falls back to `getOption("highdir.backend",
#'   "highcharter")`.
#'
#' @return An S3 object of class `"hd"` with slots:
#'   \describe{
#'     \item{`$spec`}{An [hd_spec()] object.}
#'     \item{`$geom`}{`NULL` until a `+ hd_geom_*()` layer is added.}
#'     \item{`$opts`}{An [hd_opts()] object (defaults until overridden).}
#'     \item{`$backend`}{Character. The resolved engine name.}
#'   }
#'
#' @seealso [hd_geom_column()], [hd_geom_line()], [hd_geom_arearange()],
#'   [hd_opts()], [hd_make()]
#'
#' @examples
#' df <- data.frame(
#'   age = rep(c("18-24", "25-34", "35-44", "45-54"), each = 2),
#'   sex = rep(c("Male", "Female"), 4),
#'   pct = c(42, 38, 55, 61, 48, 52, 60, 57),
#'   n   = c(120, 115, 200, 210, 180, 175, 160, 155)
#' )
#'
#' # Composable style
#' hd(df, x = "age", y = "pct", group = "sex") +
#'   hd_geom_column() +
#'   hd_opts(title = "Health survey", ylim = c(0, 80))
#'
#' # Pass an existing hd_spec
#' spec <- hd_spec(df, x = "age", y = "pct", group = "sex", n = "n")
#' hd(spec) +
#'   hd_geom_line(smooth = TRUE) +
#'   hd_opts(title = "Trend")
#'
#' # Switch backend per figure
#' hd(df, x = "age", y = "pct", backend = "ggplot2") +
#'   hd_geom_column() +
#'   hd_opts(title = "Static version")
#'
#' @export
hd <- function(data,
               x       = NULL,
               y       = NULL,
               group   = NULL,
               n       = NULL,
               colour  = NULL,
               backend = getOption("highdir.backend", "highcharter")) {

  spec <- if (inherits(data, "hd_spec")) {
    data  # already a spec - use as-is
  } else {
    hd_spec(data,
            x      = x,
            y      = y,
            group  = group,
            n      = n,
            colour = colour)
  }

  structure(
    list(
      spec    = spec,
      geom    = NULL,           # set by + hd_geom_*()
      opts    = default_opts(), # set by + hd_opts(); starts with all defaults
      backend = backend
    ),
    class = "hd"
  )
}


# ------------------------------------------------------------------------------
# hd_geom()  -- internal geom-layer constructor
# ------------------------------------------------------------------------------

#' @keywords internal
hd_geom <- function(type, ...) {
  structure(
    list(type = type, params = list(...)),
    class = "hd_geom"
  )
}



# ------------------------------------------------------------------------------
# +.hd  -- accumulate layers
# ------------------------------------------------------------------------------

#' Add Layers to an hd Object
#'
#' Implements the `+` operator for [hd()] objects, mirroring how `ggplot2`
#' builds plots incrementally.  Recognised right-hand sides:
#'
#' \describe{
#'   \item{`hd_geom` object}{Sets the geometry (from any `hd_geom_*()` call).
#'     Adding a second geom replaces the first where highdir renders one geometry
#'     per figure.}
#'   \item{`hd_opts` object}{Sets presentation options.  Adding a second
#'     `hd_opts` replaces the first.}
#' }
#'
#' Unknown right-hand sides produce an informative error rather than silently
#' being ignored.
#'
#' @param e1 An `hd` object (left-hand side).
#' @param e2 An `hd_geom` or `hd_opts` object (right-hand side).
#'
#' @return The updated `hd` object (invisibly, so the chain prints once).
#'
#' @examples
#' df <- data.frame(age = c("18-24", "25-34"), pct = c(42, 55))
#'
#' # Chain layers
#' p <- hd(df, x = "age", y = "pct") +
#'   hd_geom_column() +
#'   hd_opts(title = "Demo")
#'
#' # Reuse a partial object with different opts
#' base <- hd(df, x = "age", y = "pct") + hd_geom_column()
#' base + hd_opts(title = "English title")
#' base + hd_opts(title = "Norsk tittel")
#'
#' @export
`+.hd` <- function(e1, e2) {
  if (inherits(e2, "hd_geom")) {
    e1$geom <- e2
    .validate_geom_backend(e1)   # ← check immediately after attaching
  } else if (inherits(e2, "hd_opts")) {
    e1$opts <- e2
  } else {
    stop(
      "Cannot add an object of class <", paste(class(e2), collapse = "/"),
      "> to an <hd> object.\n",
      "Accepted types: hd_geom_*() or hd_opts().",
      call. = FALSE
    )
  }
  e1
}


# ------------------------------------------------------------------------------
# print.hd  -- render by delegating to hd_make()
# ------------------------------------------------------------------------------

#' Render an hd Object
#'
#' Printing an [hd()] object triggers rendering.  The geometry type and any
#' geometry-specific parameters are extracted from the stored `hd_geom` layer;
#' presentation options from the `hd_opts` layer; the backend from the
#' `$backend` slot.  All of these are forwarded to [hd_make()], which performs
#' the actual rendering via the registered engine.
#'
#' You rarely need to call `print.hd()` directly since R calls it automatically
#' when the object appears at the top level, in knitr/Quarto chunks, or in
#' Shiny `renderHighchart()` / `renderPlot()` blocks.
#'
#' @param x   An `hd` object.
#' @param ... Ignored; present for S3 consistency.
#'
#' @return The rendered output (a `highchart` widget or a `ggplot` object),
#'   invisibly.
#'
#' @export
print.hd <- function(x, ...) {

  # Resolve geom - default to "column" if no hd_geom_*() was added
  type        <- x$geom$type   %||% "column"
  geom_params <- x$geom$params %||% list()

  out <- do.call(
    hd_make,
    c(
      list(
        spec    = x$spec,
        type    = type,
        opts    = x$opts,
        backend = x$backend
      ),
      geom_params   # splices ymin/ymax/smooth/… back into hd_make()'s ...
    )
  )

  print(out)
  invisible(out)
}


# ------------------------------------------------------------------------------
# S3 helpers
# ------------------------------------------------------------------------------

#' @export
print.hd_geom <- function(x, ...) {
  cat("<hd_geom>\n")
  cat("  type  :", x$type, "\n")
  if (length(x$params)) {
    cat("  params:\n")
    for (nm in names(x$params))
      cat("    ", nm, "=", format(x$params[[nm]]), "\n")
  }
  invisible(x)
}

#' Build a Figure from a Specification
#'
#' Renders a [hd_spec] and [hd_opts] pair using the selected backend and
#' geometry.  This is the central function of the package - everything else
#' feeds into or flows out of `hd_make()`.
#'
#' @section Workflow:
#' ```r
#' spec <- hd_spec(df, x = "age", y = "pct", group = "sex", n = "n")
#' opts <- hd_opts(title = "Health survey", ylim = c(0, 80))
#'
#' hd_make(spec, "column", opts)                       # highcharter (default)
#' hd_make(spec, "column", opts, backend = "ggplot2")  # static ggplot2
#' hd_make(spec, "line",   opts, smooth = TRUE)        # smooth spline
#' hd_make(spec, "pie",    opts)                       # pie / donut
#' ```
#'
#' @param spec      A [hd_spec] object from [hd_spec()].
#' @param type      Character.  Geometry name - one of [list_geoms()]:
#'   `"column"`, `"line"`, `"scatter"`, `"arearange"`, `"pie"`, or any
#'   custom geometry added with [register_geom()].
#' @param opts      A [hd_opts] object or `NULL` (uses all defaults).
#'   Controls title, subtitle, caption, ylim, yint, flip, per-figure
#'   colours, and highcharter theme.
#' @param backend   Character.  Rendering engine - `"highcharter"` (default,
#'   interactive) or `"ggplot2"` (static), or any engine added with
#'   [register_backend()].
#' @param use_js    Logical.  When `TRUE` (default) injects a hover-band
#'   `htmlwidgets::JS()` callback via `point.events.mouseOver/Out`.
#'   Tooltips, accessibility module, and all other Highcharts declarative
#'   features are **always** present.  Set `FALSE` for clean, no-custom-JS
#'   widgets.  Ignored by the ggplot2 backend.
#' @param module Use available modules js from CDN <https://api.highcharts.com/highcharts/>
#' @param ...       Extra arguments forwarded to the geometry function.
#'   Required arguments (e.g. `ymin`, `ymax` for `"arearange"`) **must**
#'   be supplied here.
#'
#' @return A `highchart` widget (highcharter backend) or `ggplot` object
#'   (ggplot2 backend), invisibly wrapped so knitr/Shiny render it
#'   automatically.
#'
#' @seealso [hd_spec()], [hd_opts()], [hd_save()], [hd_set_theme()],
#'   [list_geoms()], [list_backends()], [hd_app()]
#'
#' @examples
#' df <- data.frame(
#'   age = rep(c("18-24", "25-34", "35-44", "45-54"), each = 2),
#'   sex = rep(c("Male", "Female"), 4),
#'   pct = c(42, 38, 55, 61, 48, 52, 60, 57),
#'   n   = c(120, 115, 200, 210, 180, 175, 160, 155)
#' )
#'
#' spec <- hd_spec(df, x = "age", y = "pct", group = "sex", n = "n")
#'
#' opts <- hd_opts(title    = "Health survey results",
#'                  subtitle = "Source: FHI 2024",
#'                  ylim     = c(0, 80))
#'
#' \donttest{
#' # -- Interactive charts (highcharter) --------------------------------------
#' hd_make(spec, "column", opts)
#' hd_make(spec, "line",   opts, smooth = TRUE)
#' hd_make(spec, "line",   opts, smooth = FALSE, dot_size = 6)
#' hd_make(spec, "scatter")
#'
#' # Pie chart - group is ignored; x = label, y = value
#' pie_df   <- data.frame(category = c("A","B","C","D"),
#'                         value    = c(35, 25, 20, 20))
#' pie_spec <- hd_spec(pie_df, x = "category", y = "value")
#' pie_opts <- hd_opts(title = "Share by category")
#' hd_make(pie_spec, "pie", pie_opts)
#' hd_make(pie_spec, "pie", pie_opts, inner_size = "50%")  # donut
#'
#' # Arearange - requires ymin + ymax in ...
#' df2   <- cbind(df, lo = df$pct - 5, hi = df$pct + 5)
#' spec2 <- hd_spec(df2, "age", "pct", group = "sex")
#' hd_make(spec2, "arearange", opts, ymin = "lo", ymax = "hi")
#'
#' # -- Disable JS hover band -------------------------------------------------
#' hd_make(spec, "column", opts, use_js = FALSE)
#'
#' # -- Static ggplot2 versions -----------------------------------------------
#' hd_make(spec, "column",  opts, backend = "ggplot2")
#' hd_make(spec, "line",    opts, backend = "ggplot2")
#' hd_make(spec, "scatter", opts, backend = "ggplot2")
#' hd_make(pie_spec, "pie", pie_opts, backend = "ggplot2")
#'
#' # -- Reuse spec with different presentation --------------------------------
#' opts_no <- hd_opts(title = "Helseundersøkelse", subtitle = "Alle aldre")
#' hd_make(spec, "column", opts_no)
#'
#' # -- Save outputs ----------------------------------------------------------
#' hd_save(hd_make(spec, "column", opts),               "column.html")
#' hd_save(hd_make(spec, "column", opts, backend="ggplot2"), "column.png")
#' }
#'
#' @export
hd_make <- function(spec,
                    type        = "column",
                    opts        = NULL,
                    backend     = "highcharter",
                    use_js      = TRUE,
                    module      = FALSE,
                    ...) {

  opts <- opts %||% default_opts()
  extra_args  <- list(...)

  validate_fig_inputs(spec, opts, type, backend, extra_args)
  spec <- check_decimals(spec, opts, type, extra_args)

  geom   <- .get_geom(type)
  engine <- get_backend(backend)

  engine(
    spec        = spec,
    geom        = geom,
    opts        = opts,
    geom_params = extra_args,
    use_js      = use_js,
    module      = module
  )
}

# R/shiny-app.R -- Shiny GUI launcher

#' Launch the highdir Shiny GUI
#'
#' Opens an interactive browser-based application for building figures with
#' `highcharter` or `ggplot2` without writing R code.
#'
#' The UI (`inst/app/ui.R`), server (`inst/app/server.R`) and shared setup
#' (`inst/app/global.R`) live in `inst/app/` so the folder can be deployed
#' independently to Shiny Server or shinyapps.io.
#'
#' @param return.app Logical. When deploying to server like Shiny.io to return
#'   the app object instead of launching it.
#' @section Features: \itemize{ \item Upload datasets in any format supported by
#'   **rio** (CSV, XLSX, SPSS, Stata, RDS, …). \item Choose geometry (`column`,
#'   `line`, `scatter`, `arearange`, `pie`), backend, axis variables, and group
#'   column. \item Set title, subtitle, caption, colour palette, and HC theme.
#'   \item Toggle JS hover band per figure. \item Render on demand with the
#'   **Draw** button. \item Download as HTML / JSON / PNG (highcharter) or PNG /
#'   SVG (ggplot2). \item Copy the equivalent `hd_make()` call from the **R
#'   code** tab. }
#'
#' @return Launches a Shiny app; does not return a value.
#' @seealso [hd_make()], [hd_spec()], [hd_opts()], [hd_save()]
#' @export
hd_app <- function(return.app = FALSE) {
  app_dir <- system.file("app", package = "highdir")
  if (!nzchar(app_dir) || !dir.exists(app_dir))
    stop("Could not find inst/app/. Is highdir installed?", call. = FALSE)

  app <- shiny::shinyAppDir(app_dir)   # builds the app object from the directory
  if (return.app) return(app)
  shiny::runApp(app)
}

# Internal helpers (not exported)
#
# All small utilities used across the package live here so there is a single
# place to look for them.  Nothing in this file is exported.

# -- NULL coalescing -----------------------------------------------------------

#' @keywords internal
`%||%` <- function(a, b) if (is.null(a)) b else a

# -- Type predicates -----------------------------------------------------------

#' @keywords internal
is_highchart <- function(x) inherits(x, "highchart")

#' @keywords internal
is_ggplot <- function(x) inherits(x, c("gg", "ggplot"))

# -- Validation helpers --------------------------------------------------------

#' Stop with a tidy message when columns are absent from a data frame
#' @keywords internal
check_columns <- function(data, cols, arg_name = "data") {
  cols    <- cols[!is.null(cols) & !is.na(cols)]
  missing <- setdiff(cols, names(data))
  if (length(missing) > 0)
    stop("Column(s) not found in `", arg_name, "`: ",
         paste(missing, collapse = ", "), call. = FALSE)
  invisible(NULL)
}

#' Validate the ylim argument
#' @keywords internal
check_ylim <- function(ylim) {
  if (is.null(ylim)) return(invisible(NULL))
  if (!is.numeric(ylim) || length(ylim) != 2)
    stop("`ylim` must be a numeric vector of length 2, e.g. c(0, 100)",
         call. = FALSE)
  if (ylim[1] >= ylim[2])
    stop("`ylim[1]` must be less than `ylim[2]`", call. = FALSE)
  invisible(NULL)
}

# --- Line-symbol helpers ----

#' Valid Highcharts marker symbol names
#' @keywords internal
.hc_symbols <- c("circle", "square", "diamond", "triangle", "triangle-down")

#' Resolve and validate line symbols for n groups
#'
#' @param n Integer. Number of groups.
#' @param symbols Character vector or `NULL` supplied by the user.
#' @return Character vector of length `n`.
#' @keywords internal
resolve_symbols <- function(n, symbols = NULL) {
  if (is.null(symbols))
    return(rep(.hc_symbols, length.out = n))

  invalid <- setdiff(symbols, .hc_symbols)
  if (length(invalid) > 0) {
    warning("Invalid marker symbol(s): ", paste(invalid, collapse = ", "),
            ". Valid: ", paste(.hc_symbols, collapse = ", "),
            ". Using defaults.", call. = FALSE)
    return(rep(.hc_symbols, length.out = n))
  }
  if (length(symbols) != n)
    warning("Number of symbols (", length(symbols), ") != groups (", n,
            "). Recycling.", call. = FALSE)
  rep(symbols, length.out = n)
}

## ----- Modules dependency -----------
#' Version-safe hc_add_dependency wrapper
#'
#' highcharter 0.9.4 takes the path as a positional argument.
#' Older versions used name = . This wrapper handles both.
#' @keywords internal
.hd_add_dep <- function(chart, path) {
  tryCatch(
    highcharter::hc_add_dependency(chart, path),
    error = function(e)
      highcharter::hc_add_dependency(chart, name = path)
  )
}

## ---- Axis labelling -----------
#' Resolve an axis label from opts and spec
#'
#' Three-way logic:
#'   NULL   -> hide the axis label entirely
#'   " "    -> use the column name from spec as the fallback
#'   string -> use the string as-is
#'
#' @param opts_label The value from hd_opts()$ylab or $xlab.
#' @param spec_col   The column name from hd_spec()$y or $x.
#' @return Character string or NULL.
#' @keywords internal
.resolve_axis_label <- function(opts_label, spec_col) {

  if (is.null(opts_label))
    return(NULL)          # explicit NULL -> hide

  if (identical(opts_label, " "))
    return(spec_col)      # sentinel -> use column name

  opts_label              # any other string -> use as-is
}


# -- Axis label hiding (applied AFTER theme) --------------------------------
# -- For ggplot2 ---------------------------------------------------------------
# .resolve_axis_label() is called again here (not just in base_fig) because:
#   1. gt$theme can overwrite element_blank() that base_fig() set earlier.
#   2. Geoms like ranked_bar bypass base_fig() entirely, so their labels
#      would never be set otherwise.
# Applying after gt$theme guarantees the resolved label always wins.
.apply_axis_label <- function(p, resolved, axis) {
  if (is.null(resolved)) {
    axis_blank <- stats::setNames(list(resolved), axis)
    p + do.call(ggplot2::labs, axis_blank)
  } else {
    # String -> set the label (covers both column-name fallback and custom text)
    axis_labs <- stats::setNames(list(resolved), axis)
    p + do.call(ggplot2::labs, axis_labs)
  }
}


# Round numeric column ---------------------------------------------------------
#' @keywords internal
round_column <- function(data, column, digits = 0) {
  # --- Validate dataset ---
  if (!is.data.frame(data)) {
    stop("`data` must be a data.frame.", call. = FALSE)
  }
  
  if (!column %in% names(data)) {
    stop(sprintf("Column '%s' does not exist in the dataset.", column),
         call. = FALSE)
  }
  
  # --- Validate digits ---
  if (length(digits) != 1 || is.na(digits)) {
    stop("`digits` must be a single non-NA numeric value.", call. = FALSE)
  }
  
  # Coerce digits safely
  digits_num <- suppressWarnings(as.numeric(digits))
  if (is.na(digits_num)) {
    stop("`digits` must be numeric.", call. = FALSE)
  }
  digits_int <- as.integer(round(digits_num))  # enforce integer
  
  # --- Validate column type ---
  col <- data[[column]]
  
  if (!is.numeric(col)) {
    stop(sprintf(
      "Column '%s' is not numeric (found class: %s). Cannot apply rounding.",
      column, paste(class(col), collapse = ", ")
    ),
    call. = FALSE)
  }
  
  # --- Perform rounding (safe, no mutation of input data) ---
  data[[column]] <- round(col, digits_int)
  
  # --- Return modified dataset ---
  return(data)
}

check_decimals <- function(spec, opts, type, extra_args){

  decs <- opts$decimals
  
  if (!is.null(decs))
    spec$data <- round_column(spec$data, spec$y, decs)

  if (type == "arearange" && !is.null(decs)){
    spec$data <- round_column(spec$data, extra_args$ymin, decs)
    spec$data <- round_column(spec$data, extra_args$ymax, decs)
  }
  
  return(spec)
}


#' Version-safe highcharter accessibility description setter
#'
#' highcharter 0.9.4 does not export hc_accessibility().
#' This wrapper patches chart$x$hc_opts$accessibility directly, which works across
#' all highcharter versions because hc_opts is the raw Highcharts config.
#'
#' @param chart A highchart object.
#' @param description Character. The accessibility description string.
#' @return The modified highchart object.
#' @keywords internal
.hd_accessibility <- function(chart, description) {
      # chart$x$hc_opts is the plain list serialised to JSON by htmlwidgets.
      # Setting accessibility$description here is equivalent to:
      #   Highcharts.chart({ accessibility: { description: "..." } })
      chart$x$hc_opts <- utils::modifyList(
        chart$x$hc_opts %||% list(),
        list(accessibility = list(description = description))
      )
      chart
}


# Validate that the geometry-specific arguments in hd_geom are compatible with
# the current backend. If any arguments are marked as backend-specific and the
# current backend doesn't match, issue a warning. This function is called
# immediately after adding a geom layer to an hd object, so that users get early
# feedback if they accidentally use a ggplot2-only argument while the backend is
# set to highcharter (or vice versa). The function looks up the geom type in the
# registry to find all its arguments, checks if any are marked as
# backend-specific, and if so, compares the current backend with the required
# one. If there's a mismatch and the user supplied a non-default value for that
# argument, it issues a warning that the argument will be ignored. This
# validation helps prevent silent failures where a user might set an argument
# that only applies to ggplot2 while using the highcharter backend, and then
# wonder why it has no effect. By warning them immediately, they can correct
# their code before proceeding further. The function does not stop execution; it
# only issues warnings for incompatible arguments. The actual rendering
# functions for each backend should also be designed to ignore any arguments
# that don't apply to them, so this validation is an additional user-friendly
# check rather than a strict enforcement mechanism. The function assumes that
# the geom registry entries have a structure where each argument's metadata
# includes a `backend_only` field that specifies if the argument is exclusive to
# a particular backend. It also assumes that the `hd_geom` object has a `type`
# field that identifies the geom type, and a `params` list that contains the
# user-supplied values for the geom arguments.
# 
#' @keywords internal
.validate_geom_backend <- function(hd_obj) {
  geom   <- hd_obj$geom
  be     <- hd_obj$backend
  params <- geom$params
  type   <- geom$type

  # Use .get_geom() - note the leading dot, matching registry.R line 109
  reg <- .get_geom(type)
  if (is.null(reg)) return(invisible(NULL))   # unknown geom - skip silently

  all_args <- c(
    reg$required_args %||% list(),
    reg$optional_args %||% list()
  )

  for (arg_name in names(all_args)) {
    arg_meta     <- all_args[[arg_name]]
    backend_only <- arg_meta$backend_only     # NULL if not set
    user_value   <- params[[arg_name]]
    def          <- arg_meta$default

    # Only warn when ALL of these are true:
    #   1. the arg declares a backend restriction
    #   2. the current backend is not that backend
    #   3. the user actually supplied a value for this arg
    #   4. the supplied value differs from the default (so default pass-through
    #      from the constructor does not trigger a spurious warning)
    if (!is.null(backend_only) &&
        be != backend_only &&
        !is.null(user_value) &&
        !identical(user_value, def)) {

      warning(
        "`", arg_name, " = \"", user_value, "\"` is only supported by the ",
        backend_only, " backend and will be ignored.\n",
        "Current backend is '", be, "'.",
        call. = FALSE
      )
    }
  }

  invisible(NULL)
}

test_that("%||% returns left when non-NULL", {
  expect_equal("a" %||% "b", "a")
  expect_equal(0L  %||% 99L, 0L)
  expect_equal(FALSE %||% TRUE, FALSE)
})

test_that("%||% returns right when NULL", {
  expect_equal(NULL %||% "b",   "b")
  expect_equal(NULL %||% 42,    42)
  expect_equal(NULL %||% FALSE, FALSE)
})

test_that("is_highchart / is_ggplot predicates", {
  p <- ggplot2::ggplot()
  expect_true(is_ggplot(p))
  expect_false(is_highchart(p))
  expect_false(is_ggplot(list()))
  expect_false(is_highchart(list()))
})

test_that("check_columns: passes for present columns", {
  df <- data.frame(a = 1, b = 2)
  expect_invisible(check_columns(df, c("a", "b")))
})

test_that("check_columns: stops with informative message for missing", {
  df <- data.frame(a = 1)
  expect_error(check_columns(df, c("a", "z")), "z")
  expect_error(check_columns(df, "missing"),    "missing")
})

test_that("check_columns: NULL values in cols vector are ignored", {
  df <- data.frame(a = 1)
  expect_invisible(check_columns(df, NULL))
  expect_invisible(check_columns(df, c("a", NULL)))
})

test_that("check_ylim: passes NULL and valid ranges", {
  expect_invisible(check_ylim(NULL))
  expect_invisible(check_ylim(c(0, 100)))
  expect_invisible(check_ylim(c(-10, 10)))
})

test_that("check_ylim: fails on bad inputs", {
  expect_error(check_ylim(c(100, 0)),    "ylim\\[1\\]")
  expect_error(check_ylim(c(0, 0)),      "ylim\\[1\\]")
  expect_error(check_ylim(c(1, 2, 3)),   "length 2")
  expect_error(check_ylim("text"),        "length 2")
})

test_that("resolve_symbols: defaults recycle .hc_symbols", {
  s3 <- resolve_symbols(3)
  expect_length(s3, 3)
  expect_true(all(s3 %in% .hc_symbols))
})

test_that("resolve_symbols: warns on invalid symbols and falls back", {
  expect_warning(resolve_symbols(2, c("hexagon")), "Invalid")
})

test_that("resolve_symbols: warns on length mismatch and recycles", {
  expect_warning(resolve_symbols(4, c("circle", "square")), "Recycling")
})

## --- Axis labeling --
test_that(".resolve_axis_label: sentinel returns column name", {
  expect_equal(highdir:::.resolve_axis_label(" ", "rate"), "rate")
})

test_that(".resolve_axis_label: NULL returns NULL", {
  expect_null(highdir:::.resolve_axis_label(NULL, "rate"))
})

test_that(".resolve_axis_label: custom string returned as-is", {
  expect_equal(
    highdir:::.resolve_axis_label("Rate per 100 000", "rate"),
    "Rate per 100 000"
  )
})

# tests/testthat/test-stacked-column.R
#
# Covers gg_stacked_column() and hc_stacked_column() via hd_make().
#
# Datasets (defined in helper.R):
#   olympics  — 12-row medals table: Country, Continent, Medal, Count
#                  Two stacks (Europe, North America), two series per stack.
#   twostack_df  — minimal 8-row frame for edge-case and alignment tests.
#
# HOW WE INSPECT HC OUTPUT
#   A highchart widget stores its config in fig$x$hc_opts.
#   We read series metadata from fig$x$hc_opts$series (a list of lists),
#   and plot options from fig$x$hc_opts$plotOptions.
#
# HOW WE INSPECT GG OUTPUT
#   ggplot2 stores layers in fig$layers and facet spec in fig$facet.
#   We use ggplot_build() for rendered-data checks.

# ── Shared spec / opts ─────────────────────────────────────────────────────────

spec_ol <- hd_spec(olympics,
                   x     = "Medal",
                   y     = "Count",
                   group = "Country")

opts_ol <- hd_opts(
  title    = "Olympic medals by continent",
  subtitle = "All-time totals",
  ylab     = "Count medals"
)


# ══════════════════════════════════════════════════════════════════════════════
# 1.  Return types
# ══════════════════════════════════════════════════════════════════════════════

test_that("HC stacked_column returns highchart", {
  fig <- hd_make(spec_ol, "stacked_column", opts_ol, stack = "Continent")
  expect_true(is_highchart(fig))
})

test_that("GG stacked_column returns ggplot", {
  fig <- hd_make(spec_ol, "stacked_column", opts_ol,
                 stack   = "Continent",
                 backend = "ggplot2")
  expect_true(ggplot2::is_ggplot(fig))
})


# ══════════════════════════════════════════════════════════════════════════════
# 2.  Required-arg validation
# ══════════════════════════════════════════════════════════════════════════════

test_that("HC errors when stack arg is missing", {
  expect_error(
    hd_make(spec_ol, "stacked_column", opts_ol),
    "Missing required"
  )
})

test_that("GG errors when stack arg is missing", {
  expect_error(
    hd_make(spec_ol, "stacked_column", opts_ol, backend = "ggplot2"),
    "Missing required"
  )
})

test_that("HC errors when group column absent from hd_spec", {
  spec_no_grp <- hd_spec(olympics, x = "Medal", y = "Count")
  expect_error(
    hd_make(spec_no_grp, "stacked_column", opts_ol, stack = "Continent"),
    "group column"
  )
})

test_that("GG errors when group column absent from hd_spec", {
  spec_no_grp <- hd_spec(olympics, x = "Medal", y = "Count")
  expect_error(
    hd_make(spec_no_grp, "stacked_column", opts_ol,
            stack = "Continent", backend = "ggplot2"),
    "group column"
  )
})


# ══════════════════════════════════════════════════════════════════════════════
# 3.  Highcharter series structure
# ══════════════════════════════════════════════════════════════════════════════

test_that("HC: number of series equals unique (Country x Continent) combos", {
  fig    <- hd_make(spec_ol, "stacked_column", opts_ol, stack = "Continent")
  series <- fig$x$hc_opts$series
  # 4 countries x 1 continent each = 4 series
  expect_equal(length(series), 4L)
})

test_that("HC: every series has a non-empty name", {
  fig    <- hd_make(spec_ol, "stacked_column", opts_ol, stack = "Continent")
  series <- fig$x$hc_opts$series
  names  <- vapply(series, `[[`, character(1), "name")
  expect_true(all(nzchar(names)))
})

test_that("HC: series names match Country values", {
  fig    <- hd_make(spec_ol, "stacked_column", opts_ol, stack = "Continent")
  series <- fig$x$hc_opts$series
  s_names <- vapply(series, `[[`, character(1), "name")
  expect_true(all(s_names %in% unique(olympics$Country)))
})

test_that("HC: stack field matches Continent values", {
  fig    <- hd_make(spec_ol, "stacked_column", opts_ol, stack = "Continent")
  series <- fig$x$hc_opts$series
  stacks <- vapply(series, `[[`, character(1), "stack")
  expect_true(all(stacks %in% unique(olympics$Continent)))
})

test_that("HC: Norway series has correct data values", {
  fig    <- hd_make(spec_ol, "stacked_column", opts_ol, stack = "Continent")
  series <- fig$x$hc_opts$series
  norway <- Filter(function(s) s$name == "Norway", series)[[1]]
  # Gold=148, Silver=133, Bronze=124 — order follows unique(Medal) in data
  expect_equal(unlist(norway$data), c(148, 133, 124))
})

test_that("HC: each series has a color field", {
  fig    <- hd_make(spec_ol, "stacked_column", opts_ol, stack = "Continent")
  series <- fig$x$hc_opts$series
  colors <- vapply(series, function(s) !is.null(s$color), logical(1))
  expect_true(all(colors))
})

test_that("HC: stacking mode defaults to 'normal'", {
  fig <- hd_make(spec_ol, "stacked_column", opts_ol, stack = "Continent")
  stacking <- fig$x$hc_opts$plotOptions$column$stacking
  expect_equal(stacking, "normal")
})

test_that("HC: stacking = 'percent' is passed through", {
  fig <- hd_make(spec_ol, "stacked_column", opts_ol,
                 stack    = "Continent",
                 stacking = "percent")
  stacking <- fig$x$hc_opts$plotOptions$column$stacking
  expect_equal(stacking, "percent")
})

test_that("HC: legend deduplication — duplicate names have showInLegend FALSE", {
  # Use twostack_df where the same grp values do NOT repeat across stacks,
  # but if they did, duplicates should be hidden.
  # Here we test the general contract: first occurrence is TRUE.
  spec_ts <- hd_spec(twostack_df, x = "x", y = "y", group = "grp")
  fig     <- hd_make(spec_ts, "stacked_column", opts_ol, stack = "stack")
  series  <- fig$x$hc_opts$series

  show_flags <- vapply(series, function(s) isTRUE(s$showInLegend), logical(1))
  # At least one series must be shown
  expect_true(any(show_flags))
})


# ══════════════════════════════════════════════════════════════════════════════
# 4.  Same series name across stacks — colour consistency
# ══════════════════════════════════════════════════════════════════════════════

test_that("HC: same series name gets same colour in different stacks", {
  # Build a dataset where "Shared" appears in both Stack1 and Stack2
  shared_df <- data.frame(
    x     = rep(c("A", "B"), times = 4),
    y     = c(10, 20, 30, 40, 15, 25, 35, 45),
    grp   = rep(c("Shared", "Only1", "Shared", "Only2"), each = 2),
    stack = rep(c("S1", "S1", "S2", "S2"), each = 2),
    stringsAsFactors = FALSE
  )
  spec_sh <- hd_spec(shared_df, x = "x", y = "y", group = "grp")
  fig     <- hd_make(spec_sh, "stacked_column", opts_ol, stack = "stack")
  series  <- fig$x$hc_opts$series

  shared_series <- Filter(function(s) s$name == "Shared", series)
  expect_equal(length(shared_series), 2L)   # appears twice
  # Both occurrences must have identical colour
  expect_equal(shared_series[[1]]$color, shared_series[[2]]$color)
})


# ══════════════════════════════════════════════════════════════════════════════
# 5.  x-axis alignment — missing categories get NA
# ══════════════════════════════════════════════════════════════════════════════

test_that("HC: series data length equals number of x categories", {
  fig    <- hd_make(spec_ol, "stacked_column", opts_ol, stack = "Continent")
  series <- fig$x$hc_opts$series
  n_cats <- length(unique(olympics$Medal))   # 3
  lengths <- vapply(series, function(s) length(s$data), integer(1))
  expect_true(all(lengths == n_cats))
})

test_that("HC: missing x category is filled with NA in series data", {
  # Norway has Gold + Silver but no Bronze entry in this slice
  partial_df <- olympics[olympics$Country != "Norway" |
                              olympics$Medal   != "Bronze", ]
  spec_p  <- hd_spec(partial_df, x = "Medal", y = "Count", group = "Country")
  fig     <- hd_make(spec_p, "stacked_column", opts_ol, stack = "Continent")
  series  <- fig$x$hc_opts$series

  norway <- Filter(function(s) s$name == "Norway", series)[[1]]
  # Bronze is the 3rd category (Gold, Silver, Bronze order in data)
  # The missing Bronze should be NA
  expect_true(is.na(norway$data[[3]]))
})


# ══════════════════════════════════════════════════════════════════════════════
# 6.  ggplot2 layer structure
# ══════════════════════════════════════════════════════════════════════════════

test_that("GG: figure has at least one geom_bar layer", {
  fig    <- hd_make(spec_ol, "stacked_column", opts_ol,
                   stack = "Continent", backend = "ggplot2")
  layer_classes <- vapply(fig$layers,
                          function(l) class(l$geom)[1],
                          character(1))
  expect_true(any(layer_classes == "GeomBar"))
})

test_that("GG: figure uses facet_wrap (one panel per stack)", {
  fig <- hd_make(spec_ol, "stacked_column", opts_ol,
                 stack = "Continent", backend = "ggplot2")
  expect_true(inherits(fig$facet, "FacetWrap"))
})

test_that("GG: facet variable is the stack column", {
  fig  <- hd_make(spec_ol, "stacked_column", opts_ol,
                  stack = "Continent", backend = "ggplot2")
  # FacetWrap stores facet vars in $params$facets
  facet_vars <- names(fig$facet$params$facets)
  expect_true("Continent" %in% facet_vars)
})

test_that("GG: rendered data contains all four countries", {
  fig   <- hd_make(spec_ol, "stacked_column", opts_ol,
                   stack = "Continent", backend = "ggplot2")
  built <- ggplot2::ggplot_build(fig)
  # Group levels are encoded as integers; check via the fill aesthetic
  # or via the raw data passed to the layer
  layer_data <- built$data[[1]]
  expect_true(nrow(layer_data) > 0)
})

test_that("GG: stacking = 'percent' does not error", {
  expect_silent(
    hd_make(spec_ol, "stacked_column", opts_ol,
            stack    = "Continent",
            stacking = "percent",
            backend  = "ggplot2")
  )
})


# ══════════════════════════════════════════════════════════════════════════════
# 7.  opts integration
# ══════════════════════════════════════════════════════════════════════════════

test_that("HC: title from opts is set on chart", {
  opts_t <- hd_opts(title = "My stacked chart", ylab = "Count")
  fig    <- hd_make(spec_ol, "stacked_column", opts_t, stack = "Continent")
  expect_equal(fig$x$hc_opts$title$text, "My stacked chart")
})

test_that("HC: custom colors are applied to series", {
  pal  <- c("#FF0000", "#00FF00", "#0000FF", "#FFFF00")
  opts_c <- hd_opts(title = "t", colors = pal)
  fig    <- hd_make(spec_ol, "stacked_column", opts_c, stack = "Continent")
  series <- fig$x$hc_opts$series
  s_colors <- vapply(series, `[[`, character(1), "color")
  expect_true(any(s_colors %in% pal))
})

test_that("GG: NULL ylab hides y-axis title", {
  opts_null <- hd_opts(title = "t", ylab = NULL)
  fig       <- hd_make(spec_ol, "stacked_column", opts_null,
                       stack   = "Continent",
                       backend = "ggplot2")
  t <- ggplot2::ggplot_build(fig)$plot$theme
  expect_true(inherits(t$axis.title.y, "element_blank"))
})

# tests/testthat/test-ranked-bar.R
#
# Covers: gg_ranked_bar, hc_ranked_bar, hd_make("ranked_bar", ...)
#
# Dataset: 20 Norwegian municipalities with rate + population count.
# Using a fixed seed-free frame (deterministic values) so tests never flicker.

muni_df <- data.frame(
  municipality = c(
    "Oslo", "Bergen", "Trondheim", "Stavanger", "Kristiansand",
    "Tromsø", "Drammen", "Fredrikstad", "Sandnes", "Sarpsborg",
    "Bodø", "Sandefjord", "Ålesund", "Skien", "Haugesund",
    "Tønsberg", "Moss", "Porsgrunn", "Arendal", "Hamar"
  ),
  rate = c(
    112.4,  98.7,  87.3, 104.1,  76.5,
     91.2,  83.6,  79.8,  95.3,  68.2,
     88.9,  72.4,  81.1,  65.7,  93.8,
     70.3,  74.6,  61.9,  78.2,  85.5
  ),
  pop_count = c(
    693494, 285911, 205849, 144475,  96172,
     77983,  98088,  83193,  84025,  60817,
     52678,  47064,  47160,  52520,  46890,
     56732,  51240,  35087,  46031,  31345
  ),
  stringsAsFactors = FALSE
)

# ── Shared spec/opts ──────────────────────────────────────────────────────────

spec_n  <- hd_spec(muni_df, x = "municipality", y = "rate", n = "pop_count")
spec_no_n <- hd_spec(muni_df, x = "municipality", y = "rate")
opts    <- hd_opts(title = "Rate by municipality", ylab = "Rate per 100 000")


# ═══════════════════════════════════════════════════════════════════════════════
# 1. Return types
# ═══════════════════════════════════════════════════════════════════════════════

test_that("gg ranked_bar returns ggplot", {
  fig <- hd_make(spec_n, "ranked_bar", opts, backend = "ggplot2")
  expect_true(ggplot2::is_ggplot(fig))
})

test_that("hc ranked_bar returns highchart", {
  fig <- hd_make(spec_n, "ranked_bar", opts, backend = "highcharter")
  expect_true(highcharter::is.highchart(fig))
})

test_that("gg ranked_bar works without spec$n", {
  fig <- hd_make(spec_no_n, "ranked_bar", opts, backend = "ggplot2")
  expect_true(ggplot2::is_ggplot(fig))
})

test_that("hc ranked_bar works without spec$n", {
  fig <- hd_make(spec_no_n, "ranked_bar", opts, backend = "highcharter")
  expect_true(highcharter::is.highchart(fig))
})


# ═══════════════════════════════════════════════════════════════════════════════
# 2. Sorting — gg backend
#    gg_ranked_bar builds a sorted factor on .xname; the factor levels encode
#    the sort order. We inspect them directly from the layer data.
# ═══════════════════════════════════════════════════════════════════════════════

.gg_xlevels <- function(fig) {
  # Find the geom_bar layer and extract .xname factor levels from its data
  bar_idx <- which(vapply(fig$layers, function(l)
    inherits(l$geom, "GeomBar"), logical(1)))
  stopifnot(length(bar_idx) == 1L)
  levels(fig$layers[[bar_idx]]$data$.xname)
}

test_that("gg: ascending = TRUE puts lowest rate first in factor levels", {
  fig    <- hd_make(spec_n, "ranked_bar", opts,
                    backend = "ggplot2", ascending = TRUE)
  lvls   <- .gg_xlevels(fig)
  # In ggplot2 with coord_flip the first factor level = bottom bar = lowest value
  # Porsgrunn has the lowest rate (61.9)
  expect_equal(lvls[1], "Porsgrunn (N=35087)")
  # Oslo has the highest rate (112.4)
  expect_equal(lvls[length(lvls)], "Oslo (N=693494)")
})

test_that("gg: ascending = FALSE puts highest rate first in factor levels", {
  fig  <- hd_make(spec_n, "ranked_bar", opts,
                  backend = "ggplot2", ascending = FALSE)
  lvls <- .gg_xlevels(fig)
  expect_equal(lvls[1], "Oslo (N=693494)")
  expect_equal(lvls[length(lvls)], "Porsgrunn (N=35087)")
})

test_that("gg: default ascending is TRUE", {
  fig_default <- hd_make(spec_n, "ranked_bar", opts, backend = "ggplot2")
  fig_asc     <- hd_make(spec_n, "ranked_bar", opts,
                          backend = "ggplot2", ascending = TRUE)
  expect_equal(.gg_xlevels(fig_default), .gg_xlevels(fig_asc))
})

test_that("gg: all 20 municipalities present in factor levels", {
  fig  <- hd_make(spec_n, "ranked_bar", opts, backend = "ggplot2")
  lvls <- .gg_xlevels(fig)
  expect_length(lvls, 20L)
})


# ═══════════════════════════════════════════════════════════════════════════════
# 3. N= labels in x axis — gg backend
# ═══════════════════════════════════════════════════════════════════════════════

test_that("gg: spec$n appends (N=...) to x labels", {
  fig  <- hd_make(spec_n, "ranked_bar", opts, backend = "ggplot2")
  lvls <- .gg_xlevels(fig)
  # Every level should contain "(N="
  expect_true(all(grepl("(N=", lvls, fixed = TRUE)))
})

test_that("gg: without spec$n, x labels are plain municipality names", {
  fig  <- hd_make(spec_no_n, "ranked_bar", opts, backend = "ggplot2")
  lvls <- .gg_xlevels(fig)
  expect_false(any(grepl("(N=", lvls, fixed = TRUE)))
  expect_true("Oslo" %in% lvls)
})


# ═══════════════════════════════════════════════════════════════════════════════
# 4. Comparison highlight — gg backend
#    When vs is set, a .is_vs column drives fill; the fill scale maps
#    TRUE → col2 and FALSE → col1.
# ═══════════════════════════════════════════════════════════════════════════════


test_that("gg: vs adds .is_vs column to bar layer data", {
  fig     <- hd_make(spec_n, "ranked_bar", opts,
                     backend = "ggplot2", vs = "Oslo")
  bar_idx <- which(vapply(fig$layers, function(l)
    inherits(l$geom, "GeomBar"), logical(1)))
  bar_data <- fig$layers[[bar_idx]]$data
  expect_true(".is_vs" %in% names(bar_data))
  # Exactly one row should be marked as the comparison bar
  expect_equal(sum(bar_data$.is_vs), 1L)
})

test_that("gg: vs = NULL produces no .is_vs column", {
  fig     <- hd_make(spec_n, "ranked_bar", opts, backend = "ggplot2")
  bar_idx <- which(vapply(fig$layers, function(l)
    inherits(l$geom, "GeomBar"), logical(1)))
  bar_data <- fig$layers[[bar_idx]]$data
  expect_false(".is_vs" %in% names(bar_data))
})

test_that("gg: vs produces two distinct fill colours in built plot", {
  # Test via ggplot_build() — avoids ggplot2 internal class names entirely.
  # Two fill colours means the highlight scale was applied correctly.
  fig     <- hd_make(spec_n, "ranked_bar", opts,
                     backend = "ggplot2", vs = "Oslo")
  built   <- ggplot2::ggplot_build(fig)
  bar_idx <- which(vapply(fig$layers, function(l)
    inherits(l$geom, "GeomBar"), logical(1)))
  fill_vals <- unique(built$data[[bar_idx]]$fill)
  expect_length(fill_vals, 2L)
})

test_that("gg: vs = NULL produces a single fill colour in built plot", {
  fig     <- hd_make(spec_n, "ranked_bar", opts, backend = "ggplot2")
  built   <- ggplot2::ggplot_build(fig)
  bar_idx <- which(vapply(fig$layers, function(l)
    inherits(l$geom, "GeomBar"), logical(1)))
  fill_vals <- unique(built$data[[bar_idx]]$fill)
  expect_length(fill_vals, 1L)
})

test_that("gg: partial vs match works (grepl, fixed = TRUE)", {
  # "Oslo" matches "Oslo" exactly — but also test a substring match
  fig      <- hd_make(spec_n, "ranked_bar", opts,
                      backend = "ggplot2", vs = "Ber")   # matches Bergen
  bar_idx  <- which(vapply(fig$layers, function(l)
    inherits(l$geom, "GeomBar"), logical(1)))
  bar_data <- fig$layers[[bar_idx]]$data
  vs_rows <- bar_data[bar_data$.is_vs, ]
  expect_equal(nrow(vs_rows), 1L)
  expect_true(grepl("Bergen", vs_rows$.xname[1], fixed = TRUE))
})


# ═══════════════════════════════════════════════════════════════════════════════
# 5. Aim line — gg backend
# ═══════════════════════════════════════════════════════════════════════════════

## test_that("gg: aim adds a GeomHline layer", {                     ##
##   fig <- hd_make(spec_n, "ranked_bar", opts,                      ##
##                  backend = "ggplot2", aim = 80)                   ##
##   hline_layers <- Filter(function(l)                              ##
##     inherits(l$geom, "GeomHline"), fig$layers)                    ##
##   expect_length(hline_layers, 1L)                                 ##
## })                                                                ##
##                                                                   ##
## test_that("gg: aim line sits at the correct yintercept", {        ##
##   aim_val   <- 80                                                 ##
##   fig       <- hd_make(spec_n, "ranked_bar", opts,                ##
##                        backend = "ggplot2", aim = aim_val)        ##
##   built     <- ggplot2::ggplot_build(fig)                         ##
##   hline_idx <- which(vapply(fig$layers, function(l)               ##
##     inherits(l$geom, "GeomHline"), logical(1)))                   ##
##   yint <- unique(built$data[[hline_idx]]$yintercept)              ##
##   expect_equal(yint, aim_val)                                     ##
## })                                                                ##
##                                                                   ##
## test_that("gg: aim = NULL adds no GeomHline layer", {             ##
##   fig <- hd_make(spec_n, "ranked_bar", opts, backend = "ggplot2") ##
##   hline_layers <- Filter(function(l)                              ##
##     inherits(l$geom, "GeomHline"), fig$layers)                    ##
##   expect_length(hline_layers, 0L)                                 ##
## })                                                                ##


# ═══════════════════════════════════════════════════════════════════════════════
# 6. Flip — gg backend
#    After the fix, gg_ranked_bar appends coord_flip() as a layer when
#    do_flip = TRUE, and base_fig is prevented from adding a second one.
# ═══════════════════════════════════════════════════════════════════════════════

test_that("gg: flip = TRUE (default) adds CoordFlip", {
  optsFlip    <- hd_opts(title = "Rate by municipality", ylab = "Rate per 100 000", flip = TRUE)
  fig <- hd_make(spec_n, "ranked_bar", optsFlip, backend = "ggplot2")
  expect_true(inherits(fig$coordinates, "CoordFlip"))
})

test_that("gg: flip = FALSE does not add CoordFlip", {
  optsFlip    <- hd_opts(title = "Rate by municipality", ylab = "Rate per 100 000", flip = FALSE)
  fig <- hd_make(spec_n, "ranked_bar", optsFlip,
                 backend = "ggplot2")
  expect_false(inherits(fig$coordinates, "CoordFlip"))
})


# ═══════════════════════════════════════════════════════════════════════════════
# 7. spec$data is never mutated
#    gg_ranked_bar must work on a local copy of spec$data.
# ═══════════════════════════════════════════════════════════════════════════════

test_that("gg: spec$data columns are unchanged after hd_make", {
  cols_before <- names(spec_n$data)
  hd_make(spec_n, "ranked_bar", opts, backend = "ggplot2",
          vs = "Oslo", aim = 80)
  expect_equal(names(spec_n$data), cols_before)
})

test_that("hc: spec$data columns are unchanged after hd_make", {
  cols_before <- names(spec_n$data)
  hd_make(spec_n, "ranked_bar", opts, backend = "highcharter")
  expect_equal(names(spec_n$data), cols_before)
})


# ═══════════════════════════════════════════════════════════════════════════════
# 8. HC backend — series structure
# ═══════════════════════════════════════════════════════════════════════════════

.hc_series <- function(fig) fig$x$hc_opts$series

test_that("hc: produces exactly one series", {
  fig <- hd_make(spec_n, "ranked_bar", opts, backend = "highcharter")
  expect_length(.hc_series(fig), 1L)
})

test_that("hc: series type is bar", {
  opts    <- hd_opts(title = "Rate by municipality", ylab = "Rate per 100 000", flip = FALSE)
  fig <- hd_make(spec_n, "ranked_bar", opts, backend = "highcharter")
  expect_equal(.hc_series(fig)[[1]]$type, "column")
})

test_that("hc: series type is bar", {
  opts    <- hd_opts(title = "Rate by municipality", ylab = "Rate per 100 000", flip = TRUE)
  fig <- hd_make(spec_n, "ranked_bar", opts, backend = "highcharter")
  expect_equal(.hc_series(fig)[[1]]$type, "column")
})

test_that("hc: series has one data point per row", {
  fig <- hd_make(spec_n, "ranked_bar", opts, backend = "highcharter")
  expect_length(.hc_series(fig)[[1]]$data, nrow(muni_df))
})

test_that("hc: each point has name, y, and color fields", {
  fig    <- hd_make(spec_n, "ranked_bar", opts, backend = "highcharter")
  points <- .hc_series(fig)[[1]]$data
  has_all <- vapply(points, function(p)
    all(c("name", "y", "color") %in% names(p)), logical(1))
  expect_true(all(has_all))
})

test_that("hc: point y values match original rate column (all present)", {
  fig    <- hd_make(spec_n, "ranked_bar", opts, backend = "highcharter")
  points <- .hc_series(fig)[[1]]$data
  y_vals <- vapply(points, `[[`, numeric(1), "y")
  expect_setequal(y_vals, muni_df$rate)
})


# ═══════════════════════════════════════════════════════════════════════════════
# 9. HC backend — sorting
# ═══════════════════════════════════════════════════════════════════════════════

test_that("hc: ascending = TRUE sorts lowest rate first", {
  fig    <- hd_make(spec_n, "ranked_bar", opts,
                    backend = "highcharter", ascending = TRUE)
  points <- .hc_series(fig)[[1]]$data
  y_vals <- vapply(points, `[[`, numeric(1), "y")
  expect_equal(y_vals, sort(muni_df$rate, decreasing = TRUE))
})

test_that("hc: ascending = FALSE sorts highest rate first", {
  fig    <- hd_make(spec_n, "ranked_bar", opts,
                    backend = "highcharter", ascending = FALSE)
  points <- .hc_series(fig)[[1]]$data
  y_vals <- vapply(points, `[[`, numeric(1), "y")
  expect_equal(y_vals, sort(muni_df$rate, decreasing = FALSE))
})


# ═══════════════════════════════════════════════════════════════════════════════
# 10. HC backend — N in point data (tooltip)
#     After the label-cleanup change: N lives in point$n_obs, not in the name.
# ═══════════════════════════════════════════════════════════════════════════════

test_that("hc: with spec$n, each point has n_obs field", {
  fig    <- hd_make(spec_n, "ranked_bar", opts, backend = "highcharter")
  points <- .hc_series(fig)[[1]]$data
  has_n  <- vapply(points, function(p) "n_obs" %in% names(p), logical(1))
  expect_true(all(has_n))
})

test_that("hc: with spec$n, n_obs values match pop_count column", {
  fig    <- hd_make(spec_n, "ranked_bar", opts, backend = "highcharter")
  points <- .hc_series(fig)[[1]]$data
  n_vals <- vapply(points, `[[`, numeric(1), "n_obs")
  expect_setequal(n_vals, muni_df$pop_count)
})

test_that("hc: without spec$n, points have no n_obs field", {
  fig    <- hd_make(spec_no_n, "ranked_bar", opts, backend = "highcharter")
  points <- .hc_series(fig)[[1]]$data
  has_n  <- vapply(points, function(p) "n_obs" %in% names(p), logical(1))
  expect_false(any(has_n))
})

test_that("hc: with spec$n, point names do NOT contain '(N='", {
  # After the label-cleanup change, N is in n_obs only — not in the name field
  fig    <- hd_make(spec_n, "ranked_bar", opts, backend = "highcharter")
  points <- .hc_series(fig)[[1]]$data
  names  <- vapply(points, `[[`, character(1), "name")
  expect_false(any(grepl("(N=", names, fixed = TRUE)))
})


# ═══════════════════════════════════════════════════════════════════════════════
# 11. HC backend — comparison highlight colours
# ═══════════════════════════════════════════════════════════════════════════════

test_that("hc: vs highlights exactly one bar with a different colour", {
  fig    <- hd_make(spec_n, "ranked_bar", opts,
                    backend = "highcharter", vs = "Oslo")
  points <- .hc_series(fig)[[1]]$data
  colors <- vapply(points, `[[`, character(1), "color")
  # Exactly one point should have the highlight (col2) colour
  n_unique <- length(unique(colors))
  expect_equal(n_unique, 2L)
  # Exactly one point gets the minority colour
  minority_col <- names(sort(table(colors)))[1]
  expect_equal(sum(colors == minority_col), 1L)
})

test_that("hc: vs = NULL uses a single colour for all bars", {
  fig    <- hd_make(spec_n, "ranked_bar", opts, backend = "highcharter")
  points <- .hc_series(fig)[[1]]$data
  colors <- vapply(points, `[[`, character(1), "color")
  expect_length(unique(colors), 1L)
})

test_that("hc: highlighted point name matches vs string", {
  fig    <- hd_make(spec_n, "ranked_bar", opts,
                    backend = "highcharter", vs = "Bergen")
  points <- .hc_series(fig)[[1]]$data
  colors <- vapply(points, `[[`, character(1), "color")
  names  <- vapply(points, `[[`, character(1), "name")
  minority_col  <- names(sort(table(colors)))[1]
  highlighted   <- names[colors == minority_col]
  expect_true(grepl("Bergen", highlighted, fixed = TRUE))
})


# ═══════════════════════════════════════════════════════════════════════════════
# 12. HC backend — aim line
# ═══════════════════════════════════════════════════════════════════════════════

.hc_yaxis <- function(fig) fig$x$hc_opts$yAxis

test_that("hc: aim adds a plotLines entry to yAxis", {
  fig   <- hd_make(spec_n, "ranked_bar", opts,
                   backend = "highcharter", aim = 80)
  yaxis <- .hc_yaxis(fig)
  plot_lines <- yaxis$plotLines %||% yaxis[[1]]$plotLines
  expect_true(!is.null(plot_lines) && length(plot_lines) > 0)
})

test_that("hc: aim plotLine value matches the aim argument", {
  aim_val   <- 80
  fig       <- hd_make(spec_n, "ranked_bar", opts,
                       backend = "highcharter", aim = aim_val)
  yaxis     <- .hc_yaxis(fig)
  plot_lines <- yaxis$plotLines %||% yaxis[[1]]$plotLines
  line_values <- vapply(plot_lines, `[[`, numeric(1), "value")
  expect_true(aim_val %in% line_values)
})

test_that("hc: aim = NULL adds no plotLines", {
  fig   <- hd_make(spec_n, "ranked_bar", opts, backend = "highcharter")
  yaxis <- .hc_yaxis(fig)
  plot_lines <- yaxis$plotLines %||% yaxis[[1]]$plotLines
  expect_true(is.null(plot_lines) || length(plot_lines) == 0)
})


# ═══════════════════════════════════════════════════════════════════════════════
# 13. HC backend — tooltip
#     After the label-cleanup: hc_ranked_bar sets its own hc_tooltip().
# ═══════════════════════════════════════════════════════════════════════════════

.hc_tooltip <- function(fig) fig$x$hc_opts$tooltip

test_that("hc: with spec$n, tooltip pointFormat contains {point.n_obs}", {
  fig <- hd_make(spec_n, "ranked_bar", opts, backend = "highcharter")
  tt  <- .hc_tooltip(fig)
  expect_true(grepl("point.n_obs", tt$pointFormat, fixed = TRUE))
})

test_that("hc: without spec$n, tooltip pointFormat has no {point.n_obs}", {
  fig <- hd_make(spec_no_n, "ranked_bar", opts, backend = "highcharter")
  tt  <- .hc_tooltip(fig)
  expect_false(grepl("point.n_obs", tt$pointFormat, fixed = TRUE))
})

test_that("hc: tooltip pointFormat contains {point.y}", {
  fig <- hd_make(spec_n, "ranked_bar", opts, backend = "highcharter")
  tt  <- .hc_tooltip(fig)
  expect_true(grepl("point.y", tt$pointFormat, fixed = TRUE))
})

test_that("hc: dataLabels are disabled on the series", {
  fig    <- hd_make(spec_n, "ranked_bar", opts, backend = "highcharter")
  series <- .hc_series(fig)[[1]]
  dl_enabled <- series$dataLabels$enabled %||% TRUE   # default in HC is TRUE
  expect_false(isTRUE(dl_enabled))
})


# ═══════════════════════════════════════════════════════════════════════════════
# 14. grepl NULL guard — the comp = NULL bug must not regress
# ═══════════════════════════════════════════════════════════════════════════════

test_that("hc: vs = NULL does not error (grepl NULL guard)", {
  expect_no_error(
    hd_make(spec_n, "ranked_bar", opts, backend = "highcharter", vs = NULL)
  )
})

test_that("gg: vs = NULL does not error (grepl NULL guard)", {
  expect_no_error(
    hd_make(spec_n, "ranked_bar", opts, backend = "ggplot2", vs = NULL)
  )
})

test_that("hc: vs = '' (empty string) does not error", {
  expect_no_error(
    hd_make(spec_n, "ranked_bar", opts, backend = "highcharter", vs = "")
  )
})

test_that("gg: vs = '' (empty string) does not error", {
  expect_no_error(
    hd_make(spec_n, "ranked_bar", opts, backend = "ggplot2", vs = "")
  )
})

test_that("hc: vs that matches nothing is silently ignored", {
  fig    <- hd_make(spec_n, "ranked_bar", opts,
                    backend = "highcharter", vs = "XXXXXX")
  points <- .hc_series(fig)[[1]]$data
  colors <- vapply(points, `[[`, character(1), "color")
  # No highlight → single colour
  expect_length(unique(colors), 1L)
})


# ═══════════════════════════════════════════════════════════════════════════════
# 15. Label placement helpers (gg only)
#     ypos logic: inside labels (ypos = 1) get hjust = 1.5 / white text;
#     outside labels (ypos = 0) get hjust = -0.5 / dark text.
# ═══════════════════════════════════════════════════════════════════════════════

test_that("gg: at least one GeomText layer is present", {
  fig  <- hd_make(spec_n, "ranked_bar", opts, backend = "ggplot2")
  text_layers <- Filter(function(l)
    inherits(l$geom, "GeomText"), fig$layers)
  expect_gte(length(text_layers), 1L)
})

test_that("gg: inside label layer uses white text colour", {
  fig  <- hd_make(spec_n, "ranked_bar", opts, backend = "ggplot2")
  text_layers <- Filter(function(l)
    inherits(l$geom, "GeomText"), fig$layers)
  # At least one layer should use white (#FFFFFF) — the inside label layer
  colours <- vapply(text_layers, function(l)
    l$aes_params$colour %||% "", character(1))
  expect_true(any(colours == "#FFFFFF"))
})

## test_that("gg: outside label layer uses dark text colour", {       ##
##   fig  <- hd_make(spec_n, "ranked_bar", opts, backend = "ggplot2") ##
##   text_layers <- Filter(function(l)                                ##
##     inherits(l$geom, "GeomText"), fig$layers)                      ##
##   colours <- vapply(text_layers, function(l)                       ##
##     l$aes_params$colour %||% "", character(1))                     ##
##   expect_true(any(colours == "#555555"))                           ##
## })                                                                 ##


# ═══════════════════════════════════════════════════════════════════════════════
# 16. xAxis categories — HC backend
# ═══════════════════════════════════════════════════════════════════════════════

test_that("hc: xAxis categories length matches number of rows", {
  fig  <- hd_make(spec_n, "ranked_bar", opts, backend = "highcharter")
  cats <- fig$x$hc_opts$xAxis$categories
  expect_length(cats, nrow(muni_df))
})

test_that("hc: xAxis categories are plain names (no N= suffix)", {
  fig  <- hd_make(spec_n, "ranked_bar", opts, backend = "highcharter")
  cats <- fig$x$hc_opts$xAxis$categories
  # After the label-cleanup: N is in n_obs only — axis labels must be plain
  expect_false(any(grepl("(N=", cats, fixed = TRUE)))
})

test_that("hc: xAxis categories contain all municipality names", {
  fig  <- hd_make(spec_n, "ranked_bar", opts, backend = "highcharter")
  cats <- fig$x$hc_opts$xAxis$categories
  expect_setequal(cats, muni_df$municipality)
})

# tests/testthat/test-rounding-helpers.R

test_that("round_column() - happy path rounds numeric column and returns data.frame", {
  df <- data.frame(a = c(1.234, 2.345, 3.456), b = 1:3)
  out <- round_column(df, "a", 1)

  expect_s3_class(out, "data.frame")
  expect_equal(out$a, round(df$a, 1))
  # b should be untouched
  expect_identical(out$b, df$b)
})

test_that("round_column() - accepts digits as numeric-like character and integerizes", {
  df <- data.frame(a = c(1.234, 2.345, 3.456))
  out <- round_column(df, "a", "2")   # character "2" should be accepted

  expect_equal(out$a, round(df$a, 2))
})

test_that("round_column() - digits length > 1 or NA should error", {
  df <- data.frame(a = c(1.234, 2.345, 3.456))

  expect_error(round_column(df, "a", c(1, 2)), "`digits` must be a single non-NA numeric value.")
  expect_error(round_column(df, "a", NA), "`digits` must be a single non-NA numeric value.")
})

test_that("round_column() - non-numeric digits should error after coercion", {
  df <- data.frame(a = c(1.234, 2.345, 3.456))

  expect_error(round_column(df, "a", "two"), "`digits` must be numeric.")
})

test_that("round_column() - errors if data is not a data.frame", {
  not_df <- list(a = c(1.2, 3.4))
  expect_error(round_column(not_df, "a", 1), "`data` must be a data.frame.")
})

test_that("round_column() - errors if column does not exist", {
  df <- data.frame(a = c(1.234, 2.345, 3.456))
  expect_error(round_column(df, "missing", 1), "Column 'missing' does not exist in the dataset.")
})

test_that("round_column() - errors if column is not numeric", {
  df <- data.frame(a = c("x", "y", "z"))
  expect_error(
    round_column(df, "a", 1),
    "Column 'a' is not numeric"
  )
})

test_that("round_column() - respects R's round-to-even behavior on ties", {
  # R's round() uses "banker's rounding" (ties to even)
  df <- data.frame(a = c(2.05, 2.15), b = c(2.5, 3.5))
  out1 <- round_column(df, "a", 1)
  # 2.05 -> 2.0; 2.15 -> 2.1
  expect_equal(out1$a, c(2.0, 2.1))

  out2 <- round_column(df, "b", 0)
  expect_equal(out2$b, c(2, 4))
})

test_that("round_column() - preserves NA values", {
  df <- data.frame(a = c(1.234, NA_real_, 3.456))
  out <- round_column(df, "a", 1)
  expect_true(is.na(out$a[2]))
  expect_equal(out$a[c(1,3)], round(df$a[c(1,3)], 1))
})

# ------------------------------------------------------------------------------

test_that("check_decimals() - returns spec unchanged if opts$decimals is NULL", {
  spec <- list(
    data = data.frame(x = 1:3, y = c(1.2, 2.3, 3.4)),
    y = "y"
  )
  opts <- list(decimals = NULL)

  out <- check_decimals(spec, opts, type = "line", extra_args = list())
  expect_identical(out, spec)  # unchanged
})

test_that("check_decimals() - rounds spec$data[[spec$y]] when opts$decimals is set", {
  spec <- list(
    data = data.frame(x = 1:3, y = c(1.234, 2.345, 3.456)),
    y = "y"
  )
  opts <- list(decimals = 1)

  out <- check_decimals(spec, opts, type = "line", extra_args = list())
  expect_equal(out$data$y, round(spec$data$y, 1))
})

test_that("check_decimals() - digits can be numeric-like character", {
  spec <- list(
    data = data.frame(x = 1:3, y = c(1.234, 2.345, 3.456)),
    y = "y"
  )
  opts <- list(decimals = "2")

  out <- check_decimals(spec, opts, type = "line", extra_args = list())
  expect_equal(out$data$y, round(spec$data$y, 2))
})

test_that("check_decimals() - arearange: also rounds ymin and ymax when decimals set", {
  spec <- list(
    data = data.frame(
      x = 1:3,
      mid = c(10.123, 20.234, 30.345),
      ymin = c(9.111, 19.222, 29.333),
      ymax = c(11.111, 21.222, 31.333)
    ),
    y = "mid"
  )
  opts <- list(decimals = 1)
  extra_args <- list(ymin = "ymin", ymax = "ymax")

  out <- check_decimals(spec, opts, type = "arearange", extra_args = extra_args)

  expect_equal(out$data$mid,  round(spec$data$mid,  1))
  expect_equal(out$data$ymin, round(spec$data$ymin, 1))
  expect_equal(out$data$ymax, round(spec$data$ymax, 1))
})

test_that("check_decimals() - arearange: leaves ymin/ymax if decimals is NULL", {
  spec <- list(
    data = data.frame(
      x = 1:3,
      mid = c(10.123, 20.234, 30.345),
      ymin = c(9.111, 19.222, 29.333),
      ymax = c(11.111, 21.222, 31.333)
    ),
    y = "mid"
  )
  opts <- list(decimals = NULL)
  extra_args <- list(ymin = "ymin", ymax = "ymax")

  out <- check_decimals(spec, opts, type = "arearange", extra_args = extra_args)

  expect_identical(out$data, spec$data)
})

test_that("check_decimals() - errors propagate from round_column", {
  # Non-numeric y column triggers error from round_column
  spec <- list(
    data = data.frame(y = c("a", "b")),
    y = "y"
  )
  opts <- list(decimals = 1)

  expect_error(check_decimals(spec, opts, type = "line", extra_args = list()),
               "is not numeric")
})

test_that("check_decimals() - gracefully handles missing ymin/ymax names in arearange", {
  # If y-min/y-max columns are absent, round_column should error. We test that such
  # errors are surfaced (since your function delegates to round_column).
  spec <- list(
    data = data.frame(
      x = 1:3,
      mid = c(10.123, 20.234, 30.345)
      # no ymin/ymax columns
    ),
    y = "mid"
  )
  opts <- list(decimals = 1)
  extra_args <- list(ymin = "ymin", ymax = "ymax")

  expect_error(
    check_decimals(spec, opts, type = "arearange", extra_args = extra_args),
    "does not exist in the dataset"
  )
})

spec <- hd_spec(survey_df, "age", "pct", group = "sex", n = "n")
opts <- hd_opts(title = "Test chart", ylim = c(0, 80))

# ── Input validation ──────────────────────────────────────────────────────────

test_that("hd_make: rejects non-hd_spec", {
  expect_error(hd_make(list(), "column"), "hd_spec")
})

test_that("hd_make: NULL opts uses defaults silently", {
  fig <- hd_make(spec, "column", opts = NULL)
  expect_true(is_highchart(fig))
})

test_that("hd_make: unknown geometry errors", {
  expect_error(hd_make(spec, "violin", opts), "Unknown geometry")
})

test_that("hd_make: unknown backend errors", {
  expect_error(hd_make(spec, "column", opts, backend = "plotly"),
               "Unknown backend")
})

test_that("hd_make: arearange requires ymin + ymax", {
  expect_error(hd_make(spec, "arearange", opts), "Missing required")
})

# ── highcharter backend ───────────────────────────────────────────────────────

test_that("hd_make: HC column returns highchart", {
  fig <- hd_make(spec, "column", opts)
  expect_true(is_highchart(fig))
})

test_that("hd_make: HC line (smooth) returns highchart", {
  fig <- hd_make(spec, "line", opts, smooth = TRUE)
  expect_true(is_highchart(fig))
})

test_that("hd_make: HC line (straight) returns highchart", {
  fig <- hd_make(spec, "line", opts, smooth = FALSE)
  expect_true(is_highchart(fig))
})

test_that("hd_make: HC scatter returns highchart", {
  s2  <- hd_spec(survey_df, "pct", "n")
  fig <- hd_make(s2, "scatter", opts)
  expect_true(is_highchart(fig))
})

test_that("hd_make: HC arearange returns highchart", {
  s2  <- hd_spec(survey_df, "age", "pct", group = "sex")
  fig <- hd_make(s2, "arearange", opts, ymin = "lo", ymax = "hi")
  expect_true(is_highchart(fig))
})

test_that("hd_make: HC pie (solid) returns highchart", {
  s2  <- hd_spec(pie_df, "category", "value")
  fig <- hd_make(s2, "pie", hd_opts(title = "Pie"))
  expect_true(is_highchart(fig))
})

test_that("hd_make: HC pie (donut) returns highchart", {
  s2  <- hd_spec(pie_df, "category", "value")
  fig <- hd_make(s2, "pie", hd_opts(title = "Donut"),
                  inner_size = "50%")
  expect_true(is_highchart(fig))
})

test_that("hd_make: use_js = FALSE does not break column", {
  fig <- hd_make(spec, "column", opts, use_js = FALSE)
  expect_true(is_highchart(fig))
})

test_that("hd_make: use_js = FALSE does not break line", {
  fig <- hd_make(spec, "line", opts, use_js = FALSE)
  expect_true(is_highchart(fig))
})

test_that("hd_make: use_js = FALSE does not break pie", {
  s2  <- hd_spec(pie_df, "category", "value")
  fig <- hd_make(s2, "pie", hd_opts(), use_js = FALSE)
  expect_true(is_highchart(fig))
})

# ── ggplot2 backend ───────────────────────────────────────────────────────────

test_that("hd_make: gg column returns ggplot", {
  fig <- hd_make(spec, "column", opts, backend = "ggplot2")
  expect_true(is_ggplot(fig))
})

test_that("hd_make: gg line returns ggplot", {
  fig <- hd_make(spec, "line", opts, backend = "ggplot2", smooth = FALSE)
  expect_true(is_ggplot(fig))
})

test_that("hd_make: gg scatter returns ggplot", {
  s2  <- hd_spec(survey_df, "pct", "n")
  fig <- hd_make(s2, "scatter", opts, backend = "ggplot2")
  expect_true(is_ggplot(fig))
})

test_that("hd_make: gg arearange returns ggplot", {
  s2  <- hd_spec(survey_df, "age", "pct", group = "sex")
  fig <- hd_make(s2, "arearange", opts, backend = "ggplot2",
                  ymin = "lo", ymax = "hi")
  expect_true(is_ggplot(fig))
})

test_that("hd_make: gg pie returns ggplot", {
  s2  <- hd_spec(pie_df, "category", "value")
  fig <- hd_make(s2, "pie", hd_opts(title = "Pie"),
                  backend = "ggplot2")
  expect_true(is_ggplot(fig))
})

# ── ggplot2 colors and palettes ──────────────────────────────────────────────────

test_that("gg n=2 uses hdir2 not hdir[1:2]", {
  fig   <- hd_make(spec2, "column", hd_opts(), backend = "ggplot2")
  fills <- .extract_gg_fill_values(fig)
  hdir2 <- get_palette("hdir2")
  expect_equal(sort(unname(fills)), sort(hdir2[1:2]))
  expect_false(identical(sort(unname(fills)),
                          sort(get_palette("hdir")[1:2])))
})

test_that("gg and HC assign same colour to same group", {
  fig_hc <- hd_make(spec2, "column", hd_opts())
  fig_gg <- hd_make(spec2, "column", hd_opts(), backend = "ggplot2")

  hc_male <- fig_hc$x$hc_opts$series[[
    which(vapply(fig_hc$x$hc_opts$series,
                 function(s) s$name == "Male", logical(1)))
  ]]$color

  gg_fills <- .extract_gg_fill_values(fig_gg)
  expect_equal(gg_fills[["Male"]], hc_male)
})

test_that("gg palette name string resolved not passed raw", {
  # Previously crashed with scale_color_manual(values = "hdir")
  expect_s3_class(
    hd_make(spec2, "column", hd_opts(colors = "hdir"), backend = "ggplot2"),
    "ggplot"
  )
})

test_that("gg too-short palette warns and falls back", {
  expect_warning(
    hd_make(spec2, "column", hd_opts(colors = "#FF0000"),
            backend = "ggplot2"),
    "Falling back"
  )
})

# --- ggplot2 with single color ----------------------

test_that("gg single series bar uses hdir[1] colour", {
  spec <- hd_spec(data.frame(year = 2018:2022, val = c(1,2,3,4,5)),
                  "year", "val")
  fig  <- hd_make(spec, "column", hd_opts(), backend = "ggplot2")
  expect_equal(.layer_aes(fig, "GeomCol", "fill"), get_palette("hdir")[1])
})

test_that("gg line single series uses hdir[1]", {
  spec <- hd_spec(data.frame(year = 2018:2022, val = c(1,2,3,4,5)),
                  "year", "val")
  fig  <- hd_make(spec, "line", hd_opts(), backend = "ggplot2")
  expect_equal(.layer_aes(fig, "GeomLine", "colour"), get_palette("hdir")[1])
})

test_that("gg scatter single series uses hdir[1]", {
  spec <- hd_spec(data.frame(year = 2018:2022, val = c(1,2,3,4,5)),
                  "year", "val")
  fig  <- hd_make(spec, "scatter", hd_opts(), backend = "ggplot2")
  expect_equal(.layer_aes(fig, "GeomPoint", "colour"), get_palette("hdir")[1])
})

test_that("gg single series does not add fill or colour scales", {
  spec <- hd_spec(data.frame(year = 2018:2022, val = c(1,2,3,4,5)),
                  "year", "val")
  fig  <- hd_make(spec, "column", hd_opts(), backend = "ggplot2")

  scale_aes <- unlist(lapply(fig$scales$scales, function(s) s$aesthetics))
  expect_false("fill"   %in% scale_aes)
  expect_false("colour" %in% scale_aes)
})

test_that("gg single series explicit colour override respected", {
  spec <- hd_spec(data.frame(year = 2018:2022, val = c(1,2,3,4,5)),
                  "year", "val")
  fig  <- hd_make(spec, "column", hd_opts(colors = "#FF0000"),
                  backend = "ggplot2")

  layer_fill <- .layer_aes(fig, "GeomCol", "colour")
  expect_equal(layer_fill, "#FF0000")
})

test_that("gg multi-series still uses apply_gg_colors", {
  df <- data.frame(
    year = rep(2018:2020, 2),
    val  = c(1,2,3,4,5,6),
    grp  = rep(c("A","B"), each = 3)
  )
  spec <- hd_spec(df, "year", "val", group = "grp")
  fig  <- hd_make(spec, "column", hd_opts(), backend = "ggplot2")

  scale_aes <- unlist(lapply(fig$scales$scales, function(s) s$aesthetics))
  expect_true("fill" %in% scale_aes)
})

# -- ggplot2 multi colors

test_that("gg multi-series column uses hdir2 not grey", {
  df <- data.frame(
    year = rep(2018:2020, 2),
    val  = c(1,2,3,4,5,6),
    grp  = rep(c("A","B"), each = 3)
  )
  spec <- hd_spec(df, "year", "val", group = "grp")
  fig  <- hd_make(spec, "column", hd_opts(), backend = "ggplot2")

  # Fill scale must exist — means apply_gg_colors was called
  scale_aes <- unlist(lapply(fig$scales$scales, function(s) s$aesthetics))
  expect_true("fill" %in% scale_aes)

  # The geom layer must NOT have a fixed fill — it must inherit from mapping
  col_layer_idx <- .find_layers(fig, "GeomCol")
  layer_fill    <- fig$layers[[col_layer_idx[1]]]$aes_params$fill
  expect_null(layer_fill)   # NULL = not fixed, inherits mapped aesthetic
})

test_that("gg single series column has no fill scale", {
  spec <- hd_spec(data.frame(year = 2018:2022, val = c(1,2,3,4,5)),
                  "year", "val")
  fig  <- hd_make(spec, "column", hd_opts(), backend = "ggplot2")

  scale_aes <- unlist(lapply(fig$scales$scales, function(s) s$aesthetics))
  expect_false("fill" %in% (scale_aes %||% character(0)))

  # Fixed colour is on the layer itself
  expect_equal(.layer_aes(fig, "GeomCol", "fill"), get_palette("hdir")[1])
})

# ── hd_opts reuse ────────────────────────────────────────────────────────────

test_that("same spec renders with two different opts", {
  opts_a <- hd_opts(title = "A", ylim = c(0, 60))
  opts_b <- hd_opts(title = "B", flip = TRUE)
  expect_true(is_highchart(hd_make(spec, "column", opts_a)))
  expect_true(is_highchart(hd_make(spec, "column", opts_b)))
})

# ── Per-figure colour override ────────────────────────────────────────────────

test_that("hd_make: per-figure colors in opts work for both backends", {
  o <- hd_opts(colors = c("#FF0000", "#0000FF"))
  expect_true(is_highchart(hd_make(spec, "column", o)))
  expect_true(is_ggplot(hd_make(spec, "column", o, backend = "ggplot2")))
})

# ── Line-specific args ────────────────────────────────────────────────────────

test_that("hd_make: line with custom dot_size and line_symbols", {
  fig <- hd_make(spec, "line", opts,
                  smooth       = FALSE,
                  dot_size     = 8,
                  line_symbols = c("circle", "square"))
  expect_true(is_highchart(fig))
})

# ── No-group specs ────────────────────────────────────────────────────────────

test_that("hd_make: works without group column", {
  s_ng <- hd_spec(survey_df[survey_df$sex == "Male", ],
                   "age", "pct")
  expect_true(is_highchart(hd_make(s_ng, "column", opts)))
  expect_true(is_ggplot(hd_make(s_ng, "column", opts, backend = "ggplot2")))
})

# ── Modules ────────────────────────────────────────────────────────────
test_that("hd_make: modules = TRUE adds accessibility dependency", {
  spec <- hd_spec(data.frame(x = c("A","B"), y = c(1,2)), "x", "y")
  fig  <- hd_make(spec, "column", hd_opts(), module = TRUE)
  deps <- vapply(fig$dependencies, function(d) d$name, character(1))
  expect_true(any(grepl("accessibility", deps, ignore.case = TRUE)))
})

test_that("hd_make: modules = FALSE skips standard dependencies", {
  spec <- hd_spec(data.frame(x = c("A","B"), y = c(1,2)), "x", "y")
  fig  <- hd_make(spec, "column", hd_opts(), module = FALSE)
  deps <- vapply(fig$dependencies, function(d) d$name, character(1))
  expect_false(any(grepl("accessibility", deps, ignore.case = TRUE)))
})

test_that("hd_make: modules ignored silently for ggplot2 backend", {
  spec <- hd_spec(data.frame(x = c("A","B"), y = c(1,2)), "x", "y")
  expect_s3_class(
    hd_make(spec, "column", hd_opts(),
            backend = "ggplot2", module = FALSE),
    "ggplot"
  )
})

## -- Axis labelling --------
test_that("HC: default opts uses column name as y label", {
  spec <- hd_spec(data.frame(x = c("A","B"), rate = c(1,2)), "x", "rate")
  fig  <- hd_make(spec, "column", hd_opts())
  expect_equal(fig$x$hc_opts$yAxis$title$text, "rate")
})

test_that("HC: NULL ylab hides y axis title", {
  spec <- hd_spec(data.frame(x = c("A","B"), rate = c(1,2)), "x", "rate")
  fig  <- hd_make(spec, "column", hd_opts(ylab = NULL))
  expect_equal(fig$x$hc_opts$yAxis$title$text, "")
})

test_that("HC: custom ylab used as axis title", {
  spec <- hd_spec(data.frame(x = c("A","B"), rate = c(1,2)), "x", "rate")
  fig  <- hd_make(spec, "column", hd_opts(ylab = "Rate per 100 000"))
  expect_equal(fig$x$hc_opts$yAxis$title$text, "Rate per 100 000")
})

test_that("gg: NULL ylab applies element_blank to axis.title.y", {
  spec <- hd_spec(data.frame(x = c("A","B"), rate = c(1,2)), "x", "rate")
  fig  <- hd_make(spec, "column", hd_opts(ylab = NULL), backend = "ggplot2")
  expect_s3_class(fig$theme$axis.title.y, "element")
})

# Both registries use a plain environment as a mutable named store.
# Backends and geoms are registered in .onLoad() (zzz.R).

# -- Backend registry ----------------------------------------------------------

#' @keywords internal
.backend_registry <- new.env(parent = emptyenv())

#' Register a Rendering Backend and used when loding in zzz.R file
#' @param name   Character. Unique backend identifier (e.g. `"ggplot2"`).
#' @param engine Function.
#' @return `name`, invisibly.
#' @export
register_backend <- function(name, engine) {

  if (!is.character(name) || length(name) != 1L || !nzchar(name))
    stop("`name` must be a single non-empty string.", call. = FALSE)

  if (!is.function(engine))
    stop("`engine` must be a function.", call. = FALSE)

  .backend_registry[[name]] <- engine
  invisible(name)
}

#' @keywords internal
get_backend <- function(name) .backend_registry[[name]]

#' List Registered Backends
#' @return Character vector of registered backend names.
#' @export
list_backends <- function() sort(ls(.backend_registry))

# -- Geometry registry ---------------------------------------------------------

#' @keywords internal
.geom_registry <- new.env(parent = emptyenv())

#' Register a Geometry
#'
#' Adds a geometry to the registry.  The `optional_args` field is the key
#' mechanism for user discoverability: it is what `geom_args()` prints, what
#' the Shiny app reads to build dynamic UI, and what automated documentation
#' can harvest without parsing source code.
#'
#' Structure of `optional_args`:
#' A named list where each element is itself a list with two fields:
#'   `default` - the value used when the arg is not supplied (may be `NULL`)
#'   `desc`    - a short human-readable description (shown by `geom_args()`)
#'
#' Example:
#' ```r
#' optional_args = list(
#'   smooth   = list(default = TRUE,  desc = "Spline smoothing (logical)"),
#'   dot_size = list(default = 4,     desc = "Marker radius in px (numeric)")
#' )
#' ```
#'
#' @param name            Character. Unique geometry identifier.
#' @param ggplot_fun      Function or `NULL`.
#' @param highcharter_fun Function or `NULL`.
#' @param required_args   Named list of `list(default, desc)`. Args that MUST be supplied via
#'   `...` in `hd_make()`. Validation fails if any are missing.
#' @param optional_args   Named list of `list(default, desc)`. Args that MAY
#'   be supplied and have a sensible default when omitted.  These are purely
#'   informational from the registry's perspective - the geom function applies
#'   the defaults itself via `geom_params$key %||% default`.
#' @param is_map_geom     Logical. Bypasses `base_fig()` in both engines.
#'
#' @return `name`, invisibly.
#' @export
register_geom <- function(name,
                          ggplot_fun      = NULL,
                          highcharter_fun = NULL,
                          required_args   = list(),
                          optional_args   = list(),
                          is_map_geom     = FALSE) {

  # optional_args must be a named list of list(default, desc) entries.
  # Validate structure so mis-registrations fail loudly at load time rather
  # than silently producing wrong output for users.
  if (length(optional_args) > 0) {
    if (is.null(names(optional_args)) || any(!nzchar(names(optional_args))))
      stop("register_geom(): `optional_args` must be a fully named list.",
           call. = FALSE)
    bad <- vapply(optional_args, function(x)
      !is.list(x) || !all(c("default", "desc") %in% names(x)),
      logical(1))
    if (any(bad))
      stop("register_geom(): each entry in `optional_args` must be ",
           "list(default = ..., desc = ...).  Bad entries: ",
           paste(names(optional_args)[bad], collapse = ", "),
           call. = FALSE)
  }

  .geom_registry[[name]] <- list(
    name            = name,             # stored so geom fns can self-identify
    ggplot_fun      = ggplot_fun,
    highcharter_fun = highcharter_fun,
    required_args   = required_args,
    optional_args   = optional_args,    # user-discoverable optional args
    is_map_geom     = is_map_geom
  )
  invisible(name)
}

#' @keywords internal
.get_geom <- function(name) .geom_registry[[name]]

#' List Registered Geometries
#' @return Character vector of registered geometry names.
#' @export
list_geoms <- function() sort(ls(.geom_registry))

# -- geom_args(): user-facing discoverability helper ---------------------------

#' Show Arguments for a Geometry
#'
#' Prints the required and optional `...` arguments accepted by a geometry
#' when used with [hd_make()].  This is the primary discoverability tool
#' for geometry-specific arguments that do not appear in `hd_make()`'s
#' signature.
#'
#' @section Why this exists:
#' `hd_make()` uses `...` for all geometry-specific arguments so its own
#' signature stays clean regardless of how many geometries are registered.
#' The trade-off is that users cannot see available args from `hd_make()`
#' alone.  `geom_args()` solves that: it reads the `required_args` and
#' `optional_args` fields registered for each geometry and presents them
#' in a readable table.
#'
#' @param type Character.  Geometry name, e.g. `"line"`, `"ranked_bar"`.
#'   If `NULL` (default), prints a summary for every registered geometry.
#'
#' @return A data frame of argument metadata, invisibly.  The primary purpose
#'   is the side-effect of printing.
#'
#' @examples
#' geom_args("line")
#' geom_args("ranked_bar")
#' geom_args("arearange")
#' geom_args()           # all registered geometries
#'
#' @export
geom_args <- function(type = NULL) {

  # If no type given, recurse over every registered geometry
  if (is.null(type)) {
    nms <- list_geoms()
    for (nm in nms) geom_args(nm)
    return(invisible(NULL))
  }

  geom <- .get_geom(type)
  if (is.null(geom))
    stop("Unknown geometry '", type, "'. See list_geoms().", call. = FALSE)

  # -- Build a data frame with one row per argument ----------------------------
  rows <- list()

  # Required args: have a default, may be omitted
  for (arg in names(geom$required_args)) {
    entry   <- geom$required_args[[arg]]
    default <- if (is.null(entry$default)) "NULL"
               else as.character(entry$default)
    rows[[length(rows) + 1L]] <- data.frame(
      argument = arg,
      kind     = "required",
      default  = default,
      desc     = entry$desc,
      stringsAsFactors = FALSE
    )
  }

  # Optional args: have a default, may be omitted
  for (arg in names(geom$optional_args)) {
    entry   <- geom$optional_args[[arg]]
    default <- if (is.null(entry$default)) "NULL"
               else as.character(entry$default)
    rows[[length(rows) + 1L]] <- data.frame(
      argument = arg,
      kind     = "optional",
      default  = default,
      desc     = entry$desc,
      stringsAsFactors = FALSE
    )
  }

  if (length(rows) == 0L) {
    message("geom '", type, "' has no extra arguments.")
    return(invisible(data.frame()))
  }

  out <- do.call(rbind, rows)

  # -- Print a readable table --------------------------------------------------
  cat(sprintf("\nArguments for hd_make(..., type = \"%s\", ...):\n\n", type))

  # Compute column widths for alignment
  w_arg  <- max(nchar("argument"), nchar(out$argument))
  w_kind <- max(nchar("kind"),     nchar(out$kind))
  w_def  <- max(nchar("default"),  nchar(out$default))

  fmt <- paste0("  %-", w_arg, "s  %-", w_kind, "s  %-", w_def, "s  %s\n")
  cat(sprintf(fmt, "argument", "kind", "default", "description"))
  cat(sprintf(fmt,
              strrep("-", w_arg),
              strrep("-", w_kind),
              strrep("-", w_def),
              strrep("-", 30)))

  for (i in seq_len(nrow(out))) {
    cat(sprintf(fmt, out$argument[i], out$kind[i],
                out$default[i], out$desc[i]))
  }
  cat("\n")

  invisible(out)
}

# tests/testthat/test-registry.R
#
# Covers every public and internal function in R/registry.R:
#
#   Backend registry   — register_backend(), get_backend(), list_backends()
#   Geometry registry  — register_geom(), .get_geom(), list_geoms()
#   Arg validation     — register_geom() structure checks (optional_args)
#   Discoverability    — geom_args()
#   validate_geom_args — in validate.R, driven by required_args named-list
#
# REQUIRED_ARGS STRUCTURE (current design):
#   required_args is a named list where each entry is list(default, desc),
#   matching the optional_args structure.  names(required_args) gives the
#   arg names used by validate_geom_args() and mod_data's req_args reactive.
#
# INTERNAL FUNCTIONS:
#   .get_geom() and geom_args() are not exported; accessed via highdir:::.

# -- Helpers ------------------------------------------------------------------

# Minimal dummy functions satisfying the engine contract
.dummy_gg <- function(spec, opts, geom_params, ...) list()
.dummy_hc <- function(chart, spec, opts, geom_params, ...) chart

# Minimal valid optional_args entry
.opt <- function(default = TRUE, desc = "A test arg") {
  list(default = default, desc = desc)
}

# Minimal valid required_args entry
.req <- function(desc = "A required column") {
  list(default = NULL, desc = desc)
}


# =============================================================================
# 1.  Backend registry
# =============================================================================

test_that("built-in backends are registered at package load", {
  expect_true("ggplot2"     %in% list_backends())
  expect_true("highcharter" %in% list_backends())
})

test_that("list_backends returns a sorted character vector", {
  b <- list_backends()
  expect_type(b, "character")
  expect_equal(b, sort(b))
})

test_that("register_backend: round-trip with get_backend", {
  eng <- function(spec, geom, opts, geom_params, use_js, ...) NULL
  on.exit(rm("test_engine", envir = highdir:::.backend_registry), add = TRUE)
  register_backend("test_engine", eng)

  expect_true("test_engine" %in% list_backends())
  expect_identical(get_backend("test_engine"), eng)
})

test_that("register_backend: overwrites an existing backend silently", {
  eng1 <- function(...) "v1"
  eng2 <- function(...) "v2"
  on.exit(rm("overwrite_test", envir = highdir:::.backend_registry), add = TRUE)
  register_backend("overwrite_test", eng1)

  register_backend("overwrite_test", eng2)
  expect_identical(get_backend("overwrite_test"), eng2)
})

test_that("register_backend: returns name invisibly", {
  eng    <- function(...) NULL
  on.exit(rm("invisible_test", envir = highdir:::.backend_registry), add = TRUE)
  result <- register_backend("invisible_test", eng)

  expect_equal(result, "invisible_test")
})

test_that("register_backend: rejects non-function engine", {
  expect_error(register_backend("bad", "a string"), "function")
  expect_error(register_backend("bad", 42),         "function")
  expect_error(register_backend("bad", NULL),        "function")
  expect_error(register_backend("bad", list()),      "function")
})

test_that("register_backend: rejects empty name", {
  expect_error(register_backend("",  function(...) NULL), "non-empty")
  expect_error(register_backend(NULL, function(...) NULL), "non-empty")
})

test_that("get_backend: returns NULL for unknown backend", {
  expect_null(get_backend("nonexistent_backend_xyz"))
})


# =============================================================================
# 2.  Geometry registry -- registration and retrieval
# =============================================================================

test_that("built-in geoms are registered at package load", {
  g <- list_geoms()
  expect_true(all(c("column", "line", "scatter",
                    "arearange", "pie", "ranked_bar") %in% g))
})

test_that("list_geoms returns a sorted character vector", {
  g <- list_geoms()
  expect_type(g, "character")
  expect_equal(g, sort(g))
})

test_that("register_geom / .get_geom: round-trip stores all fields", {
  on.exit(rm("rt_geom", envir = highdir:::.geom_registry), add = TRUE)
  register_geom("rt_geom",
    ggplot_fun      = .dummy_gg,
    highcharter_fun = .dummy_hc,
    required_args   = list(foo = .req("A required foo")),
    optional_args   = list(bar = .opt(42L, "Bar radius")),
    is_map_geom     = FALSE
  )

  g <- highdir:::.get_geom("rt_geom")

  expect_equal(g$name, "rt_geom")
  expect_true(is.function(g$ggplot_fun))
  expect_true(is.function(g$highcharter_fun))
  expect_false(isTRUE(g$is_map_geom))

  # required_args: named list with default + desc
  expect_true("foo" %in% names(g$required_args))
  expect_null(g$required_args$foo$default)
  expect_equal(g$required_args$foo$desc, "A required foo")

  # optional_args: named list with default + desc
  expect_true("bar" %in% names(g$optional_args))
  expect_equal(g$optional_args$bar$default, 42L)
  expect_equal(g$optional_args$bar$desc,    "Bar radius")
})

test_that("register_geom: NULL functions are accepted", {
  on.exit(rm("null_fns_geom", envir = highdir:::.geom_registry), add = TRUE)
  register_geom("null_fns_geom")

  g <- highdir:::.get_geom("null_fns_geom")
  expect_null(g$ggplot_fun)
  expect_null(g$highcharter_fun)
})

test_that("register_geom: is_map_geom = TRUE stored correctly", {
  on.exit(rm("map_test_geom", envir = highdir:::.geom_registry), add = TRUE)
  register_geom("map_test_geom", is_map_geom = TRUE)

  expect_true(isTRUE(highdir:::.get_geom("map_test_geom")$is_map_geom))
})

test_that("register_geom: returns name invisibly", {
  on.exit(rm("invis_geom", envir = highdir:::.geom_registry), add = TRUE)
  result <- register_geom("invis_geom")

  expect_equal(result, "invis_geom")
})

test_that("register_geom: overwrites an existing geom silently", {
  on.exit(rm("overwrite_geom", envir = highdir:::.geom_registry), add = TRUE)
  register_geom("overwrite_geom",
    optional_args = list(x = .opt(1L, "first")))

  register_geom("overwrite_geom",
    optional_args = list(x = .opt(2L, "second")))

  g <- highdir:::.get_geom("overwrite_geom")
  expect_equal(g$optional_args$x$default, 2L)
})

test_that(".get_geom: returns NULL for unknown geom", {
  expect_null(highdir:::.get_geom("nonexistent_geom_xyz"))
})

# test_that("register_geom: rejects empty or invalid name", {
#   expect_error(register_geom(""))
#   expect_error(register_geom(NULL))
# })


# =============================================================================
# 3.  register_geom() -- optional_args structure validation
# =============================================================================

test_that("register_geom: rejects unnamed optional_args", {
  expect_error(
    register_geom("bad_oa",
      optional_args = list(.opt(1, "ok"), .opt(2, "unnamed"))),
    "fully named"
  )
})

test_that("register_geom: rejects optional_args with empty name", {
  # list("" = ...) is a parse error in R, so build it dynamically at runtime
  oa_empty_key <- setNames(list(.opt(1, "empty key")), "")
  expect_error(
    register_geom("bad_oa2", optional_args = oa_empty_key),
    "fully named"
  )
})

test_that("register_geom: rejects optional_args entry missing 'default'", {
  expect_error(
    register_geom("bad_oa3",
      optional_args = list(x = list(desc = "no default field"))),
    "list\\(default"
  )
})

test_that("register_geom: rejects optional_args entry missing 'desc'", {
  expect_error(
    register_geom("bad_oa4",
      optional_args = list(x = list(default = 1))),
    "list\\(default"
  )
})

test_that("register_geom: rejects optional_args entry that is not a list", {
  expect_error(
    register_geom("bad_oa5",
      optional_args = list(x = "not a list")),
    "list\\(default"
  )
})

test_that("register_geom: error message names the bad entries", {
  expect_error(
    register_geom("bad_oa6",
      optional_args = list(
        good = .opt(1, "fine"),
        bad1 = list(default = 1),
        bad2 = "wrong type"
      )),
    "bad1"
  )
})

test_that("register_geom: empty optional_args is accepted", {
  on.exit(rm("empty_oa_geom", envir = highdir:::.geom_registry), add = TRUE)
  expect_silent(register_geom("empty_oa_geom", optional_args = list()))
})

test_that("register_geom: NULL default in optional_args is accepted", {
  on.exit(rm("null_default_geom", envir = highdir:::.geom_registry), add = TRUE)
  expect_silent(
    register_geom("null_default_geom",
      optional_args = list(x = list(default = NULL, desc = "nullable")))
  )
})


# =============================================================================
# 4.  Built-in geom contracts
# =============================================================================

test_that("arearange: required_args is named list with ymin and ymax", {
  g  <- highdir:::.get_geom("arearange")
  ra <- g$required_args
  expect_type(ra, "list")
  expect_true(all(c("ymin", "ymax") %in% names(ra)))
  expect_null(ra$ymin$default)
  expect_null(ra$ymax$default)
  expect_type(ra$ymin$desc, "character")
  expect_type(ra$ymax$desc, "character")
})

test_that("arearange: is_map_geom is FALSE", {
  expect_false(isTRUE(highdir:::.get_geom("arearange")$is_map_geom))
})

test_that("column: has no required_args and no optional_args", {
  g <- highdir:::.get_geom("column")
  expect_length(g$required_args, 0L)
  expect_length(g$optional_args, 0L)
})

test_that("pie: has no required_args", {
  expect_length(highdir:::.get_geom("pie")$required_args, 0L)
})

test_that("pie: has inner_size optional_arg with default '0%'", {
  oa <- highdir:::.get_geom("pie")$optional_args
  expect_true("inner_size" %in% names(oa))
  expect_equal(oa$inner_size$default, "0%")
})

test_that("line: optional_args contains smooth, dot_size, line_symbols", {
  oa <- highdir:::.get_geom("line")$optional_args
  expect_true(all(c("smooth", "dot_size", "line_symbols") %in% names(oa)))
  expect_true(isTRUE(oa$smooth$default))
  expect_equal(oa$dot_size$default, 4L)
  expect_null(oa$line_symbols$default)
})

# test_that("map: is_map_geom is TRUE", {
#   expect_true(isTRUE(highdir:::.get_geom("map")$is_map_geom))
# })

# test_that("map: optional_args contains level, low_col, high_col, na_fill", {
#   oa <- highdir:::.get_geom("map")$optional_args
#   expect_true(all(c("level", "low_col", "high_col", "na_fill") %in% names(oa)))
#   expect_equal(oa$level$default, "county")
# })

test_that("ranked_bar: optional_args contains ascending, vs, aim", {
  oa <- highdir:::.get_geom("ranked_bar")$optional_args
  expect_true(all(c("ascending", "vs", "aim") %in% names(oa)))
  expect_true(isTRUE(oa$ascending$default))
  expect_null(oa$comp$default)
})


# =============================================================================
# 5.  validate_geom_args() -- uses names(required_args)
# =============================================================================

test_that("validate_geom_args: passes when all required args are present", {
  geom <- list(
    name          = "test",
    required_args = list(ymin = .req(), ymax = .req())
  )
  expect_invisible(
    highdir:::validate_geom_args(geom, list(ymin = "lo", ymax = "hi"))
  )
})

test_that("validate_geom_args: passes when required_args is empty", {
  geom <- list(name = "test", required_args = list())
  expect_invisible(
    highdir:::validate_geom_args(geom, list())
  )
})

test_that("validate_geom_args: stops with 'Missing required' when args absent", {
  geom <- list(
    name          = "test",
    required_args = list(ymin = .req(), ymax = .req())
  )
  expect_error(
    highdir:::validate_geom_args(geom, list()),
    "Missing required"
  )
})

test_that("validate_geom_args: error names the missing arg", {
  geom <- list(
    name          = "test",
    required_args = list(ymin = .req(), ymax = .req())
  )
  expect_error(
    highdir:::validate_geom_args(geom, list(ymin = "lo")),
    "ymax"
  )
})

test_that("validate_geom_args: error mentions geom name", {
  geom <- list(
    name          = "arearange",
    required_args = list(ymin = .req(), ymax = .req())
  )
  expect_error(
    highdir:::validate_geom_args(geom, list()),
    "arearange"
  )
})

test_that("validate_geom_args: error suggests geom_args()", {
  geom <- list(
    name          = "arearange",
    required_args = list(ymin = .req())
  )
  expect_error(
    highdir:::validate_geom_args(geom, list()),
    "geom_args"
  )
})

test_that("validate_geom_args: extra args beyond required do not cause errors", {
  geom <- list(
    name          = "test",
    required_args = list(ymin = .req())
  )
  expect_invisible(
    highdir:::validate_geom_args(geom, list(ymin = "lo", smooth = TRUE))
  )
})


# =============================================================================
# 6.  geom_args() -- discoverability helper
# =============================================================================

test_that("geom_args: errors on unknown geometry", {
  expect_error(highdir:::geom_args("nonexistent_xyz"), "Unknown geometry")
})

test_that("geom_args: returns invisible data.frame", {
  result <- withVisible(highdir:::geom_args("line"))
  expect_false(result$visible)
  expect_s3_class(result$value, "data.frame")
})

test_that("geom_args: data.frame has columns argument, kind, default, desc", {
  df <- highdir:::geom_args("line")
  expect_true(all(c("argument", "kind", "default", "desc") %in% names(df)))
})

test_that("geom_args: required args have kind = 'required'", {
  df       <- highdir:::geom_args("arearange")
  req_rows <- df[df$kind == "required", ]
  expect_true(nrow(req_rows) >= 2L)
  expect_true(all(c("ymin", "ymax") %in% req_rows$argument))
})

test_that("geom_args: optional args have kind = 'optional'", {
  df       <- highdir:::geom_args("line")
  opt_rows <- df[df$kind == "optional", ]
  expect_true(nrow(opt_rows) >= 1L)
  expect_true(all(c("smooth", "dot_size") %in% opt_rows$argument))
})

test_that("geom_args: NULL default shown as string 'NULL'", {
  df       <- highdir:::geom_args("arearange")
  req_rows <- df[df$kind == "required", ]
  expect_true(all(req_rows$default == "NULL"))
})

test_that("geom_args: non-NULL default shown as character", {
  df         <- highdir:::geom_args("line")
  smooth_row <- df[df$argument == "smooth", ]
  expect_equal(smooth_row$default, "TRUE")
})

test_that("geom_args: geom with no args emits message and returns empty df", {
  expect_message(
    result <- highdir:::geom_args("column"),
    "no extra arguments"
  )
  expect_s3_class(result, "data.frame")
  expect_equal(nrow(result), 0L)
})

test_that("geom_args: NULL type iterates all geoms and returns NULL invisibly", {
  result <- withVisible(highdir:::geom_args(NULL))
  expect_false(result$visible)
  expect_null(result$value)
})

test_that("geom_args: printed output contains geometry name in header", {
  output <- capture.output(highdir:::geom_args("line"))
  expect_true(any(grepl("line", output)))
})

test_that("geom_args: printed output contains column headers", {
  output <- capture.output(highdir:::geom_args("line"))
  expect_true(any(grepl("argument", output, ignore.case = TRUE)))
  expect_true(any(grepl("kind",     output, ignore.case = TRUE)))
})

test_that("geom_args: works for a custom registered geom", {
  on.exit(rm("custom_geom_test", envir = highdir:::.geom_registry), add = TRUE)
  register_geom("custom_geom_test",
    required_args = list(
      mycol = list(default = NULL, desc = "Required column name")
    ),
    optional_args = list(
      myarg = list(default = 5L, desc = "An optional integer")
    )
  )

  df <- highdir:::geom_args("custom_geom_test")
  expect_true("mycol" %in% df$argument)
  expect_true("myarg" %in% df$argument)
  expect_equal(df[df$argument == "mycol", "kind"],    "required")
  expect_equal(df[df$argument == "myarg", "kind"],    "optional")
  expect_equal(df[df$argument == "myarg", "default"], "5")
})

##### Old test -----------------------------------------------------------------

test_that("built-in geoms registered at load", {
  g <- list_geoms()
  expect_true(all(c("column", "line", "scatter", "arearange", "pie") %in% g))
})

test_that("built-in backends registered at load", {
  b <- list_backends()
  expect_true(all(c("ggplot2", "highcharter") %in% b))
})

test_that("register_backend / get_backend round-trip", {
  eng <- function(spec, geom, opts, geom_params, use_js, filename, ...) NULL
  register_backend("test_backend", eng)
  expect_true("test_backend" %in% list_backends())
  expect_identical(get_backend("test_backend"), eng)
})

test_that("register_backend: rejects non-function", {
  expect_error(register_backend("bad", "oops"), "function")
})

test_that("register_geom / .get_geom round-trip", {
  register_geom("test_geom",
    ggplot_fun      = function(spec, opts, gp, ...) list(),
    highcharter_fun = function(chart, spec, opts, gp, ...) chart,
    required_args   = c("foo")
  )
  expect_true("test_geom" %in% list_geoms())
  g <- .get_geom("test_geom")
  expect_equal(g$required_args, "foo")
  expect_true(is.function(g$ggplot_fun))
})

test_that("validate_geom_args: passes when all required present", {
  g <- list(required_args = c("ymin", "ymax"))
  expect_invisible(validate_geom_args(g, list(ymin = "a", ymax = "b")))
})

test_that("validate_geom_args: stops on missing required args", {
  g <- list(required_args = list(ymin = NULL, ymax = NULL))
  expect_error(validate_geom_args(g, list()), "Missing required")
  expect_error(validate_geom_args(g, list(ymin = "a")), "ymax")
})

test_that("arearange has correct required_args", {
  g <- .get_geom("arearange")
  expect_setequal(names(g$required_args), c("ymin", "ymax"))
})

test_that("pie has no required_args", {
  expect_length(.get_geom("pie")$required_args, 0)
})

# ------------------------------------------------------------------------------
# LINE / SPLINE
# ------------------------------------------------------------------------------
#
# Optional arguments (all via geom_params, never hard-coded in hd_make):
#
#   smooth       Logical.  TRUE = spline/loess, FALSE = straight line + points.
#                Default TRUE.  Both backends.
#   dot_size     Numeric.  Marker radius (px for HC, ggplot size / 3 for gg).
#                Default 4.  Both backends.
#   line_symbols Character vector.  Per-group Highcharts marker shapes.
#                Default NULL (uses resolve_symbols()).  Highcharter only -
#                gg_line silently ignores it.
#
# These three are documented in the registry (zzz.R) under optional_args so
# that geom_args("line") shows them to users.  The geom functions themselves
# apply the defaults via `geom_params$key %||% default` - the registry entry
# is for discoverability only, not enforcement.
#
# Why smooth and dot_size are NOT named params of hd_make():
#   They belong to the line geometry, not to hd_make() itself.  Adding them
#   to hd_make()'s signature would mean every column/pie/map caller sees them
#   in autocomplete even though they are irrelevant.  Passing them via `...`
#   keeps hd_make() closed for modification when new geoms are added.
#
# Why line_symbols IS still accepted via `...` and not a hard-coded param:
#   Same reason - it is geom-specific.  It was previously a named param of
#   hd_make() but has been moved to `...` alongside smooth and dot_size.
# ------------------------------------------------------------------------------

#' @keywords internal
gg_line <- function(spec, opts, geom_params) {

  # Read from geom_params with fallback defaults.
  # These defaults must match what is registered in zzz.R optional_args
  # so that geom_args("line") shows the same values the geom actually uses.
  sc       <- geom_params$single_colour   # set by ggplot_engine for no-group specs
  smooth   <- geom_params$smooth   %||% TRUE
  size     <- geom_params$dot_size  %||% 4L

  # line_symbols is intentionally NOT read here - it is a Highcharts concept
  # (named marker shapes like "circle", "square").  ggplot2 uses shape
  # integers via scale_shape, which is handled by apply_gg_colors().
  # Passing line_symbols to the ggplot2 backend is therefore silently ignored,
  # which is the intended behaviour.

  if (!is.null(sc)) {
    # Single-series path: colour injected directly so no scale conflict
    layers <- list(
      ggplot2::geom_line(colour = sc, linewidth = 0.8),
      ggplot2::geom_point(colour = sc, size = size / 3)
    )
    if (smooth)
      layers <- c(layers, list(
        ggplot2::geom_smooth(method = "loess", formula = y ~ x,
                             se = FALSE, colour = sc,
                             linewidth = 0.5, linetype = "dashed")
      ))
  } else {
    # Multi-series path: colour comes from apply_gg_colors() in ggplot_engine
    layers <- list(
      ggplot2::geom_line(linewidth = 0.8),
      ggplot2::geom_point(size = size / 3)
    )
    if (smooth)
      layers <- c(layers, list(
        ggplot2::geom_smooth(method = "loess", formula = y ~ x,
                             se = FALSE, linewidth = 0.5,
                             linetype = "dashed")
      ))
  }

  layers
}


#' @keywords internal
hc_line <- function(chart, spec, opts, geom_params, use_js = TRUE, ...) {
    # Read optional args from geom_params with fallback defaults.
    # Defaults must match zzz.R optional_args for geom_args("line") accuracy.
    smooth <- geom_params$smooth %||% TRUE
    dot_size <- geom_params$dot_size %||% 4
    symbols <- geom_params$line_symbols # NULL -> resolve_symbols() picks automatically

    groups <- .group_split(spec)
    palette <- resolve_colors(length(groups), opts$colors)
    syms <- resolve_symbols(length(groups), symbols)
    point_ev <- point_events_or_null(use_js)
    xmap <- .hc_x_map(spec)

    # smooth = TRUE  -> Highcharts "spline" type (cubic interpolation)
    # smooth = FALSE -> Highcharts "line"   type (straight segments)
    ctype <- if (smooth) "spline" else "line"

    for (i in seq_along(groups)) {
        grp <- groups[[i]]
        args <- list(
            chart,
            data      = xmap$data[grp$rows, ],
            type      = ctype,
            name      = grp$name,
            xmap$mapping,
            color     = palette[i],
            lineWidth = 2,
            marker    = list(symbol = syms[i], enabled = TRUE, radius = dot_size),
            states    = list(hover = list(lineWidth = 3))
        )
        if (!is.null(point_ev)) args$point <- point_ev
        chart <- do.call(highcharter::hc_add_series, args)
    }
    chart
}


## -----------------------------------------------------------------------------
## Public constructor
## -----------------------------------------------------------------------------
##

#' Line Geometry Layer for hd Objects
#'
#' `hd_geom_line()` creates a line geometry layer that is added to an [hd()]
#' object via `+`.  The layer records the geometry type and any geometry-specific arguments;
#' rendering only happens when the `hd` object is printed.
#'
#' @param smooth Logical. TRUE = spline curves, FALSE = straight segments. Both backends.
#' @param dot_size Numeric. Marker radius in pixels. Both backends.
#' @param line_symbols Character vector. Highcharter only. Per-group marker shapes:
#'  "circle", "square", "diamond", "triangle", "triangle-down".
#' @param ... Geometry-specific arguments forwarded to [hd_make()].
#'
#' @return An S3 object of class `"hd_geom"` for use with `+.hd`.
#'
#' @examples
#' # Single series - no group column
#' spec_line1 <- hd_spec(alco1,
#'     x    = "year",
#'     y    = "adj_mean"
#' )
#'
#' opts_line <- hd_opts(
#'     title = "Alcohol consumption over time",
#'     subtitle = "Source: Norwegian Directorate of Health",
#'     ylim = c(0, 50),
#'     ylab = "Litres per capita"
#' )
#'
#'
#' # Straight segments
#' hd_make(spec_line1, "line", opts_line, smooth = FALSE)
#'
#' # Composite example with multiple geoms and custom line symbols
#' hd(alco2, x = "year", y = "adj_mean", group = "kjonn", backend = "ggplot2") +
#'   hd_geom_line(smooth = TRUE, dot_size = 3) +
#'   hd_opts(title = "Alcohol consumption over time by kjonn",
#'   subtitle = "Source: Norwegian Directorate of Health")
#'
#' @export
hd_geom_line <- function(smooth = TRUE, dot_size = 4, line_symbols = NULL,...) {
  hd_geom("line", smooth = smooth, dot_size = dot_size, line_symbols = line_symbols, ...)
}

# tests/testthat/helper-layers.R

#' Find layer indices by geom class name
#' @param fig  A ggplot object
#' @param geom_class  Character. e.g. "GeomCol", "GeomLine", "GeomPoint"
#' @return Integer vector of matching layer indices
.find_layers <- function(fig, geom_class) {
  which(vapply(fig$layers, function(l) inherits(l$geom, geom_class),
               logical(1)))
}

#' Get fixed aesthetic value from a specific layer
.layer_aes <- function(fig, geom_class, aes_name) {
  idx <- .find_layers(fig, geom_class)
  if (!length(idx)) return(NULL)
  fig$layers[[idx[1]]]$aes_params[[aes_name]]
}

# Style, theme and JavaScript configuration

# -- Session-wide style setter ------------------------------------------------

#' Set Package-Wide Style Defaults
#'
#' Configures the default theme, colour palette, font, and optional JavaScript
#' plugins for all figures produced with [hd_make()] in the current R session.
#' Call once at the top of a script or in `.Rprofile`.
#'
#' Per-figure overrides are provided via [hd_opts()], which always take
#' precedence over these session defaults.
#'
#' @param hc_theme   Character or `NULL`. Built-in highcharter theme name: one
#'   of `"default"`, `"smpl"`, `"economist"`, `"darkunica"`, `"gridlight"`,
#'   `"bloom"`, `"flat"`, `"flatdark"`, `"ggplot2"`.
#' @param gg_theme   Character, ggplot2 theme object, or `NULL`. Controls the
#'   ggplot2 backend appearance. Built-in name strings: `"minimal"` (default),
#'   `"classic"`, `"bw"`, `"light"`, `"dark"`, `"void"`, `"grey"`.
#'   Alternatively pass any `ggplot2::theme_*()` object directly for full
#'   control, e.g. `ggplot2::theme_bw(base_size = 14)`.
#' @param colors     Character vector, palette name, or `NULL`. Applied to all
#'   figures in the session. See [register_palette()].
#' @param font       Character or `NULL`. Font family name, e.g.
#'   `"Source Sans Pro"`.
#' @param js_plugins Character vector or `NULL`. Names of bundled JS plugins
#'   (files in `inst/js/`) injected into every highcharter figure. Use
#'   `character(0)` to clear all plugins.
#'
#' @return The previous option values invisibly; pass to `options()` to
#'   restore.
#'
#' @examples
#' hd_set_theme(hc_theme = "economist", gg_theme = "classic",
#'              colors   = c("#025169", "#7C145C", "#C68803"))
#' # Reset
#' hd_set_theme(hc_theme = "default", gg_theme = "minimal", colors = NULL)
#'
#' @seealso [hd_opts()] for per-figure overrides
#' @export
hd_set_theme <- function(hc_theme   = NULL,
                         gg_theme   = NULL,
                         colors     = NULL,
                         font       = NULL,
                         js_plugins = NULL) {
  prev <- list()
  if (!is.null(hc_theme)) {
    prev$highdir.hc_theme <- getOption("highdir.hc_theme")
    options(highdir.hc_theme = hc_theme)
  }
  if (!is.null(gg_theme)) {
    prev$highdir.gg_theme <- getOption("highdir.gg_theme")
    options(highdir.gg_theme = gg_theme)
  }
  if (!is.null(colors)) {
    prev$highdir.colors <- getOption("highdir.colors")
    options(highdir.colors = colors)
  }
  if (!is.null(font)) {
    prev$highdir.font <- getOption("highdir.font")
    options(highdir.font = font)
  }
  if (!is.null(js_plugins)) {
    prev$highdir.js_plugins <- getOption("highdir.js_plugins")
    options(highdir.js_plugins = js_plugins)
  }
  invisible(prev)
}

# -- Highcharter theme builder ------------------------------------------------

#' Build a Highcharts Theme Object
#'
#' Constructs a highcharter theme by merging a named base theme with colour
#' and font overrides from the current [hd_set_theme()] session defaults and
#' any per-figure `opts`.
#'
#' Called automatically inside the highcharter engine; useful when you want to
#' apply a theme to a highchart built outside highdir.
#'
#' @param name   Character or `NULL`. Theme name; `NULL` reads from
#'   `getOption("highdir.hc_theme")`.
#' @param colors Character vector or `NULL`. Colour override for this call.
#' @param ...    Named arguments forwarded to [highcharter::hc_theme()] as
#'   extra overrides on top of the base theme.
#'
#' @return A highcharter theme object (`hc_theme`).
#'
#' @examples
#' if(interactive()) {
#' t <- hd_theme("darkunica")
#' highcharter::highchart() |> highcharter::hc_add_theme(t)
#' }
#'
#' @export
hd_theme <- function(name = NULL, colors = NULL, ...) {
  name   <- name   %||% getOption("highdir.hc_theme", default = "default")
  colors <- colors %||% getOption("highdir.colors",   default = NULL)
  font   <- getOption("highdir.font", default = NULL)

  base <- switch(name,
    "default" = , "smpl" = highcharter::hc_theme_smpl(),
    "economist"  = highcharter::hc_theme_economist(),
    "darkunica"  = highcharter::hc_theme_darkunica(),
    "gridlight"  = highcharter::hc_theme_gridlight(),
    "bloom"      = highcharter::hc_theme_bloom(),
    "flat"       = highcharter::hc_theme_flat(),
    "flatdark"   = highcharter::hc_theme_flatdark(),
    "ggplot2"    = highcharter::hc_theme_ggplot2(),
    stop("Unknown theme '", name, "'. Choose from: default, smpl, economist, ",
         "darkunica, gridlight, bloom, flat, flatdark, ggplot2",
         call. = FALSE)
  )

  overrides <- list(...)
  if (!is.null(colors)) overrides$colors <- colors
  if (!is.null(font)) {
    overrides$chart <- utils::modifyList(
      overrides$chart %||% list(),
      list(style = list(fontFamily = font))
    )
  }

  if (length(overrides) > 0)
    highcharter::hc_theme_merge(base,
                                do.call(highcharter::hc_theme, overrides))
  else
    base
}

# -- ggplot2 theme builder ----------------------------------------------------

#' Build a ggplot2 Theme Object
#'
#' Constructs a ggplot2 theme by resolving a base theme, then merging colour
#' and font overrides -- exactly mirroring what [hd_theme()] does for the
#' highcharter backend.
#'
#' Priority for each argument:
#' - `theme`:  explicit argument > `getOption("highdir.gg_theme")` > `"classic"`
#' - `colors`: explicit argument > `getOption("highdir.colors")` > `NULL`
#' - `font`:   explicit argument > `getOption("highdir.font")`   > `NULL`
#'
#' Called automatically inside `ggplot_engine()`; also useful for applying the
#' package theme to a ggplot built outside highdir.
#'
#' Built-in name strings and their ggplot2 equivalents:
#'
#' | Name                  | ggplot2 function |
#' |:----------------------|:-----------------|
#' | `"classic"` (default) | `theme_classic()`|
#' | `"minimal"`           | `theme_minimal()`|
#' | `"bw"`                | `theme_bw()`     |
#' | `"light"`             | `theme_light()`  |
#' | `"dark"`              | `theme_dark()`   |
#' | `"void"`              | `theme_void()`   |
#' | `"grey"` / `"gray"`   | `theme_grey()`   |
#'
#' @param theme  Character name string, ggplot2 theme object, or `NULL`.
#'   `NULL` reads from `getOption("highdir.gg_theme")`.
#' @param colors Character vector, palette name string, or `NULL`. Resolved
#'   colours are stored on the returned object and applied by
#'   `ggplot_engine()` via `scale_color_manual` / `scale_fill_manual`.
#'   `NULL` reads from `getOption("highdir.colors")`.
#' @param font   Character or `NULL`. Font family applied to all text elements
#'   via `theme(text = element_text(family = font))`. `NULL` reads from
#'   `getOption("highdir.font")`.
#'
#' @return An object of class `"hd_gg_theme"` - a list with two fields:
#'   `$theme` (a ggplot2 `theme` object with font baked in) and `$colors`
#'   (a resolved character vector or `NULL`).  `ggplot_engine()` unpacks
#'   both.  The object can also be added directly to a ggplot with `+`
#'   via the `+.gg` method (only the theme is applied; colors are handled
#'   separately by `apply_gg_colors()`).
#'
#' @examples
#' gg_theme()                                    # session defaults
#' gg_theme("bw")                                # theme_bw(), session colors/font
#' gg_theme("classic", colors = c("#025169", "#7C145C"))
#' gg_theme("minimal", font = "Source Sans Pro")
#' gg_theme(ggplot2::theme_bw(base_size = 14), font = "mono")
#'
#' @export
gg_theme <- function(theme = NULL, colors = NULL, font = NULL) {

  # -- 1. Resolve base theme --------------------------------------------------
  resolved <- theme %||% getOption("highdir.gg_theme", default = "classic")

  base <- if (inherits(resolved, "theme")) {
    resolved
  } else {
    if (!is.character(resolved) || length(resolved) != 1L)
      stop("`gg_theme` must be a single theme name string or a ggplot2 theme ",
           "object. Got: ", class(resolved)[1L], call. = FALSE)
    switch(resolved,
      "classic" = ggplot2::theme_classic(),
      "minimal" = ggplot2::theme_minimal(),
      "bw"      = ggplot2::theme_bw(),
      "light"   = ggplot2::theme_light(),
      "dark"    = ggplot2::theme_dark(),
      "void"    = ggplot2::theme_void(),
      "grey" = ,
      "gray"    = ggplot2::theme_grey(),
      stop("Unknown gg_theme name '", resolved, "'. ",
           "Use: classic, minimal, bw, light, dark, void, grey. ",
           "Or pass a ggplot2::theme_*() object directly.",
           call. = FALSE)
    )
  }

  # -- 2. Font override (same priority chain as hd_theme) ---------------------
  font <- font %||% getOption("highdir.font", default = NULL)
  if (!is.null(font))
    base <- base + ggplot2::theme(text = ggplot2::element_text(family = font))

  # -- 3. Resolve colors (stored for ggplot_engine, not baked into theme) -----
  # Colors are plot-level scales (scale_color_manual / scale_fill_manual), not
  # theme elements, so they cannot be embedded in the theme object itself.
  # We carry them alongside the theme in the returned hd_gg_theme object so
  # ggplot_engine() has one place to read everything -- mirroring how hc_theme
  # carries colors inside the Highcharts theme JSON.
  resolved_colors <- colors %||% getOption("highdir.colors", default = NULL)

  structure(
    list(theme = base, colors = resolved_colors),
    class = "hd_gg_theme"
  )
}

#' @export
#' @keywords internal
# Allows `p + gg_theme("bw")` to work directly on a ggplot object.
# Only the theme portion is applied; colors are handled by apply_gg_colors().
"+.hd_gg_theme" <- function(e1, e2) {
  if (inherits(e1, "hd_gg_theme")) {
    # hd_gg_theme + something: apply theme to something if it is a ggplot
    stop("Use `p + gg_theme(...)` not `gg_theme(...) + p`.", call. = FALSE)
  }
  # p + hd_gg_theme: standard ggplot + theme
  e1 + e2$theme
}

# -- ggplot2 colour helper ----------------------------------------------------

#' Apply brand colours to a ggplot object
#'
#' Uses the same `resolve_colors()` priority chain as the highcharter
#' engine so both backends always produce identical colour assignments:
#'
#' 1. Explicit `colors` argument
#' 2. `getOption("highdir.colors")` session default
#' 3. Built-in rules: n==2 -> hdir2, n<=10 -> `hdir[1:n]`, n>10 -> viridis
#'
#' @param p            A ggplot object.
#' @param colors       Character vector, palette name string, or NULL.
#' @param n_groups     Integer. Number of groups in the data. Drives which
#'   palette rule fires. Passed from `ggplot_engine()`.
#' @param group_levels Character vector or NULL. Group level names in data
#'   order. Used to name the palette so ggplot2 matches colours to groups
#'   by name rather than alphabetical sort.
#' @return The modified ggplot object.
#' @keywords internal
apply_gg_colors <- function(p,
                            colors       = NULL,
                            n_groups     = NULL,
                            group_levels = NULL) {

  # -- Resolve palette through the shared rule engine ------------------------
  if (!is.null(n_groups) && n_groups >= 1L) {
    pal <- resolve_colors(n_groups, colors)
  } else {
    candidate <- colors %||% getOption("highdir.colors", default = NULL)
    if (is.character(candidate) && length(candidate) == 1L &&
        candidate %in% list_palettes()) {
      candidate <- get_palette(candidate)
    }
    pal <- candidate
  }

  if (is.null(pal)) return(p)

  # -- Name the palette by group level in data order -------------------------
  if (!is.null(group_levels) &&
      length(pal) >= length(group_levels)) {
    pal <- stats::setNames(
      pal[seq_along(group_levels)],
      group_levels
    )
  }

  p +
    ggplot2::scale_color_manual(values = pal) +
    ggplot2::scale_fill_manual(values  = pal)
}


# -- JavaScript injection -----------------------------------------------------

#' Inject JavaScript into a Highcharts Widget
#'
#' Appends custom JavaScript to a `highchart` object via
#' `chart.events.<where>`.  Use this for hand-written callbacks and plugins.
#' For Highcharts built-in modules (accessibility, exporting, etc.) use
#' [highcharter::hc_add_dependency()] instead.
#'
#' Exactly one of `code`, `file`, or `plugin` must be supplied.
#'
#' @param hc     A `highchart` object.
#' @param code   Character or `NULL`. Raw JavaScript string.
#' @param file   Character or `NULL`. Path to a `.js` file to read and inject.
#' @param plugin Character or `NULL`. Name of a plugin bundled in `inst/js/`.
#' @param where  `"load"` (default) or `"render"`.
#'
#' @return The `highchart` object with JS injected.
#'
#' @examples
#' if (interactive()) {
#' spec <- hd_spec(mtcars, "wt", "mpg")
#' fig  <- hd_make(spec, "scatter")
#' fig  <- hd_add_js(fig, code = "console.log('chart loaded');")
#' }
#'
#' @export
hd_add_js <- function(hc,
                      code   = NULL,
                      file   = NULL,
                      plugin = NULL,
                      where  = c("load", "render")) {
  where <- match.arg(where)

  if (!is.null(plugin)) {
    file <- system.file("js", paste0(plugin, ".js"), package = "highdir")
    if (!nzchar(file)) {
      avail <- tools::file_path_sans_ext(
        list.files(system.file("js", package = "highdir",
                               mustWork = FALSE)))
      stop("Plugin '", plugin, "' not found in inst/js/.",
           if (length(avail))
             paste0(" Available: ", paste(avail, collapse = ", ")),
           call. = FALSE)
    }
  }

  if (!is.null(file)) {
    if (!file.exists(file))
      stop("File not found: ", file, call. = FALSE)
    code <- paste(readLines(file, warn = FALSE), collapse = "\n")
  }

  if (is.null(code))
    stop("Supply one of `code`, `file`, or `plugin`.", call. = FALSE)

  hc |> highcharter::hc_chart(
    events = stats::setNames(list(htmlwidgets::JS(code)), where)
  )
}

#
# Covers the gg_theme() resolver and its integration across the full stack:
#
#   gg_theme()          - name resolution, object pass-through, errors
#   hd_set_theme()      - gg_theme parameter: sets / restores session option
#   hd_opts()           - gg_theme field stored and retrieved
#   ggplot_engine()     - theme actually applied to output figure
#   hd_make()           - end-to-end: per-figure opts vs session default
#
# RETURN TYPE OF gg_theme()
#   gg_theme() returns an hd_gg_theme object — a list with two fields:
#     $theme  : ggplot2 theme object (with font baked in)
#     $colors : resolved colour vector or NULL
#   Tests that inspect the theme itself access gt$theme.
#   Tests that inspect resolved colours access gt$colors.
#
# HOW WE INSPECT THE APPLIED THEME (sections 4-6)
#   ggplot_engine() adds gt$theme to the plot and calls apply_gg_colors(gt$colors).
#   We verify by reading ggplot_build(fig)$plot$theme for structural properties
#   that differ reliably between themes, avoiding fragile full-object comparison.

# ── Shared fixtures ────────────────────────────────────────────────────────────

spec      <- hd_spec(survey_df, "age", "pct", group = "sex")
opts_base <- hd_opts(title = "Theme test")

# Helper: extract a theme element from a built ggplot
theme_el <- function(fig, element) {
  ggplot2::ggplot_build(fig)$plot$theme[[element]]
}


# ══════════════════════════════════════════════════════════════════════════════
# 1.  gg_theme() — unit tests on the returned hd_gg_theme object
# ══════════════════════════════════════════════════════════════════════════════

test_that("gg_theme: returns hd_gg_theme for every built-in name", {
  nms <- c("minimal", "classic", "bw", "light", "dark", "void", "grey", "gray")
  for (nm in nms) {
    gt <- gg_theme(nm)
    expect_true(inherits(gt, "hd_gg_theme"),  label = paste("class hd_gg_theme:", nm))
    expect_true(inherits(gt$theme, "theme"),   label = paste("$theme is theme:", nm))
  }
})

test_that("gg_theme: $colors is NULL when no colors supplied and no session option", {
  withr::with_options(list(highdir.colors = NULL), {
    gt <- gg_theme("classic")
    expect_null(gt$colors)
  })
})

test_that("gg_theme: $colors resolved from explicit argument", {
  pal <- c("#025169", "#7C145C")
  gt  <- gg_theme("classic", colors = pal)
  expect_equal(gt$colors, pal)
})

test_that("gg_theme: $colors resolved from session option when not explicit", {
  pal <- c("#AAAAAA", "#BBBBBB")
  withr::with_options(list(highdir.colors = pal), {
    gt <- gg_theme("classic")
    expect_equal(gt$colors, pal)
  })
})

test_that("gg_theme: explicit colors override session option", {
  session_pal <- c("#AAAAAA", "#BBBBBB")
  explicit_pal <- c("#025169", "#7C145C")
  withr::with_options(list(highdir.colors = session_pal), {
    gt <- gg_theme("classic", colors = explicit_pal)
    expect_equal(gt$colors, explicit_pal)
  })
})

test_that("gg_theme: NULL reads session option highdir.gg_theme", {
  withr::with_options(list(highdir.gg_theme = "classic"), {
    gt <- gg_theme(NULL)
    expect_true(inherits(gt$theme, "theme"))
    # theme_classic has no panel grid lines (element_blank)
    expect_true(inherits(gt$theme$panel.grid,       "element_blank") ||
                inherits(gt$theme$panel.grid.major, "element_blank"))
  })
})

test_that("gg_theme: falls back to 'classic' when session option unset", {
  withr::with_options(list(highdir.gg_theme = NULL), {
    gt <- gg_theme(NULL)
    expect_true(inherits(gt$theme, "theme"))
    expect_true(inherits(gt$theme$panel.grid,       "element_blank") ||
                inherits(gt$theme$panel.grid.major, "element_blank"),
                label = "classic fallback has no visible panel grid")
  })
})

test_that("gg_theme: theme object passed directly is wrapped in hd_gg_theme", {
  obj <- ggplot2::theme_bw(base_size = 18)
  gt  <- gg_theme(obj)
  expect_true(inherits(gt, "hd_gg_theme"))
  # Font not set, so theme is returned as-is inside $theme
  expect_equal(gt$theme$text$size, 18)
})

test_that("gg_theme: font baked into $theme", {
  gt <- gg_theme("classic", font = "mono")
  expect_equal(gt$theme$text$family, "mono")
})

test_that("gg_theme: font read from session option when not explicit", {
  withr::with_options(list(highdir.font = "serif"), {
    gt <- gg_theme("classic")
    expect_equal(gt$theme$text$family, "serif")
  })
})

test_that("gg_theme: explicit font overrides session option", {
  withr::with_options(list(highdir.font = "serif"), {
    gt <- gg_theme("classic", font = "mono")
    expect_equal(gt$theme$text$family, "mono")
  })
})

test_that("gg_theme: errors on unknown name string", {
  expect_error(gg_theme("neon_banana"), "Unknown gg_theme name")
})

test_that("gg_theme: errors on non-string non-theme input", {
  expect_error(gg_theme(42),     "must be a single theme name string")
  expect_error(gg_theme(TRUE),   "must be a single theme name string")
  expect_error(gg_theme(list()), "must be a single theme name string")
})

test_that("gg_theme: grey and gray are aliases — same $theme panel.background", {
  gt_grey <- gg_theme("grey")
  gt_gray <- gg_theme("gray")
  expect_equal(gt_grey$theme$panel.background$fill,
               gt_gray$theme$panel.background$fill)
})


# ══════════════════════════════════════════════════════════════════════════════
# 2.  hd_set_theme() — gg_theme parameter
# ══════════════════════════════════════════════════════════════════════════════

test_that("hd_set_theme: gg_theme sets session option", {
  withr::with_options(list(highdir.gg_theme = "minimal"), {
    hd_set_theme(gg_theme = "classic")
    expect_equal(getOption("highdir.gg_theme"), "classic")
  })
})

test_that("hd_set_theme: gg_theme returns previous value invisibly", {
  withr::with_options(list(highdir.gg_theme = "minimal"), {
    prev <- hd_set_theme(gg_theme = "bw")
    expect_equal(prev$highdir.gg_theme, "minimal")
  })
})

test_that("hd_set_theme: previous value can be used to restore option", {
  withr::with_options(list(highdir.gg_theme = "minimal"), {
    prev <- hd_set_theme(gg_theme = "classic")
    options(prev)
    expect_equal(getOption("highdir.gg_theme"), "minimal")
  })
})

test_that("hd_set_theme: gg_theme NULL does not change existing option", {
  withr::with_options(list(highdir.gg_theme = "bw"), {
    hd_set_theme(gg_theme = NULL)
    expect_equal(getOption("highdir.gg_theme"), "bw")
  })
})

test_that("hd_set_theme: gg_theme accepts a theme object directly", {
  obj <- ggplot2::theme_bw(base_size = 14)
  withr::with_options(list(highdir.gg_theme = "minimal"), {
    hd_set_theme(gg_theme = obj)
    stored <- getOption("highdir.gg_theme")
    expect_true(inherits(stored, "theme"))
    expect_identical(stored, obj)
  })
})

test_that("hd_set_theme: hc_theme and gg_theme can be set simultaneously", {
  withr::with_options(list(highdir.hc_theme = "default",
                           highdir.gg_theme = "minimal"), {
    hd_set_theme(hc_theme = "gridlight", gg_theme = "classic")
    expect_equal(getOption("highdir.hc_theme"), "gridlight")
    expect_equal(getOption("highdir.gg_theme"), "classic")
  })
})


# ══════════════════════════════════════════════════════════════════════════════
# 3.  hd_opts() — gg_theme field
# ══════════════════════════════════════════════════════════════════════════════

test_that("hd_opts: gg_theme stored as NULL by default", {
  opts <- hd_opts()
  expect_null(opts$gg_theme)
})

test_that("hd_opts: gg_theme name string stored correctly", {
  opts <- hd_opts(gg_theme = "classic")
  expect_equal(opts$gg_theme, "classic")
})

test_that("hd_opts: gg_theme theme object stored correctly", {
  obj  <- ggplot2::theme_bw(base_size = 16)
  opts <- hd_opts(gg_theme = obj)
  expect_identical(opts$gg_theme, obj)
})

test_that("hd_opts: class is preserved with gg_theme set", {
  opts <- hd_opts(gg_theme = "classic")
  expect_s3_class(opts, "hd_opts")
})


# ══════════════════════════════════════════════════════════════════════════════
# 4.  ggplot_engine integration — theme + colors + font applied to figure
# ══════════════════════════════════════════════════════════════════════════════

# Inspect structural theme properties that differ reliably between themes.
# theme_classic: element_blank panel grid, no panel fill
# theme_bw:      white panel background
# theme_minimal: transparent/NA panel background

test_that("ggplot_engine: theme_classic applied — panel grid is blank", {
  opts <- hd_opts(title = "t", gg_theme = "classic")
  fig  <- hd_make(spec, "column", opts, backend = "ggplot2")
  t    <- ggplot2::ggplot_build(fig)$plot$theme
  expect_true(inherits(t$panel.grid,       "element_blank") ||
              inherits(t$panel.grid.major, "element_blank"),
              label = "classic: no visible panel grid")
})

test_that("ggplot_engine: theme_bw applied — panel background is white", {
  opts <- hd_opts(title = "t", gg_theme = "bw")
  fig  <- hd_make(spec, "column", opts, backend = "ggplot2")
  t    <- ggplot2::ggplot_build(fig)$plot$theme
  expect_equal(t$panel.background$fill, "white",
               label = "bw: white panel background")
})

test_that("ggplot_engine: theme object in opts — base_size honoured", {
  obj  <- ggplot2::theme_bw(base_size = 18)
  opts <- hd_opts(title = "t", gg_theme = obj)
  fig  <- hd_make(spec, "column", opts, backend = "ggplot2")
  t    <- ggplot2::ggplot_build(fig)$plot$theme
  # base_size is stored in the top-level text element as absolute value
  expect_equal(t$text$size, 18)
})

test_that("ggplot_engine: session default used when opts$gg_theme is NULL", {
  withr::with_options(list(highdir.gg_theme = NULL), {
    opts <- hd_opts(title = "t")   # NULL -> classic fallback
    fig  <- hd_make(spec, "column", opts, backend = "ggplot2")
    t    <- ggplot2::ggplot_build(fig)$plot$theme
    expect_true(inherits(t$panel.grid,       "element_blank") ||
                inherits(t$panel.grid.major, "element_blank"),
                label = "classic fallback: no visible panel grid")
  })
})

test_that("ggplot_engine: opts gg_theme overrides session default", {
  withr::with_options(list(highdir.gg_theme = "classic"), {
    opts <- hd_opts(title = "t", gg_theme = "bw")
    fig  <- hd_make(spec, "column", opts, backend = "ggplot2")
    t    <- ggplot2::ggplot_build(fig)$plot$theme
    expect_equal(t$panel.background$fill, "white",
                 label = "per-figure bw overrides session classic")
  })
})

test_that("ggplot_engine: colors in opts applied to figure", {
  pal  <- c("#025169", "#7C145C")
  opts <- hd_opts(title = "t", colors = pal)
  fig  <- hd_make(spec, "column", opts, backend = "ggplot2")
  built <- ggplot2::ggplot_build(fig)
  # Colours appear in the data layer fills; check at least one matches palette
  fills <- unique(unlist(lapply(built$data, function(d) d$fill)))
  expect_true(any(fills %in% pal),
              label = "at least one fill matches the supplied palette")
})

test_that("ggplot_engine: font in opts applied to figure", {
  withr::with_options(list(highdir.font = "mono"), {
    opts <- hd_opts(title = "t", gg_theme = "classic")
    fig  <- hd_make(spec, "column", opts, backend = "ggplot2")
    t    <- ggplot2::ggplot_build(fig)$plot$theme
    expect_equal(t$text$family, "mono")
  })
})


# ══════════════════════════════════════════════════════════════════════════════
# 5.  hd_make() end-to-end — gg_theme does not affect highcharter backend
# ══════════════════════════════════════════════════════════════════════════════

test_that("hd_make: gg_theme in opts does not break highcharter output", {
  opts <- hd_opts(title = "t", gg_theme = "classic")
  fig  <- hd_make(spec, "column", opts, backend = "highcharter")
  expect_true(is_highchart(fig))
})

test_that("hd_make: gg_theme object in opts does not break highcharter", {
  obj  <- ggplot2::theme_bw()
  opts <- hd_opts(title = "t", gg_theme = obj)
  fig  <- hd_make(spec, "column", opts, backend = "highcharter")
  expect_true(is_highchart(fig))
})


# ══════════════════════════════════════════════════════════════════════════════
# 6.  font preserved across all theme names (ordering guard)
# ══════════════════════════════════════════════════════════════════════════════

test_that("font option is preserved regardless of gg_theme", {
  withr::with_options(list(highdir.font = "mono"), {
    for (nm in c("minimal", "classic", "bw")) {
      opts <- hd_opts(title = "t", gg_theme = nm)
      fig  <- hd_make(spec, "column", opts, backend = "ggplot2")
      t    <- ggplot2::ggplot_build(fig)$plot$theme
      expect_equal(t$text$family, "mono",
                   label = paste("font preserved with gg_theme =", nm))
    }
  })
})

test_that("hd_set_theme: sets and restores options", {
  withr::with_options(list(highdir.hc_theme = "default"), {
    prev <- hd_set_theme(hc_theme = "darkunica")
    expect_equal(getOption("highdir.hc_theme"), "darkunica")
    options(prev)
    expect_equal(getOption("highdir.hc_theme"), "default")
  })
})

test_that("hd_set_theme: sets colors option", {
  withr::with_options(list(highdir.colors = NULL), {
    hd_set_theme(colors = c("#AAAAAA", "#BBBBBB"))
    expect_equal(getOption("highdir.colors"), c("#AAAAAA", "#BBBBBB"))
  })
})

test_that("hd_theme: returns an hc_theme object for all built-in names", {
  themes <- c("default","smpl","economist","darkunica",
              "gridlight","bloom","flat","flatdark","ggplot2")
  for (nm in themes) {
    t <- hd_theme(nm)
    expect_true(inherits(t, "hc_theme"),
                label = paste("theme:", nm))
  }
})

test_that("hd_theme: errors on unknown name", {
  expect_error(hd_theme("neon_banana"), "Unknown theme")
})

test_that("hd_theme: colour override embedded in theme", {
  pal <- c("#FF0000", "#00FF00")
  t   <- hd_theme("default", colors = pal)
  expect_true(inherits(t, "hc_theme"))
  expect_true(any(unlist(t) %in% pal))
})

test_that("hd_add_js: errors when nothing supplied", {
  hc <- highcharter::highchart()
  expect_error(hd_add_js(hc), "Supply one of")
})

test_that("hd_add_js: injects inline code into chart", {
  hc  <- highcharter::highchart()
  hc2 <- hd_add_js(hc, code = "console.log('hi');")
  expect_true(is_highchart(hc2))
})

test_that("apply_gg_colors: no-op when no palette set", {
  withr::with_options(list(highdir.colors = NULL), {
    p  <- ggplot2::ggplot()
    p2 <- apply_gg_colors(p, NULL)
    expect_identical(p, p2)
  })
})

#' @keywords internal
gg_stacked_column <- function(spec, opts, geom_params) {

  # required args --------------------------------------------------------------
  stack_col <- geom_params$stack

  # optional args ----------------------------------------------------------------
  grp_col   <- spec$group

  #   stacking <- geom_params$stacking
  #   if (stacking == "percent") {
  #     warning("Percent stacking isn't implemented in static figure yet.",
  #       call. = FALSE
  #     )
  #     stacking <- "normal"
  #   }

  if (is.null(grp_col))
    stop("stacked_column requires a group column in hd_spec().", call. = FALSE)

  list(
    ggplot2::geom_bar(
      stat     = "identity",
      position = "stack",
      width    = 0.7
    ),
    ggplot2::facet_wrap(
      stats::as.formula(paste("~", stack_col)),
      nrow   = 1,
      scales = "free_x"
    ),
    # Remove the x-axis tick labels inside facets - the facet strip
    # title already labels each stack group
    ggplot2::theme(axis.text.x = ggplot2::element_blank())
  )
}

#' @keywords internal
hc_stacked_column <- function(chart, spec, opts, geom_params,
                              use_js = TRUE, ...) {
    # Required args ----------------------------------------------------------------
    stack_col <- geom_params$stack

    # Optional args ----------------------------------------------------------------
    stacking <- geom_params$stacking %||% "normal"

    d <- spec$data
    x_col <- spec$x
    y_col <- spec$y
    grp_col <- spec$group

    if (is.null(grp_col)) {
        stop("stacked_column requires a group column in hd_spec().", call. = FALSE)
    }
    if (is.null(stack_col)) {
        stop("stacked_column requires `stack` argument naming the stack column.",
            call. = FALSE
        )
    }

    # Enable stacking for all column series
    chart <- chart |>
        highcharter::hc_plotOptions(column = list(stacking = stacking))

    #   chart <- chart |>
    #   highcharter::hc_tooltip(
    #     useHTML = TRUE,
    #     format  = paste0(
    #       "<b>{key}</b><br/>",
    #       "{series.name}: {y}<br/>",
    #       "Total: {point.stackTotal}"
    #     )
    #   )

    # -- Key insight: iterate every unique (series, stack) combination --- 
    # The same series name can appear in multiple stacks.
    # Each unique pair produces one hc_add_series() call with its own stack id.
    # Highcharts separates the stacks visually; the legend shows unique names.
    combos <- unique(d[, c(grp_col, stack_col), drop = FALSE])
    pal <- resolve_colors(length(unique(d[[grp_col]])), opts$colors)

    # Named palette so same series name always gets the same colour across stacks
    series_names <- unique(d[[grp_col]])
    color_by_name <- stats::setNames(pal, series_names)

    for (i in seq_len(nrow(combos))) {
        series_name <- combos[[grp_col]][i]
        stack_id <- combos[[stack_col]][i]

        # Rows belonging to this (series, stack) pair, in x-axis order
        mask <- d[[grp_col]] == series_name & d[[stack_col]] == stack_id
        rows <- d[mask, , drop = FALSE]

        # Align to x-axis categories (base_fig sets categories = unique(x))
        # Missing categories get NA so the series stays aligned
        x_cats <- unique(d[[x_col]])
        values <- rows[[y_col]][match(x_cats, rows[[x_col]])]

        chart <- chart |>
            highcharter::hc_add_series(
                name = series_name,
                type = "column",
                data = as.list(values),
                stack = stack_id,
                color = color_by_name[[series_name]],
                # Suppress duplicate legend entries for series that appear in
                # multiple stacks - Highcharts shows the name once per unique name
                showInLegend = !duplicated(combos[[grp_col]])[i]
            )
    }

    chart
}


## -----------------------------------------------------------------------------
## Public geom constructor
## -----------------------------------------------------------------------------

#' Stacked Column Geometry Layer
#'
#' Create a stacked column geometry layer for `hd` objects.  Each stack is a facet
#' (sub-panel) containing one or more series.  The `stack` argument specifies
#' the column in the data that defines the stacks.  The `group` aesthetic in
#' `hd_spec()` defines the series within each stack.  The `stacking`
#' argument controls how the stacks are rendered: `"normal"` (default) stacks values
#' on top of each other, while `"percent"` stacks values as percentages of the total
#' stack height.
#'
#' @param stack Character. Column name for the stack variable.  Each unique value
#' in this column creates a separate stack (facet) containing all series with that
#' stack value.  Required.
#' @param stacking Character. Stacking mode for the column geometry.  One of
#' `"normal"` (default) or `"percent"`.  See Highcharts documentation for
#' details: https://api.highcharts.com/highcharts/plotOptions.column.stacking
#' @inheritParams hd_geom_arearange
#'
#' @return An S3 object of class `"hd_geom"` for use with `+.hd`.
#' @examples
#'
#' # Example data: medal counts for four countries across three medal types
#' olympics <- data.frame(
#'     Country   = rep(c("Norway", "Germany", "United States", "Canada"), each = 3),
#'     Continent = rep(c("Europe", "Europe", "North America", "North America"), each = 3),
#'     Medal     = rep(c("Gold", "Silver", "Bronze"), times = 4),
#'     Count     = c(148, 133, 124, 102, 98, 65, 113, 122, 95, 77, 72, 80)
#' )
#'
#' # Define Specification and Options
#' spec_st <- hd_spec(olympics,
#'     x     = "Medal",
#'     y     = "Count",
#'     group = "Country"
#' )
#'
#' opts_st <- hd_opts(
#'     title    = "Olympic Games all-time medal table, grouped by continent",
#'     subtitle = "Source: Olympics",
#'     ylab     = "Count medals"
#' )
#'
#' # Interactive - stacks are separated by continent
#' hd_make(spec_st, "stacked_column", opts_st, stack = "Continent")
#'
#' # Static ggplot2 - stacks are separated by continent
#' hd(spec_st, backend = "ggplot2") +
#'   hd_geom_stacked_column(stack = "Continent") +
#'   hd_opts(title = "Olympic Games all-time medal table, grouped by continent", ylab = "Count medals")
#'
#' @export
hd_geom_stacked_column <- function(stack, stacking = c("normal", "percent"), ...) {
  # for now, we ignore the `stacking` argument in ggplot2 since it requires more
  # complex data manipulation to implement percent stacking. The Highcharts
  # version supports both modes. So stacking below is mainly for future-proofing
  # and consistency with the Highcharts API and avoid erroring if users specify
  # it in ggplot2 backend.
  stacking <- match.arg(stacking)
  hd_geom("stacked_column", stack = stack, stacking = stacking, ...)
}

#
# Defines the full contract for every registered geometry:
#   required_args - columns the user MUST supply (e.g. ymin/ymax for arearange)
#   optional_args - arguments with sensible defaults (e.g. smooth, dot_size)
#
# HOW THIS FILE IS USED
#   .onLoad() in zzz.R iterates .geom_registry_defs and calls register_geom()
#   for each entry.  Because all R/*.R files are loaded into the package
#   namespace together, .onLoad() can reference .geom_registry_defs directly -
#   no source(), no file path, no environment tricks needed.
#
# ADDING A NEW GEOM
#   1. Add a new entry to .geom_registry_defs below.
#   2. Write gg_<name> and hc_<name> functions in their own R file.
#   3. That is all - zzz.R needs no changes.
#
# RULE OF THUMB: required vs optional
#   required  - geom cannot render at all without it  (e.g. ymin/ymax)
#   optional  - geom works fine with a built-in default (e.g. smooth = TRUE)

#' @keywords internal
.geom_registry_defs <- list(

  # column ---------------------------------------------------------------------
  column = list(
    ggplot_fun      = NULL,   # filled in .onLoad() after namespace is ready
    highcharter_fun = NULL,
    required_args   = character(),
    optional_args   = list()
    # No optional_args: column has no extra knobs beyond spec / opts
  ),

  # line -----------------------------------------------------------------------
  line = list(
    ggplot_fun      = NULL,
    highcharter_fun = NULL,
    optional_args   = list(
      smooth = list(
        default = TRUE,
        desc    = "Logical. TRUE = spline curves, FALSE = straight segments. Both backends.",
        backend_only = NULL # both backends support this argument
      ),
      dot_size = list(
        default = 4L,
        desc    = "Numeric. Marker radius in pixels. Both backends.",
        backend_only = NULL # both backends support this argument
      ),
      # line_symbols is highcharter-only; gg_line silently ignores it.
      line_symbols = list(
        default = NULL,
        desc    = paste0("Character vector. Highcharter only. Per-group marker shapes: ",
                         "'circle','square','diamond','triangle','triangle-down'."),
        backend_only = "highcharter"
      )
    )
  ),

  # scatter --------------------------------------------------------------------
  scatter = list(
    ggplot_fun      = NULL,
    highcharter_fun = NULL,
    optional_args   = list(
      dot_size = list(
        default = 4L,
        desc    = "Numeric. Point size (ggplot2) or marker radius in px (highcharter).",
        backend_only = NULL # both backends support this argument
      )
    )
  ),

  # arearange ------------------------------------------------------------------
  arearange = list(
    ggplot_fun      = NULL,
    highcharter_fun = NULL,
    required_args   = list(
      ymin = list(
        default = NULL,
        desc    = "Character. Column name for the lower bound of the range."
      ),
      ymax = list(
        default = NULL,
        desc    = "Character. Column name for the upper bound of the range."
      )
    ),
    optional_args   = list()
  ),

  # pie ------------------------------------------------------------------------
  pie = list(
    ggplot_fun      = NULL,
    highcharter_fun = NULL,
    optional_args   = list(
      inner_size = list(
        default = "0%",
        desc    = "Character. Inner radius as CSS %, e.g. '50%' for a donut",
        backend_only = "highcharter"
      )
    )
  ),

  # ranked_bar -----------------------------------------------------------------
  ranked_bar = list(
    ggplot_fun      = NULL,
    highcharter_fun = NULL,
    optional_args   = list(
      ascending = list(
        default = TRUE,
        desc    = "Logical. TRUE = lowest bar at bottom, FALSE = highest at bottom. Both backends."
      ),
      vs = list(
        default = NULL,
        desc    = "Character. Category name to highlight with a second colour. Both backends."
      ),
      aim = list(
        default = NULL,
        desc    = "Numeric. Value for a dashed target/aim line. Both backends."
      ),
      char_scale = list(
        default = 0.045,
        desc    = paste0("Numeric. Scaling factor converting label character-count into ",
                         "axis-range units. Increase (e.g. 0.06) for larger text, ",
                         "decrease (e.g. 0.03) for smaller text. Default 0.045."),
        backend_only = "ggplot2"
      ),
      min_frac = list(
        default = 0.08,
        desc    = paste0("Numeric. Minimum fraction of the axis range a bar must span ",
                         "before its label fits inside. Safety floor for short labels. ",
                         "Default 0.08 (8%)."),
        backend_only = "ggplot2"
      )
    )
  ),

  stacked_column = list(
    ggplot_fun      = NULL,
    highcharter_fun = NULL,
    required_args   = list(
      stack = list(
        default = NULL,
        desc    = "Character. Column name that assigns rows to stack groups.",
        backend_only = NULL # both backends require this argument
      )
    ),
    optional_args   = list(
      stacking = list(
        default = "normal",
        desc    = "Character. Highcharter stacking mode: 'normal' or 'percent'.",
        backend_only = "highcharter"
      )
    )
  )

  # map ------------------------------------------------------------------------
  # map = list(                                                                       #
  #   ggplot_fun      = NULL,                                                         #
  #   highcharter_fun = NULL,                                                         #
  #   is_map_geom     = TRUE,                                                         #
  #   optional_args   = list(                                                         #
  #     level = list(                                                                 #
  #       default = "county",                                                         #
  #       desc    = "Character. Map granularity: 'county' or 'municipality'."         #
  #     ),                                                                            #
  #     value_lab = list(                                                             #
  #       default = NULL,                                                             #
  #       desc    = "Character. Colour scale legend label. Defaults to spec$ylab."    #
  #     ),                                                                            #
  #     low_col = list(                                                               #
  #       default = "#C6DBEF",                                                        #
  #       desc    = "Character. Hex colour for the low end of the choropleth scale."  #
  #     ),                                                                            #
  #     high_col = list(                                                              #
  #       default = "#025169",                                                        #
  #       desc    = "Character. Hex colour for the high end of the choropleth scale." #
  #     ),                                                                            #
  #     na_fill = list(                                                               #
  #       default = "#D3D3D3",                                                        #
  #       desc    = "Character. Hex fill colour for regions with no data."            #
  #     )                                                                             #
  #   )                                                                               #
  # )                                                                                 #

)

# @param ascending Logical. If \code{TRUE} (default) bars are sorted in
#   ascending order of \code{y}.
# @param vs Character string (partial match) identifying one category to
#   highlight with a contrasting fill colour (\code{col2}). If omitted all
#   bars use \code{col1}.
# @param char_scale Numeric scaling factor that converts label character-count
#   into axis-range units. Controls how generously space is estimated for each
#   character. Defaults to \code{0.045}; increase (e.g. \code{0.06}) for
#   larger text sizes, decrease (e.g. \code{0.03}) for smaller ones.
# @param min_frac  Numeric. Minimum fraction of the axis range that a bar must
#   span before its label is considered to fit inside. Acts as a safety floor
#   for very short labels. Defaults to \code{0.08} (8 \%).

# --- ggplot2 ---------------------

#' @keywords internal
gg_ranked_bar <- function(spec, opts, geom_params) {

  # -- Extract params ----------------------------------------------------------
  ascending  <- isTRUE(geom_params$ascending %||% TRUE)
  vs         <- geom_params$vs        %||% NULL   # character or NULL: vsarison group name
  aim        <- geom_params$aim         %||% NULL   # numeric or NULL: target line
  char_scale <- geom_params$char_scale  %||% 0.045
  min_frac   <- geom_params$min_frac    %||% 0.08
  sc         <- geom_params$single_colour  # verdien settes i engine
  
  # -- Resolve colours ---------------------------------------------------------
  # col1: default bar colour (single series or non-highlighted bars)
  # col2: highlighted comparison bar colour
  # Both come from the hdir palette so they stay on-brand.
  pal  <- resolve_colors(2L, NULL)   # always need 2: default + highlight
  col1 <- sc %||% pal[1]            # single_colour overrides if set
  col2 <- pal[2]

  # -- Build working data ------------------------------------------------------
  # Work on a copy - do not mutate spec$data
  d        <- spec$data
  x_col    <- spec$x
  y_col    <- spec$y

  # Append N to x labels if spec$n is set
  # spec$n mirrors the `num` argument from the original regbar function
  if (!is.null(spec$n)) {
    d$.xname <- sprintf("%s (N=%s)", d[[x_col]], d[[spec$n]])
  } else {
    d$.xname <- as.character(d[[x_col]])
  }

  # -- Smart label placement ---------------------------------------------------
  # Mirrors the original regbar logic exactly.
  # ypos = 1 -> label inside bar (bar is long enough)
  # ypos = 0 -> label outside bar (bar too short)
  y_range     <- max(d[[y_col]]) - min(0, min(d[[y_col]]))
  label_chars <- nchar(as.character(d[[y_col]]))
  label_width <- label_chars * char_scale * y_range
  threshold   <- pmax(label_width, min_frac * y_range)
  d$ypos      <- ifelse(d[[y_col]] > threshold, 1L, 0L)

  # -- Sort bars ---------------------------------------------------------------
  if (ascending) {
    d$.xname <- factor(d$.xname,
                        levels = d$.xname[order(d[[y_col]])])
  } else {
    d$.xname <- factor(d$.xname,
                        levels = d$.xname[order(d[[y_col]], decreasing = TRUE)])
  }

  # -- Resolve comparison highlight --------------------------------------------
  # vs matches against the original x column (before N= appending)
  # so the user passes the raw category name e.g. vs = "Oslo"
  use_vs <- !is.null(vs) && nzchar(vs)
  if (use_vs) {
    vs_match  <- d$.xname[grepl(vs, d[[x_col]], fixed = TRUE)]
    d$.is_vs  <- d$.xname %in% vs_match
    bar_fill_aes <- ggplot2::aes(fill = .data[[".is_vs"]])
    fill_scale   <- ggplot2::scale_fill_manual(
      values = c("FALSE" = col1, "TRUE" = col2),
      guide  = "none"
    )
  } else {
    bar_fill_aes <- NULL
    fill_scale   <- NULL
  }
  
  # -- Split data for inside / outside labels ----------------------------------
  inside  <- d[d$ypos == 1L, , drop = FALSE]
  outside <- d[d$ypos == 0L, , drop = FALSE]

  # -- Build layer list --------------------------------------------------------
  pos  <- ggplot2::position_dodge(width = 0.80)
  wdth <- 0.80

  layers <- list()

  # Aim line (drawn first so it sits behind bars)
  if (!is.null(aim)) {
    layers <- c(layers, list(
      ggplot2::geom_hline(
        yintercept = aim,
        colour     = resolve_colors(3L, NULL)[3],   # hdir[3] for contrast
        linewidth  = 1,
        linetype   = "dashed"
      )
    ))
  }

  # Bars
  if (use_vs) {
    layers <- c(layers, list(
      ggplot2::geom_bar(
        data     = d,
        mapping  = ggplot2::aes(x = .data[[".xname"]],
                                y = .data[[y_col]],
                                fill = .data[[".is_vs"]]),
        width    = wdth,
        stat     = "identity",
        position = pos
      )
    ))
  } else {
    layers <- c(layers, list(
      ggplot2::geom_bar(
        data    = d,
        mapping = ggplot2::aes(x = .data[[".xname"]],
                               y = .data[[y_col]]),
        width   = wdth,
        stat    = "identity",
        fill    = col1,
        position = pos
      )
    ))
  }

  # Inside labels
  if (nrow(inside) > 0) {
    layers <- c(layers, list(
      ggplot2::geom_text(
        data     = inside,
        mapping  = ggplot2::aes(x     = .data[[".xname"]],
                                y     = .data[[y_col]],
                                label = .data[[y_col]]),
        hjust    = 1.5,
        position = pos,
        size     = 3.5,
        colour   = "#FFFFFF"
      )
    ))
  }

  # Outside labels
  if (nrow(outside) > 0) {
    layers <- c(layers, list(
      ggplot2::geom_text(
        data     = outside,
        mapping  = ggplot2::aes(x     = .data[[".xname"]],
                                y     = .data[[y_col]],
                                label = .data[[y_col]]),
        hjust    = -0.5,
        position = pos,
        size     = 3.5,
        colour   = "#555555"
      )
    ))
  }

  # Fill scale for comparison highlighting
  if (!is.null(fill_scale))
    layers <- c(layers, list(fill_scale))

  # -- Scales, coord, labels ----------------------------------------------------
  # ranked_bar bypasses base_fig() entirely (engine detects geom$name ==
  # "ranked_bar" and skips base_fig).  Therefore it owns ALL ggplot2
  # infrastructure: scales, coord_flip, and labs.

  # Discrete x scale for the sorted .xname factor
  layers <- c(layers, list(
    ggplot2::scale_x_discrete()
  ))

  # Continuous y scale - padding only, no limits.
  #
  # IMPORTANT: limits = ... is intentionally NOT set here.
  # scale_y_continuous(limits) removes data outside the range BEFORE
  # geoms draw.  For bar charts this drops entire rows because the bar
  # baseline (y = 0) is outside the user's range, making bars disappear.
  # ylim is handled by the coord object below (zooms after drawing).
  layers <- c(layers, list(
    ggplot2::scale_y_continuous(
      expand = ggplot2::expansion(mult = c(0, 0.12))
    )
  ))
 
  # Coord - ONE object handles both flip and ylim zoom.
  #
  # ggplot2 allows only one coord per plot.  Adding coord_cartesian() and
  # coord_flip() as separate layers triggers:
  #   "Coordinate system already present. Adding new coordinate system,
  #    which will replace the existing one."
  # and the second coord silently replaces the first - flip is lost when
  # ylim is set, or the ylim zoom is lost when flip replaces cartesian.
  #
  # Solution: coord_flip(ylim = ...) collapses both into one object.
  #   flip=TRUE  + ylim set  -> coord_flip(ylim = opts$ylim)
  #   flip=TRUE  + no ylim   -> coord_flip()           [ylim=NULL is safe]
  #   flip=FALSE + ylim set  -> coord_cartesian(ylim = opts$ylim)
  #   flip=FALSE + no ylim   -> NULL (ggplot2 uses default CartesianCoord)
  do_flip  <- isTRUE(opts$flip %||% TRUE)
  has_ylim <- !is.null(opts$ylim)
 
  coord_layer <- if (do_flip) {
    ggplot2::coord_flip(ylim = opts$ylim)      # ylim=NULL is safe here
  } else if (has_ylim) {
    ggplot2::coord_cartesian(ylim = opts$ylim)
  } else {
    NULL
  }
 
  if (!is.null(coord_layer))
    layers <- c(layers, list(coord_layer))
 
  # Axis labels - opts$xlab / opts$ylab used directly (NULL -> element_blank
  # applied by the engine after theme is set).
  layers <- c(layers, list(
    ggplot2::labs(
      x = opts$xlab,
      y = opts$ylab
    )
  ))

  layers
}


# Highcharter ---------------

#' @keywords internal
hc_ranked_bar <- function(chart, spec, opts, geom_params,
                          use_js = TRUE, ...) {

  ascending  <- isTRUE(geom_params$ascending %||% TRUE)
  vs         <- geom_params$vs               %||% NULL   # character or NULL: vsarison group name
  aim        <- geom_params$aim              %||% NULL   # numeric or NULL: target line

  d     <- spec$data
  x_col <- spec$x
  y_col <- spec$y

  # .xname is always the plain category label.
  # N is attached to each point's data object for the tooltip only.
  d$.xname <- as.character(d[[x_col]])

  # Sort by value
  ord <- order(d[[y_col]], decreasing = ascending)
  d   <- d[ord, ]

  # Colours
  pal  <- resolve_colors(2L, opts$colors)
  col1 <- pal[1]
  col2 <- pal[2]

  # -- Highlight comparison bar ------------------------------------------------
  # grepl must only be called when vs is a non-NULL non-empty string.
  # The & operator in ifelse() does NOT short-circuit, so
  # ifelse(use_vs & grepl(vs, ...), ...) evaluates grepl(NULL, ...)
  # when comp is NULL -> "invalid 'pattern' argument" error.
  use_vs <- !is.null(vs) && nzchar(vs)

  if (use_vs) {
    is_vs    <- grepl(vs, d[[x_col]], fixed = TRUE)
    colors_vec <- ifelse(is_vs, col2, col1)
  } else {
    colors_vec <- rep(col1, nrow(d))
  }

  # Each point carries separate fields for name, y, and n_obs.
  # The tooltip pointFormat references {point.name}, {point.y},
  # and {point.n_obs} individually - Highcharts renders them cleanly
  # without cluttering the axis label.
  # n_obs is only added to the point when spec$n is set.
  point_data <- lapply(seq_len(nrow(d)), function(i) {
    pt <- list(
      name  = d$.xname[i],
      y     = d[[y_col]][i],
      color = colors_vec[i]
    )
    if (!is.null(spec$n)) {
      pt$n_obs <- d[[spec$n]][i]
    }
    pt
  })

  # x categories in sorted order (plain labels, no N= suffix)
  chart <- chart |>
    highcharter::hc_xAxis(
      categories = d$.xname,
      title      = list(text = opts$xlab)
    )

  # Aim line as plot line on y-axis
  if (!is.null(aim)) {
    chart <- chart |>
      highcharter::hc_yAxis(
        plotLines = list(list(
          value     = aim,
          color     = resolve_colors(3L, opts$colors)[3],
          width     = 2,
          dashStyle = "Dash",
          zIndex    = 5
        ))
      )
  }

  # Ranked_bar sets its own tooltip via hc_tooltip() here.
  # This overrides the engine-level tooltip for this geom only.
  # pointFormat shows value and - when spec$n is set - N on a
  # second line. When spec$n is NULL the N line is omitted entirely.
  point_fmt <- if (!is.null(spec$n)) {
    paste0(
      '<span style="color:{point.color}">\u25CF</span> ',
      '<b>{point.name}</b><br/>',
      opts$ylab %||% y_col, ': <b>{point.y}</b><br/>',
      'N: <b>{point.n_obs}</b>'
    )
  } else {
    paste0(
      '<span style="color:{point.color}">\u25CF</span> ',
      '<b>{point.name}</b><br/>',
      opts$ylab %||% y_col, ': <b>{point.y}</b>'
    )
  }

  chart <- chart |>
    highcharter::hc_tooltip(
      useHTML      = TRUE,
      shared       = FALSE,   # ranked bar: one bar per hover, not shared
      headerFormat = "",      # name already in pointFormat
      pointFormat  = point_fmt
    )

  # Add series
  # DataLabels disabled entirely. Value and N both live in
  # the tooltip which is always readable regardless of bar length.
  #
  # Highcharts series type vs flip:
  #   type = "bar"    is natively horizontal - ignores hc_chart(inverted).
  #   type = "column" is natively vertical   - respects hc_chart(inverted).
  #
  # base_fig() sets hc_chart(inverted = isTRUE(opts$flip)).
  # For ranked_bar we always use type = "column" and rely on inverted to
  # control orientation - this makes flip = TRUE/FALSE work correctly.
  #
  #   flip = TRUE  (default) -> inverted = TRUE  -> horizontal bars
  #   flip = FALSE           -> inverted = FALSE -> vertical bars
  chart <- chart |>
    highcharter::hc_chart(inverted = isTRUE(opts$flip %||% TRUE)) |>
    highcharter::hc_add_series(
      type         = "column",
      name         = opts$ylab %||% y_col,
      data         = point_data,
      showInLegend = FALSE,
      dataLabels   = list(enabled = FALSE)
    )

  chart
}

## -----------------------------------------------------------------------------
## Public constructor for ranked bar geometry
## -----------------------------------------------------------------------------

#' Ranked bar geometry
#'
#' Draws a ranked bar chart with smart label placement and optional comparison
#' highlighting.  Bars are sorted by value, and labels are placed inside or
#' outside the bar depending on available space.  A single category can be
#' highlighted with a contrasting fill colour, and an optional horizontal line
#' can be added to indicate a target or benchmark value.
#'
#' @param ascending Logical. If `TRUE` (default) bars are sorted in
#'  ascending order of `y`.
#' @param vs Character string (partial match) identifying one category to
#'  highlight with a contrasting fill colour. If omitted all bars use the same
#'  colour.
#' @param aim Numeric. Optional horizontal line indicating a target or benchmark
#'  value.  If `NULL` (default) no line is drawn.
#' @param char_scale Numeric scaling factor that converts label character-count
#'  into axis-range units. Controls how generously space is estimated for each
#'  character. Defaults to `0.045`; increase (e.g. `0.06`0.06)
#'  for larger text sizes, decrease (e.g. `0.03`) for smaller ones.
#' @param min_frac  Numeric. Minimum fraction of the axis range that a bar must
#'  span before its label is considered to fit inside. Acts as a safety floor
#'  for very short labels. Defaults to `0.08` (8%).
#' @inheritParams hd_geom_column
#'
#' @return An S3 object of class `"hd_geom"` for use with `+.hd`.
#'
#' @examples
#' # Regional health indicator dataset
#' regions <- data.frame(
#'   region = c("Oslo", "Viken", "Vestland", "Rogaland",
#'              "Trondelag", "Innlandet", "Agder",
#'              "Nordland", "Troms og Finnmark"),
#'   rate   = c(68.4, 71.2, 87.8, 10.5, 61.3, 6.1, 54.2, 49.8, 42.1),
#'   n      = c(402, 448, 681, 318, 297, 251, 198, 177, 148)
#' )
#'
#' # Declarative API ----
#' spec_rb <- hd_spec(regions,
#'                   x    = "region",
#'                   y    = "rate",
#'                   n    = "n")
#'
#' opts_rb <- hd_opts(
#'  title    = "Health indicator by region",
#'  subtitle = "Source: Norwegian Directorate of Health",
#'  ylab     = "Rate per 100 000",
#'  flip     = TRUE
#' )
#'
#' hd_make(spec_rb, "ranked_bar", opts_rb, vs = "Oslo", aim = 63)
#'
#' # Layered API ----
#' hd(regions, x = "region", y = "rate", n = "n", backend = "ggplot2") +
#'  hd_geom_ranked_bar(
#'   ascending  = TRUE,
#'   vs         = "Oslo",
#'   aim        = 63,
#'   char_scale = 0.045,
#'   min_frac   = 0.08) +
#'  hd_opts(
#'  title    = "Health indicator by region",
#'  subtitle = "Source: Norwegian Directorate of Health",
#'  ylab     = "Rate per 100 000",
#'  flip     = TRUE
#' )
#'
#' @family Geoms
#' @seealso [hd_geom_column()], [hd_geom_line()], [hd_geom_arearange()],
#'  [hd_opts()], [hd_make()]
#'
#' @export
hd_geom_ranked_bar <- function(ascending = TRUE,
                               vs = NULL,
                               aim = NULL,
                               char_scale = 0.045,
                               min_frac = 0.08, ...) {
  hd_geom("ranked_bar",
          ascending = ascending,
          vs = vs,
          aim = aim,
          char_scale = char_scale,
          min_frac = min_frac, ...)
}

# Blank canvas constructors
#
# .base_constructors is a dispatch table (named list) keyed by backend name.
# Each entry is a function(spec, opts) that returns an empty backend object
# with axes, labels, and chart-level options already applied.
#
# Adding a new backend = adding one entry here.  No if/else chains.

#' @keywords internal
.base_constructors <- list(

  # -- ggplot2 -----------------------------------------------------------------
  ggplot2 = function(spec, opts) {

    # -- Coerce group column to factor so ggplot2 treats it as discrete --------
    # If the group column is numeric (1, 2, 3...) ggplot2 maps it as a
    # continuous variable. scale_color_manual is discrete-only and errors.
    # Converting to factor here fixes the aesthetic type before any layer
    # or scale is added - the fix applies to all geoms automatically.
    plot_data <- spec$data
    grp_col   <- spec$colour %||% spec$group
    if (!is.null(grp_col) && is.numeric(plot_data[[grp_col]])) {
      plot_data[[grp_col]] <- as.factor(plot_data[[grp_col]])
    }

    mapping <- ggplot2::aes(x = .data[[spec$x]], y = .data[[spec$y]])

    if (!is.null(grp_col)) {
      mapping <- utils::modifyList(mapping, ggplot2::aes(
        colour = .data[[grp_col]],
        group  = .data[[grp_col]],
        fill   = .data[[grp_col]]
      ))
    }

    # use plot_data not spec$data incase factoring to grp_col
    p <- ggplot2::ggplot(plot_data, mapping) + 
        ggplot2::labs(
            x        = opts$xlab,
            y        = opts$ylab,
            title    = opts$title,
            subtitle = opts$subtitle,
            caption  = opts$caption
        )

    # NOTE: axis.title hiding (element_blank) is NOT applied here.
    # It is applied in ggplot_engine() AFTER gt$theme so it always wins
    # regardless of which ggplot2 theme was chosen.  Applying it here and
    # then adding a theme on top would silently undo the hiding.

    # if (!is.null(opts$ylim))
    # p <- p + ggplot2::scale_y_continuous(limits = opts$ylim)

    # if (!is.null(opts$yint))
    #   p <- p + ggplot2::geom_hline(
    #     yintercept = opts$yint,
    #     linetype   = "dashed",
    #     colour     = "#AAAAAA")

    # if (isTRUE(opts$flip))
    #   p <- p + ggplot2::coord_flip()

    p
  },

  # -- highcharter -------------------------------------------------------------
  highcharter = function(spec, opts) {
    
    # --- percent % symbol ---
    pros_fmt <- if (is.null(opts$ysuffix)){
      list(format = "{value}")
    } else {
      list(format = paste0("{value}", opts$ysuffix))
    }

    # NOTE: %||% "" is intentional for both axis title fields.
    # R silently drops NULL entries from lists: list(text = NULL) -> list().
    # Highcharts then sees an empty title object and falls back to "Value".
    # Passing "" serialises as {"text": ""}, which Highcharts correctly
    # renders as a hidden title -- matching the NULL -> hide contract
    # enforced upstream by .resolve_axis_label().

    chart <- highcharter::highchart() |>
      highcharter::hc_chart(inverted = isTRUE(opts$flip)) |>
      highcharter::hc_yAxis(
        title        = list(text = opts$ylab %||% ""),
        labels       = pros_fmt,
        tickInterval = opts$yint,
        min          = if (!is.null(opts$ylim)) opts$ylim[1] else 0,
        max          = if (!is.null(opts$ylim)) opts$ylim[2] else NULL
      )

    # x-axis: categories for character, numeric labels otherwise
    if (!is.numeric(spec$data[[spec$x]])) {
      chart <- chart |> highcharter::hc_xAxis(
        title        = list(text = opts$xlab %||% ""),
        categories   = unique(spec$data[[spec$x]]),
        tickInterval = 1,
        labels       = list(step = 1)
      )
    } else {
      chart <- chart |> highcharter::hc_xAxis(
        title        = list(text = opts$xlab %||% ""),
        labels = list(step = 1)
      )
    }

    # x-tick: Should it be different than x-axis
    if (!is.null(opts$xtick_labels)){
      if (is.numeric(spec$data[[spec$x]])){
        message("Just a reminder, highchart index starts from 0")
      }
      chart <- chart |> highcharter::hc_xAxis(categories = spec$data[[opts$xtick_labels]])
    }

    if (!is.null(opts$title))
      chart <- chart |> highcharter::hc_title(text = opts$title)

    chart <- chart |>
      highcharter::hc_subtitle(
        text = opts$subtitle %||% "Kilde: Navn av kilder"
      )

    if (!is.null(opts$caption))
      chart <- chart |> highcharter::hc_caption(text = opts$caption)

    # Accessibility description - read by screen readers via the Highcharts
    # accessibility module (loaded automatically in highcharter_engine).
    # NULL means the module is still active (keyboard nav, ARIA roles) but
    # no explicit figure description is announced.
    # .hd_accessibility() is used instead of highcharter::hc_accessibility()
    # because hc_accessibility() is not exported in highcharter 0.9.4.
    # The wrapper tries the exported function first and falls back to patching
    # chart$x$hc_opts directly - see utils.R.
    if (!is.null(opts$description))
      chart <- .hd_accessibility(chart, opts$description)

    chart
  }
)

#' Build a Blank Backend Canvas
#'
#' Constructs the empty backend object (ggplot or highchart) with all
#' chart-level options applied from `spec` and `opts`. Called by the
#' backend engines; you rarely need this directly.
#'
#' @param spec    A [hd_spec] object.
#' @param opts    A [hd_opts] object.
#' @param backend Character. Backend name.
#' @return A `ggplot` or `highchart` object.
#' @keywords internal
base_fig <- function(spec, opts, backend) {

  ## Resolve axis labels
  opts$xlab <- .resolve_axis_label(opts$xlab, spec$x)
  opts$ylab <- .resolve_axis_label(opts$ylab, spec$y)

  ctor <- .base_constructors[[backend]]
  if (is.null(ctor))
    stop("No base constructor registered for backend '", backend, "'.",
         call. = FALSE)
  ctor(spec, opts)
}

# Centralised pre-render validation
#
# All checks that need to happen before a figure is rendered live here.
# Having one function keeps error messages consistent and easy to update.

#' Validate inputs before rendering a figure
#'
#' Called by [hd_make()] immediately before dispatching to a backend engine.
#' Stops with an informative message if anything is wrong.
#'
#' @param spec        A `hd_spec` object.
#' @param opts        A `hd_opts` object.
#' @param type        Character. Geometry name.
#' @param backend     Character. Backend name.
#' @param extra_args  Named list of additional arguments (for required-arg
#'   check).
#'
#' @return `invisible(NULL)` on success.
#' @keywords internal
validate_fig_inputs <- function(spec, opts, type, backend, extra_args) {

  if (!inherits(spec, "hd_spec"))
    stop("`spec` must be a hd_spec object created by hd_spec().",
         call. = FALSE)

  if (!inherits(opts, "hd_opts"))
    stop("`opts` must be a hd_opts object created by hd_opts().",
         call. = FALSE)

  geom <- .get_geom(type)
  if (is.null(geom))
    stop("Unknown geometry '", type, "'. Available: ",
         paste(sort(list_geoms()), collapse = ", "), call. = FALSE)

  engine <- get_backend(backend)
  if (is.null(engine))
    stop("Unknown backend '", backend, "'. Available: ",
         paste(sort(list_backends()), collapse = ", "), call. = FALSE)

  validate_geom_args(geom, extra_args)

  invisible(NULL)
}

# -- Geom args validation --------------------------------------------

#' @keywords internal
validate_geom_args <- function(geom, extra_args) {
  # Only required_args are checked here
  missing_args <- setdiff(names(geom$required_args), names(extra_args))
  if (length(missing_args) > 0)
    stop("Missing required argument(s) for geometry '", geom$name, "': ",
         paste(missing_args, collapse = ", "),
         ".  Run geom_args('", geom$name, "') to see all arguments.",
         call. = FALSE)
}

#' Alcohol Consumption Data
#'
#' Annual estimates of alcohol consumption, including adjusted mean values,
#' standard errors, and 95% confidence intervals. Includes an alternative
#' unit (`adj_enhet`) with corresponding uncertainty estimates.
#'
#' @format A data frame with 14 rows and 9 variables:
#'
#' @source Helsedirektoratet. \emph{Tall om alkohol}.
#'   \url{https://www.helsedirektoratet.no/rapporter/tall-om-alkohol}
#'
#' @usage NULL
#' 
"alco1"

#' Alcohol Consumption by Gender
#'
#' Annual estimates of alcohol consumption stratified by \enc{kjønn}{kjonn}
#' (gender), including adjusted means, standard errors, and 95\% confidence
#' intervals. Also includes an alternative unit (`adj_enhet`) with uncertainty.
#'
#' @format A data frame with 28 rows and 10 variables:
#'
#' @source Helsedirektoratet. \emph{Tall om alkohol}.
#'   \url{https://www.helsedirektoratet.no/rapporter/tall-om-alkohol}
#'
#' @usage NULL
#'
"alco2"

#
#' Create Figure Presentation Options
#'
#' Defines the **visual presentation** of a figure independently from the data
#' mapping.  Pass the result as the `opts` argument of [hd_make()], or omit
#' it to accept all defaults.
#'
#' Because opts are separate from [hd_spec()], the same data mapping can
#' be rendered with multiple styles without repetition:
#'
#' ```r
#' spec    <- hd_spec(df, "age", "pct", group = "sex")
#' opts_en <- hd_opts(title = "Health survey",    subtitle = "All ages")
#' opts_no <- hd_opts(title = "Helseundersøkelse", subtitle = "Alle aldre")
#'
#' hd_make(spec, "column", opts_en)
#' hd_make(spec, "column", opts_no)
#' ```
#'
#' @param title       Character or `NULL`. Chart title.
#' @param subtitle    Character or `NULL`. Subtitle. Highcharter default:
#'   `"Kilde: Navn av kilder"`.
#' @param caption     Character or `NULL`. Caption text displayed below the
#'   figure (highcharter only). This is a **visible** footnote, distinct from
#'   `description` which is read only by assistive technology.
#' @param description Character or `NULL`. Invisible accessibility description
#'   of the figure intended for screen readers and other assistive technology.
#'   \describe{
#'     \item{highcharter}{Passed to `hc_accessibility(description = ...)`.
#'       Requires the accessibility module, which highdir loads automatically.
#'       Screen readers announce this text when the user focuses the chart.}
#'     \item{ggplot2}{Set as the `alt` label via `labs(alt = ...)`, available
#'       since ggplot2 3.3.0.  Rendered as an HTML `alt` attribute when the
#'       plot is saved to SVG or included in an R Markdown / Quarto document
#'       with `fig.alt` support.}
#'   }
#'   Write a concise one- or two-sentence summary of what the figure shows,
#'   including the key trend or comparison, so the information is equally
#'   accessible to users who cannot see the chart.  Example:
#'   `"Bar chart showing alcohol use by age group. Use is highest in the
#'   45-54 age group at 65% and lowest in 18-24 at 42%."`
#' @param xlab        Character or `NULL`. X-axis label. \describe{
#'   \item{`" "` (default)}{Use the `x` column name from [hd_spec()].}
#'   \item{`NULL`}{Hide the x-axis label completely.}
#'   \item{any string}{Use that string as the label.} }
#' @param ylab        Character or `NULL`. Y-axis label. Same rules as `xlab`.
#' @param ylim        Numeric vector of length 2 or `NULL`. Fixed y-axis
#'   limits, e.g. `c(0, 100)`.
#' @param yint        Positive numeric. Y-axis tick interval. Default `10`.
#' @param ysuffix     Character or `NULL`. String appended to every y-axis
#'   tick label, e.g. `"%"` or `" km"`. `NULL` shows no suffix.
#' @param xtick_labels Character or `NULL`. Column name supplying custom
#'   x-axis tick labels when the plotted x-values are numeric but the
#'   displayed labels should come from another column. Highcharter only.
#'   Note: Highcharts indexes categories from 0, not 1.
#' @param decimals    Integer or `NULL`. Number of decimal places to round
#'   the y-values to before rendering. Applied to the data via
#'   `check_decimals()` in [hd_make()]. `NULL` leaves values unchanged.
#' @param flip        Logical. Invert axes (horizontal bars / inverted chart).
#'   Default `FALSE`.
#' @param colors      Character vector, palette name string, or `NULL`.
#'   Per-figure colour override; takes precedence over [hd_set_theme()].
#' @param hc_theme    Character or `NULL`. Per-figure highcharter theme name;
#'   takes precedence over [hd_set_theme()]. See [hd_theme()] for valid names.
#' @param gg_theme    Character name string, ggplot2 theme object, or `NULL`.
#'   Per-figure ggplot2 theme; takes precedence over [hd_set_theme()].
#'   Name strings: `"classic"` (default), `"minimal"`, `"bw"`, `"light"`,
#'   `"dark"`, `"void"`, `"grey"`. Or pass a `ggplot2::theme_*()` object
#'   directly, e.g. `ggplot2::theme_bw(base_size = 14)`.
#'
#' @return An S3 object of class `"hd_opts"`.
#'
#' @seealso [hd_spec()], [hd_make()], [hd_set_theme()]
#'
#' @examples
#' opts <- hd_opts(
#'   title       = "Health survey results",
#'   subtitle    = "Source: FHI 2024",
#'   caption     = "Tall om helse",
#'   description = paste(
#'     "Grouped bar chart showing alcohol use by age group and sex.",
#'     "Use is highest in the 45-54 age group at 65% for women."
#'   ),
#'   ylim        = c(0, 100),
#'   yint        = 20,
#'   colors      = c("#025169", "#7C145C")
#' )
#' opts
#'
#' @export
hd_opts <- function(title        = NULL,
                    subtitle     = NULL,
                    caption      = NULL,
                    description  = NULL,
                    xlab         = " ",
                    ylab         = " ",
                    ylim         = NULL,
                    yint         = 10,
                    ysuffix      = NULL,
                    xtick_labels = NULL,
                    decimals     = NULL,
                    flip         = FALSE,
                    colors       = NULL,
                    hc_theme     = NULL,
                    gg_theme     = NULL
                    ) {

  check_ylim(ylim)
  if (!is.numeric(yint) || length(yint) != 1 || yint <= 0)
    stop("`yint` must be a single positive number.", call. = FALSE)
  if (!is.null(description) && (!is.character(description) || length(description) != 1L))
    stop("`description` must be a single character string or NULL.", call. = FALSE)

  structure(
    list(title        = title,
         subtitle     = subtitle,
         caption      = caption,
         description  = description,
         xlab         = xlab,
         ylab         = ylab,
         ylim         = ylim,
         yint         = yint,
         ysuffix      = ysuffix,
         xtick_labels = xtick_labels,
         decimals     = decimals,
         flip         = flip,
         colors       = colors,
         hc_theme     = hc_theme,
         gg_theme     = gg_theme
         ),
    class = "hd_opts"
  )
}

# -- S3 methods ----------------------------------------------------------------

#' @export
print.hd_opts <- function(x, ...) {
  cat("<hd_opts>\n")
  if (!is.null(x$title))       cat("  title        :", x$title,       "\n")
  if (!is.null(x$subtitle))    cat("  subtitle     :", x$subtitle,    "\n")
  if (!is.null(x$caption))     cat("  caption      :", x$caption,     "\n")
  if (!is.null(x$description)) {
    # Truncate long descriptions in print output for readability
    desc_print <- if (nchar(x$description) > 80)
      paste0(substr(x$description, 1, 77), "...")
    else
      x$description
    cat("  description  :", desc_print, "\n")
  }
  if (!is.null(x$xlab))        cat("  xlab         :", x$xlab,        "\n")
  if (!is.null(x$ylab))        cat("  ylab         :", x$ylab,        "\n")
  if (!is.null(x$ylim))        cat("  ylim         :", x$ylim,        "\n")
  cat("  yint         :", x$yint, "\n")
  if (!is.null(x$ysuffix))     cat("  ysuffix      :", x$ysuffix,     "\n")
  if (!is.null(x$xtick_labels)) cat("  xtick_labels :", format(x$xtick_labels), "\n")
  if (!is.null(x$decimals))    cat("  decimals     :", x$decimals,    "\n")
  cat("  flip         :", x$flip, "\n")
  if (!is.null(x$colors))
    cat("  colors       :", paste(x$colors, collapse = ", "), "\n")
  if (!is.null(x$hc_theme))    cat("  hc_theme     :", x$hc_theme,    "\n")
  if (!is.null(x$gg_theme))    cat("  gg_theme     :", format(x$gg_theme), "\n")
  invisible(x)
}

#' @export
as.list.hd_opts <- function(x, ...) unclass(x)

# -- Internal default ----------------------------------------------------------

#' @keywords internal
default_opts <- function() hd_opts()

# ── hd_spec ──────────────────────────────────────────────────────────────────
test_that("hd_spec: no longer accepts xlab/ylab", {
  expect_error(
    hd_spec(data.frame(x=1, y=1), "x", "y", ylab = "Rate"),
    "unused argument"
  )
})

test_that("hd_spec: constructs with correct class and fields", {
  spec <- hd_spec(survey_df, x = "age", y = "pct")
  expect_s3_class(spec, "hd_spec")
  expect_equal(spec$x,    "age")
  expect_equal(spec$y,    "pct")
  expect_null(spec$group)
  expect_null(spec$n)
  expect_equal(nrow(spec$data), nrow(survey_df))
})

test_that("hd_spec: rejects non-data.frame", {
  expect_error(hd_spec(list(x = 1), "x", "x"), "data.frame")
})

test_that("hd_spec: stops on missing columns", {
  expect_error(hd_spec(survey_df, "age", "missing"),       "missing")
  expect_error(hd_spec(survey_df, "age", "pct", group = "nope"), "nope")
  expect_error(hd_spec(survey_df, "age", "pct", n = "nope"),     "nope")
})

test_that("hd_spec: strips tibble subclass (stores plain data.frame)", {
  skip_if_not_installed("tibble")
  tbl  <- tibble::as_tibble(survey_df)
  spec <- hd_spec(tbl, "age", "pct")
  expect_true(is.data.frame(spec$data))
  expect_false(inherits(spec$data, "tbl_df"))
})

test_that("hd_spec: print output contains key fields", {
  spec <- hd_spec(survey_df, "age", "pct", group = "sex", n = "n")
  expect_output(print(spec), "hd_spec")
  expect_output(print(spec), "group")
  expect_output(print(spec), "rows")
})

test_that("hd_spec: as.list replaces data with descriptive string", {
  spec <- hd_spec(survey_df, "age", "pct")
  lst  <- as.list(spec)
  expect_true(is.character(lst$data))
  expect_match(lst$data, "data.frame")
})

# ── fig_opts ──────────────────────────────────────────────────────────────────

test_that("hd_opts: constructs with correct defaults", {
  opts <- hd_opts()
  expect_s3_class(opts, "hd_opts")
  expect_null(opts$title)
  expect_equal(opts$yint, 10)
  expect_false(opts$flip)
})

test_that("hd_opts: stores all fields", {
  opts <- hd_opts(title = "T", subtitle = "S", caption = "C",
                   ylim = c(0, 100), yint = 20, flip = TRUE,
                   colors = "#FF0000", hc_theme = "bloom")
  expect_equal(opts$title,    "T")
  expect_equal(opts$ylim,     c(0, 100))
  expect_equal(opts$yint,     20)
  expect_true(opts$flip)
  expect_equal(opts$hc_theme, "bloom")
})

test_that("hd_opts: validates ylim", {
  expect_error(hd_opts(ylim = c(100, 0)),  "ylim\\[1\\]")
  expect_error(hd_opts(ylim = c(0)),        "length 2")
  expect_silent(hd_opts(ylim = c(0, 100)))
})

test_that("hd_opts: validates yint", {
  expect_error(hd_opts(yint = 0),  "positive")
  expect_error(hd_opts(yint = -5), "positive")
  expect_silent(hd_opts(yint = 1))
})

test_that("hd_opts: print works", {
  opts <- hd_opts(title = "Test", ylim = c(0, 80))
  expect_output(print(opts), "hd_opts")
  expect_output(print(opts), "title")
  expect_output(print(opts), "ylim")
})

test_that("hd_opts: as.list returns plain list", {
  opts <- hd_opts(title = "T", yint = 5)
  lst  <- as.list(opts)
  expect_type(lst, "list")
  expect_equal(lst$title, "T")
  expect_equal(lst$yint,  5)
})

test_that("default_opts returns hd_opts with defaults", {
  expect_s3_class(default_opts(), "hd_opts")
  expect_equal(default_opts()$yint, 10)
})

test_that("hd_opts: default ylab is sentinel ' '", {
  opts <- hd_opts()
  expect_equal(opts$ylab, " ")
})

test_that("hd_opts: NULL ylab preserved as NULL", {
  opts <- hd_opts(ylab = NULL)
  expect_null(opts$ylab)
})

test_that("hd_opts: custom ylab stored correctly", {
  opts <- hd_opts(ylab = "Rate per 100 000")
  expect_equal(opts$ylab, "Rate per 100 000")
})

# R/palettes.R -- Named colour palette registry
#
# Palettes are stored in a plain environment (.palette_registry).  Built-in
# palettes are registered in .onLoad() (see zzz.R).  Users and extension
# packages can add their own with register_palette().
#
# resolve_colors() is the *single* function every geom calls to obtain a
# colour vector - it respects both explicit overrides and session defaults.

# -- Registry environment ------------------------------------------------------

#' @keywords internal
.palette_registry <- new.env(parent = emptyenv())

# -- Public API ----------------------------------------------------------------

#' Register a Named Colour Palette
#'
#' Adds a named palette to the highdir palette registry so it can be
#' referenced by name wherever colours are accepted
#' (e.g. `hd_opts(colors = "my_palette")`). This function is evaluated
#' when loading a file in zzz.R file.
#'
#' Built-in palettes registered at package load time:
#'
#' | Name     | Description                               |
#' |:---------|:------------------------------------------|
#' | `"hdir"` | Helsedirektoratet 10-colour brand palette |
#' | `"hdir2"`| 2-colour teal / purple pair               |
#'
#' @param name   Character. Unique palette identifier.
#' @param colors Non-empty character vector of CSS/hex colour strings.
#'
#' @return `name`, invisibly.
#'
#' @examples
#' register_palette("blues", c("#084594", "#2171b5", "#4292c6", "#6baed6"))
#' get_palette("blues")
#'
#' @export
register_palette <- function(name, colors) {
  if (!is.character(colors) || length(colors) == 0)
    stop("`colors` must be a non-empty character vector.", call. = FALSE)
  .palette_registry[[name]] <- colors
  invisible(name)
}

#' List Registered Palettes
#'
#' @return Character vector of registered palette names.
#' @examples
#' list_palettes()
#' @export
list_palettes <- function() sort(ls(.palette_registry))

#' Retrieve a Named Palette
#'
#' @param name Character. Palette name (see [list_palettes()]).
#' @return Character vector of colours, or `NULL` if not found.
#' @examples
#' get_palette("hdir")
#' @export
get_palette <- function(name) .palette_registry[[name]]

# -- Colour resolution (internal) ----------------------------------------------
# R/palettes.R

#' Resolve a Colour Vector for n Groups
#'
#' Returns exactly `n` colours. Priority order:
#'
#' 1. Explicit `colors` argument - vector or palette name string.
#' 2. `getOption("highdir.colors")` - set via [hd_set_theme()].
#' 3. Built-in hdir rules:
#'    * n == 2  -> `"hdir2"` two-colour teal/purple pair
#'    * n <= 10 -> `"hdir"` 10-colour brand palette
#'    * n > 10  -> viridis continuous scale
#'
#' @param n      Integer. Number of colours required.
#' @param colors Character vector, palette name, or `NULL`.
#' @return Character vector of exactly length `n`.
#' @keywords internal
resolve_colors <- function(n, colors = NULL) {

  # -- Priority 1 + 2: explicit or session-level override ----------------------
  candidate <- colors %||% getOption("highdir.colors", default = NULL)

  if (!is.null(candidate)) {
    # Resolve palette name string -> colour vector
    if (is.character(candidate) && length(candidate) == 1 &&
        candidate %in% list_palettes()) {
      candidate <- get_palette(candidate)
    }
    if (length(candidate) >= n)
      return(candidate[seq_len(n)])

    warning(
      "Supplied palette has ", length(candidate), " colour(s) but ",
      n, " are needed. Falling back to built-in rules.",
      call. = FALSE
    )
  }

  # -- Priority 3: built-in n-aware rules --------------------------------------

  # Rule A - exactly 2 groups: dedicated high-contrast pair
  if (n == 2) {
    pal2 <- get_palette("hdir2")
    if (!is.null(pal2) && length(pal2) >= 2)
      return(pal2[seq_len(n)])
  }

  # Rule B - up to 10 groups: first n from hdir
  hdir_pal <- get_palette("hdir")
  if (!is.null(hdir_pal) && n <= length(hdir_pal))
    return(hdir_pal[seq_len(n)])

  # Rule C - more than 10 groups: viridis continuous interpolation
  if (!requireNamespace("viridis", quietly = TRUE))
    stop(
      n, " colours requested but hdir only has 10 and {viridis} is not ",
      "installed.\nInstall it: install.packages('viridis')",
      call. = FALSE
    )
  viridis::viridis(n, option = "D")
}

# tests/testthat/test-palettes.R

test_that("resolve_colors: correct length for various n", {
  skip_if_not_installed("viridis")
  for (n in c(1, 2, 5, 10, 14)) {
    result <- resolve_colors(n)
    expect_equal(
      length(result), n,
      label = paste0("resolve_colors(", n, ")")
    )
  }
})

test_that("resolve_colors: n=2 returns hdir2 not hdir[1:2]", {
  result <- resolve_colors(2)
  hdir2  <- get_palette("hdir2")
  expect_equal(result, hdir2[1:2])
  expect_false(identical(result, get_palette("hdir")[1:2]))
})

test_that("resolve_colors: n=1 returns first hdir colour", {
  result <- resolve_colors(1)
  expect_equal(length(result), 1L)
  expect_equal(result, get_palette("hdir")[1])
})

test_that("resolve_colors: n=10 returns full hdir palette", {
  result <- resolve_colors(10)
  expect_equal(result, get_palette("hdir"))
})

test_that("resolve_colors: n=11 uses viridis with no duplicates", {
  skip_if_not_installed("viridis")
  result <- resolve_colors(11)
  expect_length(result, 11)
  expect_equal(length(unique(result)), 11)
})

test_that("resolve_colors: explicit colours used when long enough", {
  cols   <- c("#AA0000", "#00AA00", "#0000AA")
  result <- resolve_colors(3, colors = cols)
  expect_equal(result, cols)
})

test_that("resolve_colors: too-short explicit palette warns and falls through", {
  expect_warning(
    result <- resolve_colors(5, colors = c("#FF0000", "#00FF00")),
    regexp = "2 colour"
  )
  expect_equal(result, get_palette("hdir")[1:5])
})

test_that("resolve_colors: palette name string resolved correctly", {
  result <- resolve_colors(3, colors = "hdir")
  expect_equal(result, get_palette("hdir")[1:3])
})

test_that("resolve_colors: all returned values are character strings", {
  for (n in c(1, 2, 5, 10)) {
    result <- resolve_colors(n)
    expect_type(result, "character")
  }
})

test_that("all registered palette colours are valid R colour strings", {
  for (name in list_palettes()) {
    pal <- get_palette(name)
    for (col in pal) {
      expect_true(
        tryCatch({ grDevices::col2rgb(col); TRUE }, error = function(e) FALSE),
        label = paste0("palette '", name, "' colour '", col,
                       "' must be a valid R colour")
      )
    }
  }
})

test_that("hdir2 colours parse in both ggplot2 and as hex", {
  hdir2 <- get_palette("hdir2")
  expect_equal(length(hdir2), 2L)
  # Both must be valid R colours
  expect_no_error(grDevices::col2rgb(hdir2[1]))
  expect_no_error(grDevices::col2rgb(hdir2[2]))
  # Both should be hex format
  expect_true(all(grepl("^#[0-9A-Fa-f]{6}$", hdir2)))
})

# R/highdir-package.R -- Package-level documentation and global imports

#' highdir: Backend-Agnostic Figure Builder for Helsedirektoratet
#'
#' @description
#' **highdir** provides a unified API for building interactive figures with
#' **highcharter** or static figures with **ggplot2**.  A figure is described
#' once as a data-mapping object ([hd_spec]) and an optional presentation
#' object ([hd_opts]), then rendered to either backend without changing the
#' calling code.
#'
#' ## Core workflow
#'
#' ```r
#' library(highdir)
#'
#' # -- 1. Sample data --------------------------------------------------------
#' df <- data.frame(
#'   age = rep(c("18-24", "25-34", "35-44", "45-54"), each = 2),
#'   sex = rep(c("Male", "Female"), 4),
#'   pct = c(42, 38, 55, 61, 48, 52, 60, 57),
#'   n   = c(120, 115, 200, 210, 180, 175, 160, 155)
#' )
#'
#' # -- 2. Describe the data mapping (once) ----------------------------------
#' spec <- hd_spec(df, x = "age", y = "pct", group = "sex", n = "n",
#'                  ylab = "Percentage (%)")
#'
#' # -- 3. Describe the presentation (reusable across specs) -----------------
#' opts <- hd_opts(title    = "Health survey results",
#'                  subtitle = "Source: FHI 2024",
#'                  caption  = "Tall om helse",
#'                  ylim     = c(0, 80))
#'
#' # -- 4. Render — swap backend without touching spec or opts ----------------
#' hd_make(spec, "column", opts)                        # interactive HC
#' hd_make(spec, "column", opts, backend = "ggplot2")   # static ggplot2
#' hd_make(spec, "line",   opts, smooth = TRUE)         # HC spline
#' hd_make(spec, "line",   opts, backend = "ggplot2")   # gg line
#' hd_make(spec, "scatter")                             # HC scatter
#'
#' # Donut chart
#' hd_make(pie_spec, "pie", pie_opts, inner_size = "50%")
#'
#' # -- 5. Arearange (confidence intervals) ----------------------------------
#' df2   <- cbind(df, lo = df$pct - 5, hi = df$pct + 5)
#' spec2 <- hd_spec(df2, "age", "pct", group = "sex")
#' hd_make(spec2, "arearange", opts, ymin = "lo", ymax = "hi")
#'
#' # -- 6. Save to disk -------------------------------------------------------
#' hd_save(hd_make(spec, "column", opts),               "chart.html")
#' hd_save(hd_make(spec, "column", opts, backend="ggplot2"), "chart.png")
#'
#' # -- 7. Session-wide styling -----------------------------------------------
#' hd_set_theme(hc_theme = "economist",
#'              colors   = c("#025169","#7C145C","#C68803"))
#'
#' # -- 8. Launch the Shiny GUI -----------------------------------------------
#' hd_app()
#' ```
#'
#' ## Key functions
#'
#' | Function | Purpose |
#' |:--|:--|
#' | [hd_spec()] | Describe data mapping (x, y, group, n, labels) |
#' | [hd_opts()] | Describe presentation (title, ylim, colours, theme) |
#' | [hd_make()] | Render spec + opts → highchart or ggplot |
#' | [hd_save()] | Export to HTML / JSON / PNG / SVG / PDF |
#' | [hd_set_theme()] | Session-wide colour / font / theme defaults |
#' | [hd_theme()] | Build a highcharter theme object directly |
#' | [hd_add_js()] | Inject custom JS into a highchart |
#' | [register_geom()] | Add a custom geometry |
#' | [register_backend()] | Add a custom rendering backend |
#' | [register_palette()] | Add a named colour palette |
#' | [list_geoms()] | Show available geometries |
#' | [list_backends()] | Show available backends |
#' | [list_palettes()] | Show available palettes |
#' | [hd_app()] | Launch the Shiny GUI |
#'
#' @keywords internal
"_PACKAGE"

## usethis namespace: start
#' @importFrom rlang .data
#' @importFrom utils modifyList head
#' @importFrom stats setNames
#' @importFrom tools file_ext file_path_sans_ext
## usethis namespace: end
NULL

# ggplot2 engine ----------------------------------------------------
#
# Engine contract:
#   function(spec, geom, opts, geom_params, use_js, ...)
#
# Theme, colours, and font are resolved in one call to gg_theme() at the
# top of the engine -- mirroring how highcharter_engine() calls hd_theme()
# at the end.  Both follow the same priority chain:
#   explicit opts > session option > package default
#
# gg_theme() returns an hd_gg_theme object with:
#   $theme  -- ggplot2 theme object with font already merged in
#   $colors -- resolved colour vector (or NULL)
# gt$colors is then passed to apply_gg_colors() so colours flow through
# a single resolved value rather than being read from opts twice.

#' @keywords internal
ggplot_engine <- function(spec, geom, opts, geom_params,
                          use_js = TRUE, ...) {
  
  # -- Resolve theme + colors + font in one step (mirrors hd_theme() call) ---
  # Priority for each: explicit opts > getOption("highdir.*") > default
  gt <- gg_theme(opts$gg_theme, colors = opts$colors)
  # gt$theme  = ggplot2 theme object with font baked in
  # gt$colors = resolved colour vector used by apply_gg_colors() below

  # -- Self-contained geoms: bypass base_fig() entirely -----------------------
  #
  # Some geoms own their complete ggplot - they supply their own aes(),
  # scales, coord, and labs from scratch.  Passing them through base_fig()
  # would produce a phantom axis mapping (spec$x / spec$y) underneath the
  # geom's own mapping, leaving an artefact axis line visible after
  # coord_flip().
  #
  # Geoms in this category:
  #   is_map_geom  - registered flag in the registry (e.g. choropleth maps)
  #   ranked_bar   - uses .xname (sorted factor) not spec$x; manages its own
  #                  coord_flip(), scale_x_discrete(), scale_y_continuous()
  #                  and labs() internally via the layer list it returns.
  #
  # These geoms receive single_colour via geom_params (same as standard path)
  # so colour resolution is consistent.  Theme, axis label hiding, and
  # accessibility alt text are still applied below after the early return.
  .self_contained <- isTRUE(geom$is_map_geom) || geom$name == "ranked_bar"

  if (.self_contained) {
    # Inject single_colour so the geom can use the resolved brand colour
    single_colour              <- resolve_colors(1L, gt$colors)[1]
    geom_params$single_colour  <- single_colour
    
    layers <- geom$ggplot_fun(spec, opts, geom_params)

    p <- ggplot2::ggplot() +
      ggplot2::labs(
        title    = opts$title,
        subtitle = opts$subtitle,
        caption  = opts$caption
      )
    for (layer in layers) p <- p + layer

    # Apply theme + axis hiding + accessibility then return - no scale block
    p <- p + gt$theme
    if (is.null(opts$ylab))
      p <- p + ggplot2::theme(axis.title.y = ggplot2::element_blank())
    if (is.null(opts$xlab))
      p <- p + ggplot2::theme(axis.title.x = ggplot2::element_blank())
    if (!is.null(opts$description))
      p <- p + ggplot2::labs(alt = opts$description)

    return(p)
  }

  # -- Standard path: base_fig() builds canvas, engine adds layers ------------
  p <- base_fig(spec, opts, "ggplot2")

  # -- Colour + layers ---------------------------------------------------------
  grp_col <- spec$colour %||% spec$group

  if (!is.null(grp_col)) {
    # Multi-series: resolve palette from gt$colors (already priority-resolved)
    n_groups     <- length(unique(spec$data[[grp_col]]))
    group_levels <- as.character(unique(spec$data[[grp_col]]))

    layers <- geom$ggplot_fun(spec, opts, geom_params)
    for (layer in layers) p <- p + layer
    p <- apply_gg_colors(p,
                         colors       = gt$colors,
                         n_groups     = n_groups,
                         group_levels = group_levels)

  } else {
    # Single series: one brand colour injected as a fixed aesthetic
    single_colour              <- resolve_colors(1L, gt$colors)[1]
    geom_params$single_colour  <- single_colour

    layers <- geom$ggplot_fun(spec, opts, geom_params)
    for (layer in layers) p <- p + layer
  }


  # scale_y_continuous + coord_cartesian - two separate jobs:
  #
  #   scale_y_continuous(expand = ...) controls axis padding only.
  #   limits = ... is intentionally NOT set here.
  #
  #   Why: scale limits REMOVE data outside the range before geoms draw.
  #   A bar chart draws from y=0 (the baseline) up to the value.  If
  #   limits = c(25, 100) is set, the baseline y=0 is outside the range
  #   and ggplot2 drops the entire bar row silently, making columns
  #   disappear.  This is ggplot2's oob_censor default behaviour.
  #
  #   coord_cartesian(ylim = ...) ZOOMS the viewport AFTER the geom has
  #   drawn.  Bars are drawn from their true origin and then clipped at
  #   the viewport edge - this is always what users expect from ylim.
  if (is.numeric(spec$data[[spec$y]]) || is.integer(spec$data[[spec$y]])) {
    p <- p + ggplot2::scale_y_continuous(
      expand = ggplot2::expansion(mult = c(0, .1))
    )
    if (!is.null(opts$ylim))
      p <- p + ggplot2::coord_cartesian(ylim = opts$ylim)
  }
  

  # -- Theme (per-figure opts$gg_theme > session default) --------------------
  # Applied last so it sits on top of every layer -- same position as
  # hd_theme() in highcharter_engine().  Font is already merged into gt$theme
  # by gg_theme(), so no separate font block is needed here.
  p <- p + gt$theme

  # -- Axis label hiding (applied AFTER theme) --------------------------------
  # .resolve_axis_label() in base_fig() already set opts$xlab / opts$ylab to
  # NULL when the user passed NULL.  We re-apply element_blank() here, AFTER
  # gt$theme, because many ggplot2 themes reset axis.title to their own
  # default, silently undoing the element_blank() that base_fig() set earlier.
  # Applying it last guarantees NULL -> hide regardless of theme choice.
  # This also covers geoms like ranked_bar that bypass base_fig() entirely.
  if (is.null(opts$ylab))
    p <- p + ggplot2::theme(axis.title.y = ggplot2::element_blank())
  if (is.null(opts$xlab))
    p <- p + ggplot2::theme(axis.title.x = ggplot2::element_blank())

  # Accessibility alt text - set via labs(alt = ...) (ggplot2 >= 3.3.0).
  # Rendered as an HTML alt attribute when the plot is included in
  # R Markdown / Quarto documents.  NULL leaves alt text unset.
  # But this is only when saved as SVG
  if (!is.null(opts$description))
    p <- p + ggplot2::labs(alt = opts$description)

  p
}

# ------------------------------------------------------------------------------
# AREARANGE
# ------------------------------------------------------------------------------
# 
#' ggplot2 Arearange Geom Function
#'
#' Returns a list of ggplot2 layers (ribbon + optional centre line + points)
#' that the ggplot2 engine adds to the base figure.  Not intended to be called
#' directly; use [hd_geom_arearange()] or `hd_make(..., type = "arearange")`.
#'
#' @section geom_params recognised:
#' \describe{
#'   \item{`ymin`}{Character. **Required.** Column name for the lower bound.}
#'   \item{`ymax`}{Character. **Required.** Column name for the upper bound.}
#'   \item{`show_line`}{Logical. Draw a centre line + open-circle points on top
#'     of the ribbon.  Default `TRUE`.}
#'   \item{`single_colour`}{Character or `NULL`.  Fixed hex colour injected by
#'     the ggplot2 engine for single-series figures.  `NULL` for multi-series
#'     figures (colour is handled by [apply_gg_colors()]).}
#' }
#'
#' @param spec       An [hd_spec()] object.
#' @param opts       An [hd_opts()] object.
#' @param geom_params Named list of geom-specific arguments (see above).
#'
#' @return A list of `ggplot2` layer objects.
#'
#' @keywords internal

gg_arearange <- function(spec, opts, geom_params) {
 
  # -- Required args -----------------------------------------------------------
  ymin <- geom_params$ymin
  ymax <- geom_params$ymax
 
  if (is.null(ymin) || is.null(ymax))
    stop(
      "hd_geom_arearange() requires both `ymin` and `ymax`.\n",
      "Supply them in hd_geom_arearange(ymin = \"col_lo\", ymax = \"col_hi\").",
      call. = FALSE
    )
 
  # -- Optional args -----------------------------------------------------------
  # show_line: draw centre line + open-circle points on top of the ribbon.
  # Default TRUE to match the highcharter version.
  show_line     <- isTRUE(geom_params$show_line %||% TRUE)
 
  # single_colour: injected by the ggplot2 engine for single-series figures.
  # NULL when a group column is present (colour handled by apply_gg_colors()).
  single_colour <- geom_params$single_colour
 
  # -- Build layers ------------------------------------------------------------
  # Layer order matters: ribbon below, line + points on top (higher z-order).
 
  if (!is.null(single_colour)) {
    # Single-series: fixed colour for every layer
    layers <- list(
      ggplot2::geom_ribbon(
        ggplot2::aes(ymin = .data[[ymin]], ymax = .data[[ymax]]),
        fill  = single_colour,
        alpha = 0.25
      )
    )
    if (show_line) {
      layers <- c(layers, list(
        ggplot2::geom_line(colour = single_colour, linewidth = 0.8),
        ggplot2::geom_point(
          colour = single_colour,
          size   = 3,
          shape  = 21,
          fill   = "white",
          stroke = 1.2
        )
      ))
    }
  } else {
    # Multi-series: colour mapped via aes (resolved later by apply_gg_colors())
    layers <- list(
      ggplot2::geom_ribbon(
        ggplot2::aes(ymin = .data[[ymin]], ymax = .data[[ymax]]),
        alpha = 0.25
      )
    )
    if (show_line) {
      layers <- c(layers, list(
        ggplot2::geom_line(linewidth = 0.8),
        ggplot2::geom_point(
          size   = 3,
          shape  = 21,
          fill   = "white",
          stroke = 1.2
        )
      ))
    }
  }
 
  layers
}


# ------------------------------------------------------------------------------
# hc_arearange -- highcharter geom function
# ------------------------------------------------------------------------------
# 
#' Highcharter Arearange Geom Function
#'
#' Adds an arearange series (and optionally a linked centre-line series) to a
#' partially-built `highchart` object.  Not intended to be called directly; use
#' [hd_geom_arearange()] or `hd_make(..., type = "arearange")`.
#'
#' @section Series strategy:
#' For each group in `spec$data` two Highcharts series are added:
#' \enumerate{
#'   \item A `"line"` series for the centre values (`spec$y`), rendered first
#'     so it appears on top (higher z-index).  This series owns the legend
#'     entry (`showInLegend = TRUE`).
#'   \item An `"arearange"` series for the confidence band, linked back to the
#'     line series via `linkedTo = line_id`.  This means clicking the legend
#'     hides *both* line and band together.  `showInLegend = FALSE` avoids a
#'     duplicate legend entry.
#' }
#' When `show_line = FALSE` only the arearange series is added, and it owns
#' the legend entry itself.
#'
#' @section geom_params recognised:
#' \describe{
#'   \item{`ymin`}{Character. **Required.** Column name for the lower bound.}
#'   \item{`ymax`}{Character. **Required.** Column name for the upper bound.}
#'   \item{`show_line`}{Logical. Overlay the centre line.  Default `TRUE`.}
#' }
#'
#' @param chart      A `highchart` object (from [base_fig()]).
#' @param spec       An [hd_spec()] object.
#' @param opts       An [hd_opts()] object.
#' @param geom_params Named list of geom-specific arguments (see above).
#' @param use_js     Logical. Inject the hover-band JS callback.  Default `TRUE`.
#' @param ...        Unused; present for engine-contract consistency.
#'
#' @return The updated `highchart` object.
#'
#' @keywords internal
hc_arearange <- function(chart, spec, opts, geom_params,
                         use_js = TRUE, ...) {
 
  # -- Required args -----------------------------------------------------------
  ymin <- geom_params$ymin
  ymax <- geom_params$ymax
 
  if (is.null(ymin) || is.null(ymax))
    stop(
      "hd_geom_arearange() requires both `ymin` and `ymax`.\n",
      "Supply them in hd_geom_arearange(ymin = \"col_lo\", ymax = \"col_hi\").",
      call. = FALSE
    )
 
  # -- Optional args -----------------------------------------------------------
  show_line <- isTRUE(geom_params$show_line %||% TRUE)
 
  # -- Series ------------------------------------------------------------------
  groups    <- .group_split(spec)
  palette   <- resolve_colors(length(groups), opts$colors)
  point_ev  <- point_events_or_null(use_js)
 
  for (i in seq_along(groups)) {
    grp     <- groups[[i]]
    color_i <- palette[i]
 
    # Unique id per group so the arearange series can link back to its line.
    # Without a per-group id all arearanges would link to the same line.
    line_id <- paste0("line_series_", i)
 
    # -- 1. Centre-line series ------------------------------------------------- 
    # Added BEFORE the arearange so Highcharts renders it on top (z-order).
    # showInLegend = TRUE: this series owns the legend entry for the group.
    if (show_line) {
      line_args <- list(
        chart,
        data         = spec$data[grp$rows, ],
        type         = "line",
        id           = line_id,         # anchor for linkedTo below
        name         = grp$name,
        highcharter::hcaes(
          x = !!rlang::sym(spec$x),
          y = !!rlang::sym(spec$y)      # spec$y is the centre/mean column
        ),
        color        = color_i,
        lineWidth    = 2,
        showInLegend = TRUE,            # this series owns the legend entry
        marker       = list(
          symbol  = "circle",
          enabled = TRUE,
          radius  = 4
        ),
        zIndex       = 2               # draw on top of the ribbon
      )
      if (!is.null(point_ev)) line_args$point <- point_ev
      chart <- do.call(highcharter::hc_add_series, line_args)
    }
 
    # -- 2. Confidence-band (arearange) series --------------------------------- 
    # linkedTo ties the ribbon to the line series above so both hide/show
    # together when the user clicks the legend entry.
    # showInLegend = FALSE when show_line is TRUE (line owns the legend entry).
    range_args <- list(
      chart,
      data         = spec$data[grp$rows, ],
      type         = "arearange",
      name         = grp$name,
      linkedTo     = if (show_line) line_id else ":previous",
      highcharter::hcaes(
        x    = !!rlang::sym(spec$x),
        low  = !!rlang::sym(ymin),
        high = !!rlang::sym(ymax)
      ),
      color        = color_i,
      fillOpacity  = 0.15,            # subtle band, not a heavy block
      lineWidth    = 0,               # no border line on the ribbon edges
      showInLegend = !show_line       # only show in legend if no line
    )
    if (!is.null(point_ev)) range_args$point <- point_ev
    chart <- do.call(highcharter::hc_add_series, range_args)
  }
 
  chart
}

# ------------------------------------------------------------------------------
# Public geom arearange constructor
# ------------------------------------------------------------------------------

#' Add an Arearange (Confidence Band) Layer
#'
#' Geometry layer for ribbon / confidence-interval charts.  Unlike the other
#' `hd_geom_*()` functions, `ymin` and `ymax` are **required** named arguments
#' (they map to column names in `spec$data`) rather than optional `...` extras.
#' This makes the contract explicit at the call site instead of burying
#' required information inside `...`.
#'
#' @param ymin Character. Column name for the lower bound of the range.
#' @param ymax Character. Column name for the upper bound of the range.
#' @param ...  Additional optional arguments forwarded to the geom function
#'   (e.g. `show_line = FALSE`, `single_colour = "#025169"`).
#'   Run `geom_args("arearange")` for the full list.
#'
#' @return An S3 object of class `"hd_geom"` for use with `+.hd`.
#'
#' @examples
#'
#' # Single series
#' spec_ar1 <- hd_spec(alco1,
#'                     x    = "year",
#'                     y    = "adj_enhet")
#' 
#' opts_ar <- hd_opts(
#'   title    = "Alcohol consumption with 95% CI",
#'   subtitle = "Source: Norwegian Directorate of Health",
#'   ylim     = c(0, 30),
#'   ylab = "Number of units alcohol"
#' )
#'
#' hd_make(spec_ar1, "arearange", opts_ar,
#'        ymin = "lower_enhet", ymax = "upper_enhet")
#'
#' # Static ggplot2 version
#' hd_make(spec_ar1, "arearange", opts_ar,
#'        ymin = "lower_enhet", ymax = "upper_enhet", backend = "ggplot2")
#'
#' #' # Multi-series with group column
#' # Grouped by kjonn
#' spec_ar2 <- hd_spec(alco2,
#'                     x     = "year",
#'                     y     = "adj_enhet",
#'                     group = "kjonn")
#' 
#' hd_make(spec_ar2, "arearange", opts_ar,
#'         ymin = "lower_enhet", ymax = "upper_enhet")
#' 
#' @export
hd_geom_arearange <- function(ymin = NULL, ymax = NULL, ...) {
  hd_geom("arearange", ymin = ymin, ymax = ymax, ...)
}

---
title: "Geom examples"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Geom examples}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---


```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse  = TRUE,
  comment   = "#>",
  eval      = FALSE,  # figures require a browser; set TRUE for local preview
  out.width = "60%",
  fig.align = "center",
  fig.width = 7,
  fig.height = 4,
  message = FALSE,
  warning = FALSE   # optional
)
```

This vignette shows how to use each geometry in highdir.
Every example follows the same two-step workflow:

1. **`hd_spec()`** — describe *what* the data means (columns, labels)
2. **`hd_make()`** — decide *how* to render it (geom, backend, options)


```{r dataset}
#| echo: FALSE
#| eval: TRUE

alco1 <- 
structure(list(year = 2012:2025, adj_mean = c(29.5688209339651, 
26.3820822909097, 30.2479196947083, 24.9407936417249, 25.1552075711597, 
27.7067665662493, 26.6906409447571, 25.4251533588897, 22.5086285695819, 
25.7139807039743, 26.9308848055767, 26.2342670110855, 27.3435961548172, 
26.1531628575368), SE = c(1.31401820814255, 0.898223842915541, 
1.22017849767529, 0.957939144175014, 1.07138968311867, 0.953327936942322, 
1.04394544742553, 0.982999807380581, 0.964926259096375, 1.06360056066082, 
1.0789167751084, 1.04762055989223, 0.738525586449375, 0.807543719380914
), lower_95CI = c(26.991788087279, 24.6205796839981, 27.8550502174718, 
23.0622482786227, 23.0540848610285, 25.837246795542, 24.6434098689602, 
23.4974198595009, 20.6163601352583, 23.62820174148, 24.8149252900896, 
24.1797904994535, 25.8957573623531, 24.569989140382), upper_95CI = c(32.1458537806512, 
28.1435848978212, 32.6407891719448, 26.819339004827, 27.2563302812909, 
29.5762863369565, 28.737872020554, 27.3528868582785, 24.4008970039054, 
27.7997596664686, 29.0468443210639, 28.2887435227175, 28.7914349472813, 
27.7363365746917), adj_enhet = c(19.7, 17.6, 20.2, 16.6, 16.8, 
18.5, 17.8, 17, 15, 17.1, 18, 17.5, 18.2, 17.4), SE_enhet = c(0.9, 
0.6, 0.8, 0.6, 0.7, 0.6, 0.7, 0.7, 0.6, 0.7, 0.7, 0.7, 0.5, 0.5
), lower_enhet = c(18, 16.4, 18.6, 15.4, 15.4, 17.2, 16.4, 15.7, 
13.7, 15.8, 16.5, 16.1, 17.3, 16.4), upper_enhet = c(21.4, 18.8, 
21.8, 17.9, 18.2, 19.7, 19.2, 18.2, 16.3, 18.5, 19.4, 18.9, 19.2, 
18.5)), row.names = c(NA, 14L), class = "data.frame")

alco2 <- 
structure(list(kjonn = c(1L, 2L, 2L, 1L, 2L, 1L, 1L, 2L, 1L, 
2L, 1L, 2L, 1L, 2L, 2L, 1L, 1L, 2L, 1L, 2L, 2L, 1L, 2L, 1L, 1L, 
2L, 1L, 2L), year = c(2012L, 2012L, 2013L, 2013L, 2014L, 2014L, 
2015L, 2015L, 2016L, 2016L, 2017L, 2017L, 2018L, 2018L, 2019L, 
2019L, 2020L, 2020L, 2021L, 2021L, 2022L, 2022L, 2023L, 2023L, 
2024L, 2024L, 2025L, 2025L), adj_mean = c(39.9534494013342, 19.2158069961127, 
16.2987229537047, 36.4366085172398, 20.4071713171405, 40.1218495785968, 
33.5071360168995, 16.3684523600791, 33.7999998752251, 16.5144875220263, 
34.0407778185931, 21.4046942833816, 34.4416520612159, 18.8811860315978, 
18.8902437612744, 31.9644150234791, 28.3125963087005, 16.6742604519355, 
34.6111331093157, 16.7988662922662, 17.7598442065029, 36.0934348693515, 
18.1477985410825, 34.3912521953532, 35.4129728498304, 19.2754560917665, 
33.5140480551725, 18.7864609065691), SE = c(2.37961465920224, 
1.05842075962891, 0.805528008154603, 1.58875647328087, 1.31567166848581, 
2.0546200666149, 1.63223247158945, 1.00102712568767, 1.93202810956404, 
0.868936995947687, 1.49238954144028, 1.18378539463991, 1.72844402301913, 
1.13242241796748, 0.926270989221471, 1.69287641926772, 1.66628519349481, 
0.933798980264774, 1.86851200933237, 0.99289119454813, 0.97504817595482, 
1.91289210595667, 1.04561834491425, 1.80516167675502, 1.1224943436911, 
0.95929056172763, 1.40593962010721, 0.812443770607349), lower_95CI = c(35.2837640271632, 
17.1387107311799, 14.7180739832502, 33.319140066092, 17.8255902148301, 
36.0902209662603, 30.3044984625189, 14.404369655018, 30.0088950080673, 
14.8093609774398, 31.1124633028755, 19.0819939701495, 31.0502826663131, 
16.6591559746196, 17.0726142656199, 28.6428742031312, 25.0431691496594, 
14.8419870446381, 30.9448270211324, 14.8506720256205, 15.8463948865644, 
32.3395415558719, 16.096031819571, 30.8492011715321, 33.2118342823919, 
17.3943660331341, 30.7569565608956, 17.1932864023274), upper_95CI = c(44.6231347755053, 
21.2929032610456, 17.8793719241591, 39.5540769683877, 22.9887524194509, 
44.1534781909332, 36.7097735712802, 18.3325350651401, 37.591104742383, 
18.2196140666128, 36.9690923343106, 23.7273945966137, 37.8330214561187, 
21.1032160885759, 20.7078732569288, 35.2859558438269, 31.5820234677416, 
18.5065338592329, 38.277439197499, 18.7470605589119, 19.6732935264414, 
39.8473281828311, 20.1995652625939, 37.9333032191744, 37.6141114172688, 
21.1565461503989, 36.2711395494494, 20.3796354108107), adj_enhet = c(26.6, 
12.8, 10.9, 24.3, 13.6, 26.7, 22.3, 10.9, 22.5, 11, 22.7, 14.3, 
23, 12.6, 12.6, 21.3, 18.9, 11.1, 23.1, 11.2, 11.8, 24.1, 12.1, 
22.9, 23.6, 12.9, 22.3, 12.5), SE_enhet = c(1.6, 0.7, 0.5, 1.1, 
0.9, 1.4, 1.1, 0.7, 1.3, 0.6, 1, 0.8, 1.2, 0.8, 0.6, 1.1, 1.1, 
0.6, 1.2, 0.7, 0.7, 1.3, 0.7, 1.2, 0.7, 0.6, 0.9, 0.5), lower_enhet = c(23.5, 
11.4, 9.8, 22.2, 11.9, 24.1, 20.2, 9.6, 20, 9.9, 20.7, 12.7, 
20.7, 11.1, 11.4, 19.1, 16.7, 9.9, 20.6, 9.9, 10.6, 21.6, 10.7, 
20.6, 22.1, 11.6, 20.5, 11.5), upper_enhet = c(29.7, 14.2, 11.9, 
26.4, 15.3, 29.4, 24.5, 12.2, 25.1, 12.1, 24.6, 15.8, 25.2, 14.1, 
13.8, 23.5, 21.1, 12.3, 25.5, 12.5, 13.1, 26.6, 13.5, 25.3, 25.1, 
14.1, 24.2, 13.6)), row.names = c(NA, 28L), class = "data.frame")

```

The same `hd_spec` object can be passed to `hd_make()` multiple times with
different geom types, backends, or `hd_opts()` objects — there is no need
to rebuild the spec.

```{r shared-data}
#| eval: TRUE

library(highdir)
# ── Age-group survey dataset (bars and pie) ────────────────────────────────────
survey <- data.frame(
  age_group = rep(c("18-24", "25-34", "35-44", "45-54", "55-64"), each = 2),
  kjonn       = rep(c("Male", "Female"), times = 5),
  pct       = c(42, 38, 55, 61, 48, 52, 60, 57, 65, 70),
  n         = c(120, 115, 200, 210, 180, 175, 160, 155, 140, 145)
)


```


## Column chart

Column charts are the default geometry and work with both grouped and
ungrouped data.
The `n` column is shown in the highcharter tooltip alongside the percentage.

```{r column-grouped}
#| eval: TRUE

spec_col <- hd_spec(survey,
                    x     = "age_group",
                    y     = "pct",
                    group = "kjonn",
                    n     = "n")

opts_col <- hd_opts(
  title    = "Alcohol use by age group and kjonn",
  subtitle = "Source: Norwegian Directorate of Health",
  ylim     = c(0, 100),
  yint     = 20,
  ylab     = "Percentage (%)"
)

## Alternatively using ggplot2 syntax
# hd(survey, x = "age_group", y = "pct", group = "kjonn", n = "n") +
#  hd_opts(title = "Alcohol use by age group and kjonn",
#       subtitle = "Source: Norwegian Directorate of Health",
#       ylim = c(0, 100),
#       yint = 20,
#       ylab = "Percentage (%)") +
#       hd_geom_column()

```

### Interactive figure 

```{r hc-column1}
#| eval: TRUE

# Interactive (default)
hd_make(spec_col, "column", opts_col)

# alternatively
# hd(spec_col) + hd_geom_column() + hd_opts(opts_col)

```

### Static figure

```{r gg-column1}
#| eval: TRUE

# Static ggplot2
hd_make(spec_col, "column", opts_col, backend = "ggplot2")
```

### Column with groups

```{r hd-column2}
#| eval: TRUE

# Horizontal bars — flip applies to both backends
opts_flip <- hd_opts(
  title    = "Alcohol use by age group and kjonn",
  subtitle = "Source: Norwegian Directorate of Health",
  ylim     = c(0, 100),
  flip     = TRUE
)

hd_make(spec_col, "column", opts_flip)
```

```{r gg-column}
#| eval: TRUE

hd_make(spec_col, "column", opts_flip, backend = "ggplot2")
```

## Line chart

Line charts suit time-series data.
Use `smooth = TRUE` (default) for spline curves, or `FALSE` for straight
segments between points.
`dot_size` controls the marker radius.

```{r line-single}
#| eval: TRUE

# ── Time-series dataset (single group) ────────────────────────────────────────
# Annual alcohol consumption estimate with 95 % confidence interval.
# data("alco1")

# ── Time-series dataset (kjonn groups) ──────────────────────────────────────────
# data("alco2")

# Single series — no group column
spec_line1 <- hd_spec(alco1,
                      x    = "year",
                      y    = "adj_mean"
                      )

opts_line <- hd_opts(
  title    = "Alcohol consumption over time",
  subtitle = "Source: Norwegian Directorate of Health",
  ylim     = c(0, 50),
  ylab = "Litres per capita"
)


# Straight segments
hd_make(spec_line1, "line", opts_line, smooth = FALSE)
hd_make(spec_line1, "line", opts_line, smooth = FALSE, backend = "ggplot2")
```

### Interactive figure

```{r hc-line1}
#| eval: TRUE

# Spline (smooth) — default
hd_make(spec_line1, "line", opts_line)
```

### Static

```{r gg-line1}
#| eval: TRUE

# Spline (smooth) — default
hd_make(spec_line1, "line", opts_line, backend = "ggplot2")
```

```{r line-grouped}
# Grouped by kjonn
spec_line2 <- hd_spec(alco2,
                      x     = "year",
                      y     = "adj_mean",
                      group = "kjonn")

# Larger markers, straight lines
hd_make(spec_line2, "line", opts_line, smooth = FALSE, dot_size = 6)
hd_make(spec_line2, "line", opts_line, smooth = FALSE, dot_size = 6,
        backend = "ggplot2")

# Highcharter only: custom per-group marker shapes
hd_make(spec_line2, "line", opts_line,
        line_symbols = c("circle", "square"))
```

## Ranked bar chart

A ranked bar chart sorts bars by value so comparisons across categories are
immediate.
Use `ranked_bar` when the order of categories matters more than their nominal
labels - for example, ranking regions by a health indicator.

**Automatic label placement** is one of the key features of this geom.
In the ggplot2 backend, value labels are placed _inside_ a bar when the bar
is long enough to fit the text, and _outside_ when the bar is too short.
The cut-off is calculated per bar from the label character width relative to
the axis range, so the placement is fully automatic — no manual threshold
is needed.

```{r ranked-data}
#| eval: TRUE

# Regional health indicator dataset
regions <- data.frame(
  region = c("Oslo", "Viken", "Vestland", "Rogaland",
             "Trondelag", "Innlandet", "Agder",
             "Nordland", "Troms og Finnmark"),
  rate   = c(68.4, 71.2, 87.8, 64.5, 61.3, 6.1, 54.2, 49.8, 42.1),
  n      = c(402, 448, 681, 318, 297, 251, 198, 177, 148)
)
```

```{r ranked-basic}
#| eval: TRUE

spec_rb <- hd_spec(regions,
                   x    = "region",
                   y    = "rate",
                   n    = "n")

opts_rb <- hd_opts(
  title    = "Health indicator by region",
  subtitle = "Source: Norwegian Directorate of Health",
  ylab     = "Rate per 100 000",
  flip     = TRUE
)

# Static ggplot2 — value labels placed inside or outside bars automatically
hd_make(spec_rb, "ranked_bar", opts_rb, backend = "ggplot2")
```

```{r ranked-basic-hc}
# Interactive — default ascending order (lowest bar at bottom)
hd_make(spec_rb, "ranked_bar", opts_rb)
```

Pass `ascending = FALSE` to flip the sort order so the highest-ranked
category appears at the top:

```{r ranked-descending}
hd_make(spec_rb, "ranked_bar", opts_rb, ascending = FALSE)
hd_make(spec_rb, "ranked_bar", opts_rb, ascending = FALSE, backend = "ggplot2")
```

Use `aim` to draw a dashed reference line at a target value.
The line is drawn in a contrasting brand colour so it reads clearly
against the bars:

```{r gg-ranked-aim}
# Draw a dashed target line at rate = 60
hd_make(spec_rb, "ranked_bar", opts_rb, aim = 60, backend = "ggplot2")
```

```{r hc-ranked-aim}
#| eval: TRUE

# Draw a dashed target line at rate = 60
hd_make(spec_rb, "ranked_bar", opts_rb, aim = 60)
```

Use `vs` to highlight one specific category with a contrasting fill colour.
The argument accepts a partial string match against the `x` column, so
`vs = "Oslo"` will match `"Oslo"` exactly, and `vs = "Troms"` would match
`"Troms og Finnmark"`:

```{r ranked-vs}
# Highlight Oslo as the comparison reference
hd_make(spec_rb, "ranked_bar", opts_rb, vs = "Oslo")
hd_make(spec_rb, "ranked_bar", opts_rb, vs = "Oslo", backend = "ggplot2")

# Combine: descending order + target line + comparison highlight
hd_make(spec_rb, "ranked_bar", opts_rb,
        ascending = FALSE,
        aim       = 60,
        vs        = "Oslo")
```

```{r ranked-vs-gg}
#| eval: TRUE

hd_make(spec_rb, "ranked_bar", opts_rb,
        ascending = TRUE,
        aim       = 60,
        vs        = "Oslo",
        backend   = "ggplot2")
```

The `n` column in `hd_spec()` appends sample sizes to category labels in
the ggplot2 backend (`"Oslo (N=502)"`) and shows them in the highcharter
tooltip.
Omit `n` from `hd_spec()` if you do not want sample sizes displayed:

```{r ranked-no-n}
spec_rb_no_n <- hd_spec(regions,
                        x    = "region",
                        y    = "rate")

# No N= labels in ggplot2; no N line in HC tooltip
hd_make(spec_rb_no_n, "ranked_bar", opts_rb, backend = "ggplot2")
hd_make(spec_rb_no_n, "ranked_bar", opts_rb)
```


## Arearange chart

Arearange is designed for displaying a central estimate alongside a
confidence interval or min/max band.
`ymin` and `ymax` are **required** — they name the columns that define the
lower and upper bounds of the shaded band.

```{r arearange-single}
#| eval: TRUE

# Single series
spec_ar1 <- hd_spec(alco1,
                    x    = "year",
                    y    = "adj_enhet")

opts_ar <- hd_opts(
  title    = "Alcohol consumption with 95% CI",
  subtitle = "Source: Norwegian Directorate of Health",
  ylim     = c(0, 30),
  ylab = "Number of units alcohol"
)

```

### Interactive 

```{r hc-area1}
#| eval: TRUE

hd_make(spec_ar1, "arearange", opts_ar,
        ymin = "lower_enhet", ymax = "upper_enhet")

```

### Static

```{r gg-area1}
#| eval: TRUE

hd_make(spec_ar1, "arearange", opts_ar,
        ymin = "lower_enhet", ymax = "upper_enhet", backend = "ggplot2")

```

```{r arearange-grouped}
#| eval: TRUE

# Grouped by kjonn
spec_ar2 <- hd_spec(alco2,
                    x     = "year",
                    y     = "adj_enhet",
                    group = "kjonn")

hd_make(spec_ar2, "arearange", opts_ar,
        ymin = "lower_enhet", ymax = "upper_enhet")

```

```{r gg-area2}

hd_make(spec_ar2, "arearange", opts_ar,
        ymin = "lower_95CI", ymax = "upper_95CI", backend = "ggplot2")
```

## Pie chart

Pie charts map one categorical column to slice labels and one numeric
column to slice values.
Pass `inner_size = "50%"` (or any CSS percentage) to draw a donut chart.
The `group` column is ignored for pie charts — use `x` for labels and `y`
for values.

```{r pie}
#| eval: TRUE

# Category share dataset (pie)
drinking_freq <- data.frame(
  category = c("Never", "Rarely", "Monthly", "Weekly", "Daily"),
  pct      = c(18, 25, 30, 20, 7))
  
spec_pie <- hd_spec(drinking_freq,
                    x    = "category",
                    y    = "pct")

opts_pie <- hd_opts(
  title    = "Drinking frequency",
  subtitle = "Source: Norwegian Directorate of Health",
  ylab = "Share (%)"
)

```

### Donut

```{r hc-pie}
#| eval: TRUE

# Donut interactive
hd_make(spec_pie, "pie", opts_pie, inner_size = "50%")
```

```{r gg-pie}
# Donut static
hd_make(spec_pie, "pie", opts_pie, inner_size = "50%", backend = "ggplot2")
```

### Solid pie

```{r pie-solid}
# Solid pie
hd_make(spec_pie, "pie", opts_pie)
hd_make(spec_pie, "pie", opts_pie, backend = "ggplot2")
```



## Stacked column chart

Stacked column charts display multiple series grouped into named stacks.
Each stack is a separate pile of bars; series that share the same stack
accumulate on top of each other.

The data must be in **long format** with one row per `(x, group, stack)`
combination:

- `x` — the category axis (e.g. medal type)
- `y` — the numeric value
- `group` — the series label shown in the legend (e.g. country)
- `stack` — the column that assigns rows to a stack group (e.g. continent)

`stack` is a **required argument** passed via `...` in `hd_make()`.

```{r stacked-data}
#| eval: TRUE

# Olympics all-time medal table
olympics <- data.frame(
  Country   = c("Norway", "Norway", "Norway",
                "Germany", "Germany", "Germany",
                "United States", "United States", "United States",
                "Canada", "Canada", "Canada"),
  Continent = c("Europe", "Europe", "Europe",
                "Europe", "Europe", "Europe",
                "North America", "North America", "North America",
                "North America", "North America", "North America"),
  Medal     = rep(c("Gold", "Silver", "Bronze"), times = 4),
  Count     = c(148, 133, 124,
                102,  98,  65,
                113, 122,  95,
                 77,  72,  80)
)

spec_st <- hd_spec(olympics,
                   x     = "Medal",
                   y     = "Count",
                   group = "Country")

opts_st <- hd_opts(
  title    = "Olympic Games all-time medal table, grouped by continent",
  subtitle = "Source: Olympics",
  ylab     = "Count medals"
)

# Interactive — stacks are separated by continent
hd_make(spec_st, "stacked_column", opts_st, stack = "Continent")

```

```{r stacked-basic}
# Static ggplot2 — continents become facet panels
hd_make(spec_st, "stacked_column", opts_st,
        stack   = "Continent",
        backend = "ggplot2")
```

The `stacking` argument controls how values accumulate.
`"normal"` (default) shows absolute values; `"percent"` rescales each
stack to 100 % so you can compare compositions across stacks:

```{r stacked-percent}
#| eval: TRUE

# 100% stacked — shows share within each continent, not absolute counts
hd_make(spec_st, "stacked_column", opts_st,
        stack    = "Continent",
        stacking = "percent")
```

```{r stacked-percent-gg}
## Static
hd_make(spec_st, "stacked_column", opts_st,
        stack    = "Continent",
        stacking = "percent",
        backend  = "ggplot2")
```


The same `hd_opts()` controls — title, axis labels, colours, and theme —
work identically for stacked columns as for every other geom:

```{r stacked-opts}
# Custom colour palette and Norwegian labels
opts_no <- hd_opts(
  title    = "Olympiske medaljer etter kontinent",
  subtitle = "Kilde: OL-statistikk",
  ylab     = "Antall medaljer",
  colors   = c("#025169", "#7C145C", "#C68803", "#047FA4")
)

hd_make(spec_st, "stacked_column", opts_no, stack = "Continent")
hd_make(spec_st, "stacked_column", opts_no,
        stack   = "Continent",
        backend = "ggplot2")
```

## Reusing a spec across geoms

Because `hd_spec()` and `hd_opts()` are separate from `hd_make()`, the
same data description can be rendered as any geometry — no code duplication.

```{r reuse}
# One spec, three geoms
spec_shared <- hd_spec(alco2,
                       x     = "year",
                       y     = "adj_mean",
                       group = "kjonn")

opts_shared <- hd_opts(
  title    = "Alcohol consumption by kjonn",
  subtitle = "Source: Norwegian Directorate of Health",
  ylim     = c(0, 50),
  ylab  = "Litres per capita"
)

hd_make(spec_shared, "column",    opts_shared)
hd_make(spec_shared, "line",      opts_shared)
hd_make(spec_shared, "arearange", opts_shared,
        ymin = "lower_95CI", ymax = "upper_95CI")
```


## Reusing opts across languages

An `hd_opts()` object only controls presentation.
Create one per language and pair it with the same spec.

```{r bilingual}
opts_no <- hd_opts(
  title    = "Alkoholbruk over tid",
  subtitle = "Kilde: Helsedirektoratet",
  caption  = "Tall om alkohol",
  ylim     = c(0, 50)
)

opts_en <- hd_opts(
  title    = "Alcohol use over time",
  subtitle = "Source: Norwegian Directorate of Health",
  caption  = "Annual health report",
  ylim     = c(0, 50)
)

spec_ts <- hd_spec(alco2,
                   x     = "year",
                   y     = "adj_mean",
                   group = "kjonn")

hd_make(spec_ts, "line", opts_no)
hd_make(spec_ts, "line", opts_en)
```


## Theming

Use `hd_set_theme()` once per session to apply a consistent visual style
across all figures.
Per-figure overrides are set via `hd_opts(hc_theme = ..., gg_theme = ...)`.

```{r theming}
# Session-wide defaults
hd_set_theme(
  hc_theme = "gridlight",
  gg_theme = "classic",
  colors   = c("#025169", "#7C145C", "#C68803")
)

hd_make(spec_ts, "line", opts_en)
hd_make(spec_ts, "line", opts_en, backend = "ggplot2")

# Per-figure override — does not change the session default
hd_make(spec_ts, "line",
        hd_opts(title = "Dark theme", hc_theme = "darkunica"),
        backend = "highcharter")

# Reset to package defaults
hd_set_theme(hc_theme = "default", gg_theme = "classic", colors = NULL)
```


## Saving figures

`hd_save()` exports highcharter figures to HTML or JSON and ggplot2
figures to PNG, SVG, or PDF.
The format is inferred from the file extension.

```{r save, eval = FALSE}
hc_fig <- hd_make(spec_ts, "line", opts_en)
gg_fig <- hd_make(spec_ts, "line", opts_en, backend = "ggplot2")

hd_save(hc_fig, "alcohol_line.html")
hd_save(hc_fig, "alcohol_line.json")

hd_save(gg_fig, "alcohol_line.png")
hd_save(gg_fig, "alcohol_line.svg")
```

---
title: "Specifications"
output:
  rmarkdown::html_vignette:
    number_sections: true
vignette: >
  %\VignetteIndexEntry{Specifications}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---


```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

# Types

The highdir framework defines four categories of specifications used to
construct plots. Each category serves a distinct role in data definition, visual
configuration, and geom‑level customization.

## 1. `hd_spec()` 

Defines the data specifications that apply universally across all
geom types. These specifications describe the input data structure and mappings
required by the plotting system.

## 2. `hd_opts()`

Specifies presentation options that control chart appearance. These options are
shared across multiple geoms and include settings related to layout, style, and
display behavior.

## 3. Required Specifications

Parameters that a given geom must provide for successful rendering. These vary
by geom and represent the minimum information needed for the geom to function.

## 4. Optional Specifications

Additional parameters that may be supplied depending on the selected geom. These
extend or refine the behavior of a geom but are not mandatory.

To see the required and optionals arguments:

```{r args, eval = FALSE}
geom_args("line") #args in geom line
geom_args("arearange")

# or
highdir:::.get_geom("ranked_bar")$optional_args
highdir:::.get_geom("arearange")$required_args
```

---

# Theming {.unnumbered}

```{r, eval = FALSE}
# Set package-wide defaults for the session
hd_set_theme(
  hc_theme = "helsedirektoratet",
  colors   = c("#025169", "#7C145C", "#C68803"),
  font     = "Source Sans Pro"
)

# All subsequent hd_make() calls use these settings automatically
hd_make(spec, "column")
```

---

# JavaScript injection {.unnumbered}

```{r, eval = FALSE}
fig <- hd_make(spec, "column")

# Inline JS
fig <- hd_add_js(fig, code = "console.log('chart loaded');")

# From a .js file
fig <- hd_add_js(fig, file = "path/to/my-plugin.js")

# From a bundled plugin (inst/js/<name>.js)
fig <- hd_add_js(fig, plugin = "my-plugin")
```

---
title: "Quick start"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Quick start}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

library(highdir)
```

There are two ways to use `highdir`:

1. Declarative API approach 
2. Layered API approach

## Declarative API 

```{r setup, eval = FALSE}

library(highdir)

# Create dataset
df <- data.frame(
  age  = rep(c("18-24", "25-34", "35-44", "45-54"), each = 2),
  sex  = rep(c("Male", "Female"), 4),
  pct  = c(42, 38, 55, 61, 48, 52, 60, 57),
  n    = c(120, 115, 200, 210, 180, 175, 160, 155)
)

# Figure data specification 
spec <- hd_spec(df, x = "age", y = "pct", group = "sex", n = "n")

# Specify figure presentation
opts <- hd_opts(title = "Tall om Report",
                subtitle = "Source: Example data",
                caption  = "Tall om helse",
                ylim = c(0, 80))

hd_make(spec, "column", opts)                       # highcharter (default)
hd_make(spec, "column", opts, backend = "ggplot2")  # static ggplot2
hd_make(spec, "line",   opts, smooth = TRUE)        # smooth spline
hd_make(spec, "pie",    opts)                       # pie / donut

# Disable JavaScript (for static HTML export)
hd_make(spec, "column", opts, use_js = FALSE)

# Save
hd01 <- hd_make(spec, "column")
hd_save(fig = hd01, file = "chart.html")

gg01 <- hd_make(spec, "column", backend = "ggplot2")
hd_save(fig = gg01, file = "chart.png")
```

## Layered API 

```{r gg, eval = FALSE}

# Interactive
hd(df, x = "age", y = "pct", group = "sex", n = "n") +
  hd_geom_column() +
  hd_opts(title = "Tall om Report",
          subtitle = "Source: Example data",
          caption  = "Tall om helse",
          ylim = c(0, 80))

# Static ggplot2
hd(df, x = "age", y = "pct", group = "sex", n = "n", backend = "ggplot2") +
  hd_geom_column() +
  hd_opts(title = "Tall om Report",
          subtitle = "Source: Example data",
          caption  = "Tall om helse",
          ylim = c(0, 80))

```

For more example on how to use other geoms, please read in
[other geom examples](https://folkehelsestats.github.io/highdir/articles/examples.html)

## Geom 

To list all geom features.

```{r geom}
list_geoms()
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/geom-arearange.R
\name{hd_geom_arearange}
\alias{hd_geom_arearange}
\title{Add an Arearange (Confidence Band) Layer}
\usage{
hd_geom_arearange(ymin = NULL, ymax = NULL, ...)
}
\arguments{
\item{ymin}{Character. Column name for the lower bound of the range.}

\item{ymax}{Character. Column name for the upper bound of the range.}

\item{...}{Additional optional arguments forwarded to the geom function
(e.g. \code{show_line = FALSE}, \code{single_colour = "#025169"}).
Run \code{geom_args("arearange")} for the full list.}
}
\value{
An S3 object of class \code{"hd_geom"} for use with \code{+.hd}.
}
\description{
Geometry layer for ribbon / confidence-interval charts.  Unlike the other
\verb{hd_geom_*()} functions, \code{ymin} and \code{ymax} are \strong{required} named arguments
(they map to column names in \code{spec$data}) rather than optional \code{...} extras.
This makes the contract explicit at the call site instead of burying
required information inside \code{...}.
}
\examples{

# Single series
spec_ar1 <- hd_spec(alco1,
                    x    = "year",
                    y    = "adj_enhet")

opts_ar <- hd_opts(
  title    = "Alcohol consumption with 95\% CI",
  subtitle = "Source: Norwegian Directorate of Health",
  ylim     = c(0, 30),
  ylab = "Number of units alcohol"
)

hd_make(spec_ar1, "arearange", opts_ar,
       ymin = "lower_enhet", ymax = "upper_enhet")

# Static ggplot2 version
hd_make(spec_ar1, "arearange", opts_ar,
       ymin = "lower_enhet", ymax = "upper_enhet", backend = "ggplot2")

#' # Multi-series with group column
# Grouped by kjonn
spec_ar2 <- hd_spec(alco2,
                    x     = "year",
                    y     = "adj_enhet",
                    group = "kjonn")

hd_make(spec_ar2, "arearange", opts_ar,
        ymin = "lower_enhet", ymax = "upper_enhet")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/geom-line.R
\name{hd_geom_line}
\alias{hd_geom_line}
\title{Line Geometry Layer for hd Objects}
\usage{
hd_geom_line(smooth = TRUE, dot_size = 4, line_symbols = NULL, ...)
}
\arguments{
\item{smooth}{Logical. TRUE = spline curves, FALSE = straight segments. Both backends.}

\item{dot_size}{Numeric. Marker radius in pixels. Both backends.}

\item{line_symbols}{Character vector. Highcharter only. Per-group marker shapes:
"circle", "square", "diamond", "triangle", "triangle-down".}

\item{...}{Geometry-specific arguments forwarded to \code{\link[=hd_make]{hd_make()}}.}
}
\value{
An S3 object of class \code{"hd_geom"} for use with \code{+.hd}.
}
\description{
\code{hd_geom_line()} creates a line geometry layer that is added to an \code{\link[=hd]{hd()}}
object via \code{+}.  The layer records the geometry type and any geometry-specific arguments;
rendering only happens when the \code{hd} object is printed.
}
\examples{
# Single series - no group column
spec_line1 <- hd_spec(alco1,
    x    = "year",
    y    = "adj_mean"
)

opts_line <- hd_opts(
    title = "Alcohol consumption over time",
    subtitle = "Source: Norwegian Directorate of Health",
    ylim = c(0, 50),
    ylab = "Litres per capita"
)


# Straight segments
hd_make(spec_line1, "line", opts_line, smooth = FALSE)

# Composite example with multiple geoms and custom line symbols
hd(alco2, x = "year", y = "adj_mean", group = "kjonn", backend = "ggplot2") +
  hd_geom_line(smooth = TRUE, dot_size = 3) +
  hd_opts(title = "Alcohol consumption over time by kjonn",
  subtitle = "Source: Norwegian Directorate of Health")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{.hd_add_dep}
\alias{.hd_add_dep}
\title{Version-safe hc_add_dependency wrapper}
\usage{
.hd_add_dep(chart, path)
}
\description{
highcharter 0.9.4 takes the path as a positional argument.
Older versions used name = . This wrapper handles both.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/palettes.R
\name{get_palette}
\alias{get_palette}
\title{Retrieve a Named Palette}
\usage{
get_palette(name)
}
\arguments{
\item{name}{Character. Palette name (see \code{\link[=list_palettes]{list_palettes()}}).}
}
\value{
Character vector of colours, or \code{NULL} if not found.
}
\description{
Retrieve a Named Palette
}
\examples{
get_palette("hdir")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/geom-rank.R
\name{hd_geom_ranked_bar}
\alias{hd_geom_ranked_bar}
\title{Ranked bar geometry}
\usage{
hd_geom_ranked_bar(
  ascending = TRUE,
  vs = NULL,
  aim = NULL,
  char_scale = 0.045,
  min_frac = 0.08,
  ...
)
}
\arguments{
\item{ascending}{Logical. If \code{TRUE} (default) bars are sorted in
ascending order of \code{y}.}

\item{vs}{Character string (partial match) identifying one category to
highlight with a contrasting fill colour. If omitted all bars use the same
colour.}

\item{aim}{Numeric. Optional horizontal line indicating a target or benchmark
value.  If \code{NULL} (default) no line is drawn.}

\item{char_scale}{Numeric scaling factor that converts label character-count
into axis-range units. Controls how generously space is estimated for each
character. Defaults to \code{0.045}; increase (e.g. \code{0.06}0.06)
for larger text sizes, decrease (e.g. \code{0.03}) for smaller ones.}

\item{min_frac}{Numeric. Minimum fraction of the axis range that a bar must
span before its label is considered to fit inside. Acts as a safety floor
for very short labels. Defaults to \code{0.08} (8\%).}

\item{...}{Geometry-specific arguments forwarded to \code{\link[=hd_make]{hd_make()}}.}
}
\value{
An S3 object of class \code{"hd_geom"} for use with \code{+.hd}.
}
\description{
Draws a ranked bar chart with smart label placement and optional comparison
highlighting.  Bars are sorted by value, and labels are placed inside or
outside the bar depending on available space.  A single category can be
highlighted with a contrasting fill colour, and an optional horizontal line
can be added to indicate a target or benchmark value.
}
\examples{
# Regional health indicator dataset
regions <- data.frame(
  region = c("Oslo", "Viken", "Vestland", "Rogaland",
             "Trondelag", "Innlandet", "Agder",
             "Nordland", "Troms og Finnmark"),
  rate   = c(68.4, 71.2, 87.8, 10.5, 61.3, 6.1, 54.2, 49.8, 42.1),
  n      = c(402, 448, 681, 318, 297, 251, 198, 177, 148)
)

# Declarative API ----
spec_rb <- hd_spec(regions,
                  x    = "region",
                  y    = "rate",
                  n    = "n")

opts_rb <- hd_opts(
 title    = "Health indicator by region",
 subtitle = "Source: Norwegian Directorate of Health",
 ylab     = "Rate per 100 000",
 flip     = TRUE
)

hd_make(spec_rb, "ranked_bar", opts_rb, vs = "Oslo", aim = 63)

# Layered API ----
hd(regions, x = "region", y = "rate", n = "n", backend = "ggplot2") +
 hd_geom_ranked_bar(
  ascending  = TRUE,
  vs         = "Oslo",
  aim        = 63,
  char_scale = 0.045,
  min_frac   = 0.08) +
 hd_opts(
 title    = "Health indicator by region",
 subtitle = "Source: Norwegian Directorate of Health",
 ylab     = "Rate per 100 000",
 flip     = TRUE
)

}
\seealso{
\code{\link[=hd_geom_column]{hd_geom_column()}}, \code{\link[=hd_geom_line]{hd_geom_line()}}, \code{\link[=hd_geom_arearange]{hd_geom_arearange()}},
\code{\link[=hd_opts]{hd_opts()}}, \code{\link[=hd_make]{hd_make()}}
}
\concept{Geoms}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/registry.R
\name{list_geoms}
\alias{list_geoms}
\title{List Registered Geometries}
\usage{
list_geoms()
}
\value{
Character vector of registered geometry names.
}
\description{
List Registered Geometries
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/registry.R
\name{register_backend}
\alias{register_backend}
\title{Register a Rendering Backend and used when loding in zzz.R file}
\usage{
register_backend(name, engine)
}
\arguments{
\item{name}{Character. Unique backend identifier (e.g. \code{"ggplot2"}).}

\item{engine}{Function.}
}
\value{
\code{name}, invisibly.
}
\description{
Register a Rendering Backend and used when loding in zzz.R file
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/palettes.R
\name{register_palette}
\alias{register_palette}
\title{Register a Named Colour Palette}
\usage{
register_palette(name, colors)
}
\arguments{
\item{name}{Character. Unique palette identifier.}

\item{colors}{Non-empty character vector of CSS/hex colour strings.}
}
\value{
\code{name}, invisibly.
}
\description{
Adds a named palette to the highdir palette registry so it can be
referenced by name wherever colours are accepted
(e.g. \code{hd_opts(colors = "my_palette")}). This function is evaluated
when loading a file in zzz.R file.
}
\details{
Built-in palettes registered at package load time:\tabular{ll}{
   Name \tab Description \cr
   \code{"hdir"} \tab Helsedirektoratet 10-colour brand palette \cr
   \code{"hdir2"} \tab 2-colour teal / purple pair \cr
}
}
\examples{
register_palette("blues", c("#084594", "#2171b5", "#4292c6", "#6baed6"))
get_palette("blues")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/theme.R
\name{gg_theme}
\alias{gg_theme}
\title{Build a ggplot2 Theme Object}
\usage{
gg_theme(theme = NULL, colors = NULL, font = NULL)
}
\arguments{
\item{theme}{Character name string, ggplot2 theme object, or \code{NULL}.
\code{NULL} reads from \code{getOption("highdir.gg_theme")}.}

\item{colors}{Character vector, palette name string, or \code{NULL}. Resolved
colours are stored on the returned object and applied by
\code{ggplot_engine()} via \code{scale_color_manual} / \code{scale_fill_manual}.
\code{NULL} reads from \code{getOption("highdir.colors")}.}

\item{font}{Character or \code{NULL}. Font family applied to all text elements
via \code{theme(text = element_text(family = font))}. \code{NULL} reads from
\code{getOption("highdir.font")}.}
}
\value{
An object of class \code{"hd_gg_theme"} - a list with two fields:
\verb{$theme} (a ggplot2 \code{theme} object with font baked in) and \verb{$colors}
(a resolved character vector or \code{NULL}).  \code{ggplot_engine()} unpacks
both.  The object can also be added directly to a ggplot with \code{+}
via the \code{+.gg} method (only the theme is applied; colors are handled
separately by \code{apply_gg_colors()}).
}
\description{
Constructs a ggplot2 theme by resolving a base theme, then merging colour
and font overrides -- exactly mirroring what \code{\link[=hd_theme]{hd_theme()}} does for the
highcharter backend.
}
\details{
Priority for each argument:
\itemize{
\item \code{theme}:  explicit argument > \code{getOption("highdir.gg_theme")} > \code{"classic"}
\item \code{colors}: explicit argument > \code{getOption("highdir.colors")} > \code{NULL}
\item \code{font}:   explicit argument > \code{getOption("highdir.font")}   > \code{NULL}
}

Called automatically inside \code{ggplot_engine()}; also useful for applying the
package theme to a ggplot built outside highdir.

Built-in name strings and their ggplot2 equivalents:\tabular{ll}{
   Name \tab ggplot2 function \cr
   \code{"classic"} (default) \tab \code{theme_classic()} \cr
   \code{"minimal"} \tab \code{theme_minimal()} \cr
   \code{"bw"} \tab \code{theme_bw()} \cr
   \code{"light"} \tab \code{theme_light()} \cr
   \code{"dark"} \tab \code{theme_dark()} \cr
   \code{"void"} \tab \code{theme_void()} \cr
   \code{"grey"} / \code{"gray"} \tab \code{theme_grey()} \cr
}
}
\examples{
gg_theme()                                    # session defaults
gg_theme("bw")                                # theme_bw(), session colors/font
gg_theme("classic", colors = c("#025169", "#7C145C"))
gg_theme("minimal", font = "Source Sans Pro")
gg_theme(ggplot2::theme_bw(base_size = 14), font = "mono")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/geom-extra.R
\name{point_events_or_null}
\alias{point_events_or_null}
\title{Hover-band point events (or NULL)}
\usage{
point_events_or_null(
  use_js,
  band_color = "rgba(204, 211, 255, 0.25)",
  half_width = 0.4
)
}
\arguments{
\item{use_js}{Logical.}

\item{band_color}{CSS colour string for the hover band.}

\item{half_width}{Half-width of the band in category units.}
}
\description{
When \code{use_js = TRUE} returns a Highcharts \code{point.events} list that draws a
translucent highlight band behind the hovered category.  When \code{FALSE}
returns \code{NULL} so the key is omitted entirely from the serialised config -
an empty \code{list()} would break shared tooltips.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/shiny-app.R
\name{hd_app}
\alias{hd_app}
\title{Launch the highdir Shiny GUI}
\usage{
hd_app(return.app = FALSE)
}
\arguments{
\item{return.app}{Logical. When deploying to server like Shiny.io to return
the app object instead of launching it.}
}
\value{
Launches a Shiny app; does not return a value.
}
\description{
Opens an interactive browser-based application for building figures with
\code{highcharter} or \code{ggplot2} without writing R code.
}
\details{
The UI (\code{inst/app/ui.R}), server (\code{inst/app/server.R}) and shared setup
(\code{inst/app/global.R}) live in \verb{inst/app/} so the folder can be deployed
independently to Shiny Server or shinyapps.io.
}
\section{Features}{
 \itemize{ \item Upload datasets in any format supported by
\strong{rio} (CSV, XLSX, SPSS, Stata, RDS, …). \item Choose geometry (\code{column},
\code{line}, \code{scatter}, \code{arearange}, \code{pie}), backend, axis variables, and group
column. \item Set title, subtitle, caption, colour palette, and HC theme.
\item Toggle JS hover band per figure. \item Render on demand with the
\strong{Draw} button. \item Download as HTML / JSON / PNG (highcharter) or PNG /
SVG (ggplot2). \item Copy the equivalent \code{hd_make()} call from the \strong{R
code} tab. }
}

\seealso{
\code{\link[=hd_make]{hd_make()}}, \code{\link[=hd_spec]{hd_spec()}}, \code{\link[=hd_opts]{hd_opts()}}, \code{\link[=hd_save]{hd_save()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/theme.R
\name{hd_theme}
\alias{hd_theme}
\title{Build a Highcharts Theme Object}
\usage{
hd_theme(name = NULL, colors = NULL, ...)
}
\arguments{
\item{name}{Character or \code{NULL}. Theme name; \code{NULL} reads from
\code{getOption("highdir.hc_theme")}.}

\item{colors}{Character vector or \code{NULL}. Colour override for this call.}

\item{...}{Named arguments forwarded to \code{\link[highcharter:hc_theme]{highcharter::hc_theme()}} as
extra overrides on top of the base theme.}
}
\value{
A highcharter theme object (\code{hc_theme}).
}
\description{
Constructs a highcharter theme by merging a named base theme with colour
and font overrides from the current \code{\link[=hd_set_theme]{hd_set_theme()}} session defaults and
any per-figure \code{opts}.
}
\details{
Called automatically inside the highcharter engine; useful when you want to
apply a theme to a highchart built outside highdir.
}
\examples{
if(interactive()) {
t <- hd_theme("darkunica")
highcharter::highchart() |> highcharter::hc_add_theme(t)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{check_columns}
\alias{check_columns}
\title{Stop with a tidy message when columns are absent from a data frame}
\usage{
check_columns(data, cols, arg_name = "data")
}
\description{
Stop with a tidy message when columns are absent from a data frame
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dataset.R
\docType{data}
\name{alco1}
\alias{alco1}
\title{Alcohol Consumption Data}
\format{
A data frame with 14 rows and 9 variables:
}
\source{
Helsedirektoratet. \emph{Tall om alkohol}.
\url{https://www.helsedirektoratet.no/rapporter/tall-om-alkohol}
}
\description{
Annual estimates of alcohol consumption, including adjusted mean values,
standard errors, and 95\% confidence intervals. Includes an alternative
unit (\code{adj_enhet}) with corresponding uncertainty estimates.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/theme.R
\name{hd_set_theme}
\alias{hd_set_theme}
\title{Set Package-Wide Style Defaults}
\usage{
hd_set_theme(
  hc_theme = NULL,
  gg_theme = NULL,
  colors = NULL,
  font = NULL,
  js_plugins = NULL
)
}
\arguments{
\item{hc_theme}{Character or \code{NULL}. Built-in highcharter theme name: one
of \code{"default"}, \code{"smpl"}, \code{"economist"}, \code{"darkunica"}, \code{"gridlight"},
\code{"bloom"}, \code{"flat"}, \code{"flatdark"}, \code{"ggplot2"}.}

\item{gg_theme}{Character, ggplot2 theme object, or \code{NULL}. Controls the
ggplot2 backend appearance. Built-in name strings: \code{"minimal"} (default),
\code{"classic"}, \code{"bw"}, \code{"light"}, \code{"dark"}, \code{"void"}, \code{"grey"}.
Alternatively pass any \verb{ggplot2::theme_*()} object directly for full
control, e.g. \code{ggplot2::theme_bw(base_size = 14)}.}

\item{colors}{Character vector, palette name, or \code{NULL}. Applied to all
figures in the session. See \code{\link[=register_palette]{register_palette()}}.}

\item{font}{Character or \code{NULL}. Font family name, e.g.
\code{"Source Sans Pro"}.}

\item{js_plugins}{Character vector or \code{NULL}. Names of bundled JS plugins
(files in \verb{inst/js/}) injected into every highcharter figure. Use
\code{character(0)} to clear all plugins.}
}
\value{
The previous option values invisibly; pass to \code{options()} to
restore.
}
\description{
Configures the default theme, colour palette, font, and optional JavaScript
plugins for all figures produced with \code{\link[=hd_make]{hd_make()}} in the current R session.
Call once at the top of a script or in \code{.Rprofile}.
}
\details{
Per-figure overrides are provided via \code{\link[=hd_opts]{hd_opts()}}, which always take
precedence over these session defaults.
}
\examples{
hd_set_theme(hc_theme = "economist", gg_theme = "classic",
             colors   = c("#025169", "#7C145C", "#C68803"))
# Reset
hd_set_theme(hc_theme = "default", gg_theme = "minimal", colors = NULL)

}
\seealso{
\code{\link[=hd_opts]{hd_opts()}} for per-figure overrides
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/figure-opts.R
\name{hd_opts}
\alias{hd_opts}
\title{Create Figure Presentation Options}
\usage{
hd_opts(
  title = NULL,
  subtitle = NULL,
  caption = NULL,
  description = NULL,
  xlab = " ",
  ylab = " ",
  ylim = NULL,
  yint = 10,
  ysuffix = NULL,
  xtick_labels = NULL,
  decimals = NULL,
  flip = FALSE,
  colors = NULL,
  hc_theme = NULL,
  gg_theme = NULL
)
}
\arguments{
\item{title}{Character or \code{NULL}. Chart title.}

\item{subtitle}{Character or \code{NULL}. Subtitle. Highcharter default:
\code{"Kilde: Navn av kilder"}.}

\item{caption}{Character or \code{NULL}. Caption text displayed below the
figure (highcharter only). This is a \strong{visible} footnote, distinct from
\code{description} which is read only by assistive technology.}

\item{description}{Character or \code{NULL}. Invisible accessibility description
of the figure intended for screen readers and other assistive technology.
\describe{
\item{highcharter}{Passed to \code{hc_accessibility(description = ...)}.
Requires the accessibility module, which highdir loads automatically.
Screen readers announce this text when the user focuses the chart.}
\item{ggplot2}{Set as the \code{alt} label via \code{labs(alt = ...)}, available
since ggplot2 3.3.0.  Rendered as an HTML \code{alt} attribute when the
plot is saved to SVG or included in an R Markdown / Quarto document
with \code{fig.alt} support.}
}
Write a concise one- or two-sentence summary of what the figure shows,
including the key trend or comparison, so the information is equally
accessible to users who cannot see the chart.  Example:
\code{"Bar chart showing alcohol use by age group. Use is highest in the 45-54 age group at 65\% and lowest in 18-24 at 42\%."}}

\item{xlab}{Character or \code{NULL}. X-axis label. \describe{
\item{\code{" "} (default)}{Use the \code{x} column name from \code{\link[=hd_spec]{hd_spec()}}.}
\item{\code{NULL}}{Hide the x-axis label completely.}
\item{any string}{Use that string as the label.} }}

\item{ylab}{Character or \code{NULL}. Y-axis label. Same rules as \code{xlab}.}

\item{ylim}{Numeric vector of length 2 or \code{NULL}. Fixed y-axis
limits, e.g. \code{c(0, 100)}.}

\item{yint}{Positive numeric. Y-axis tick interval. Default \code{10}.}

\item{ysuffix}{Character or \code{NULL}. String appended to every y-axis
tick label, e.g. \code{"\%"} or \code{" km"}. \code{NULL} shows no suffix.}

\item{xtick_labels}{Character or \code{NULL}. Column name supplying custom
x-axis tick labels when the plotted x-values are numeric but the
displayed labels should come from another column. Highcharter only.
Note: Highcharts indexes categories from 0, not 1.}

\item{decimals}{Integer or \code{NULL}. Number of decimal places to round
the y-values to before rendering. Applied to the data via
\code{check_decimals()} in \code{\link[=hd_make]{hd_make()}}. \code{NULL} leaves values unchanged.}

\item{flip}{Logical. Invert axes (horizontal bars / inverted chart).
Default \code{FALSE}.}

\item{colors}{Character vector, palette name string, or \code{NULL}.
Per-figure colour override; takes precedence over \code{\link[=hd_set_theme]{hd_set_theme()}}.}

\item{hc_theme}{Character or \code{NULL}. Per-figure highcharter theme name;
takes precedence over \code{\link[=hd_set_theme]{hd_set_theme()}}. See \code{\link[=hd_theme]{hd_theme()}} for valid names.}

\item{gg_theme}{Character name string, ggplot2 theme object, or \code{NULL}.
Per-figure ggplot2 theme; takes precedence over \code{\link[=hd_set_theme]{hd_set_theme()}}.
Name strings: \code{"classic"} (default), \code{"minimal"}, \code{"bw"}, \code{"light"},
\code{"dark"}, \code{"void"}, \code{"grey"}. Or pass a \verb{ggplot2::theme_*()} object
directly, e.g. \code{ggplot2::theme_bw(base_size = 14)}.}
}
\value{
An S3 object of class \code{"hd_opts"}.
}
\description{
Defines the \strong{visual presentation} of a figure independently from the data
mapping.  Pass the result as the \code{opts} argument of \code{\link[=hd_make]{hd_make()}}, or omit
it to accept all defaults.
}
\details{
Because opts are separate from \code{\link[=hd_spec]{hd_spec()}}, the same data mapping can
be rendered with multiple styles without repetition:

\if{html}{\out{<div class="sourceCode r">}}\preformatted{spec    <- hd_spec(df, "age", "pct", group = "sex")
opts_en <- hd_opts(title = "Health survey",    subtitle = "All ages")
opts_no <- hd_opts(title = "Helseundersøkelse", subtitle = "Alle aldre")

hd_make(spec, "column", opts_en)
hd_make(spec, "column", opts_no)
}\if{html}{\out{</div>}}
}
\examples{
opts <- hd_opts(
  title       = "Health survey results",
  subtitle    = "Source: FHI 2024",
  caption     = "Tall om helse",
  description = paste(
    "Grouped bar chart showing alcohol use by age group and sex.",
    "Use is highest in the 45-54 age group at 65\% for women."
  ),
  ylim        = c(0, 100),
  yint        = 20,
  colors      = c("#025169", "#7C145C")
)
opts

}
\seealso{
\code{\link[=hd_spec]{hd_spec()}}, \code{\link[=hd_make]{hd_make()}}, \code{\link[=hd_set_theme]{hd_set_theme()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/geom-arearange.R
\name{hc_arearange}
\alias{hc_arearange}
\title{Highcharter Arearange Geom Function}
\usage{
hc_arearange(chart, spec, opts, geom_params, use_js = TRUE, ...)
}
\arguments{
\item{chart}{A \code{highchart} object (from \code{\link[=base_fig]{base_fig()}}).}

\item{spec}{An \code{\link[=hd_spec]{hd_spec()}} object.}

\item{opts}{An \code{\link[=hd_opts]{hd_opts()}} object.}

\item{geom_params}{Named list of geom-specific arguments (see above).}

\item{use_js}{Logical. Inject the hover-band JS callback.  Default \code{TRUE}.}

\item{...}{Unused; present for engine-contract consistency.}
}
\value{
The updated \code{highchart} object.
}
\description{
Adds an arearange series (and optionally a linked centre-line series) to a
partially-built \code{highchart} object.  Not intended to be called directly; use
\code{\link[=hd_geom_arearange]{hd_geom_arearange()}} or \code{hd_make(..., type = "arearange")}.
}
\section{Series strategy}{

For each group in \code{spec$data} two Highcharts series are added:
\enumerate{
\item A \code{"line"} series for the centre values (\code{spec$y}), rendered first
so it appears on top (higher z-index).  This series owns the legend
entry (\code{showInLegend = TRUE}).
\item An \code{"arearange"} series for the confidence band, linked back to the
line series via \code{linkedTo = line_id}.  This means clicking the legend
hides \emph{both} line and band together.  \code{showInLegend = FALSE} avoids a
duplicate legend entry.
}
When \code{show_line = FALSE} only the arearange series is added, and it owns
the legend entry itself.
}

\section{geom_params recognised}{

\describe{
\item{\code{ymin}}{Character. \strong{Required.} Column name for the lower bound.}
\item{\code{ymax}}{Character. \strong{Required.} Column name for the upper bound.}
\item{\code{show_line}}{Logical. Overlay the centre line.  Default \code{TRUE}.}
}
}

\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/highdir-package.R
\docType{package}
\name{highdir-package}
\alias{highdir}
\alias{highdir-package}
\title{highdir: Backend-Agnostic Figure Builder for Helsedirektoratet}
\description{
\strong{highdir} provides a unified API for building interactive figures with
\strong{highcharter} or static figures with \strong{ggplot2}.  A figure is described
once as a data-mapping object (\link{hd_spec}) and an optional presentation
object (\link{hd_opts}), then rendered to either backend without changing the
calling code.
\subsection{Core workflow}{

\if{html}{\out{<div class="sourceCode r">}}\preformatted{library(highdir)

# -- 1. Sample data --------------------------------------------------------
df <- data.frame(
  age = rep(c("18-24", "25-34", "35-44", "45-54"), each = 2),
  sex = rep(c("Male", "Female"), 4),
  pct = c(42, 38, 55, 61, 48, 52, 60, 57),
  n   = c(120, 115, 200, 210, 180, 175, 160, 155)
)

# -- 2. Describe the data mapping (once) ----------------------------------
spec <- hd_spec(df, x = "age", y = "pct", group = "sex", n = "n",
                 ylab = "Percentage (\%)")

# -- 3. Describe the presentation (reusable across specs) -----------------
opts <- hd_opts(title    = "Health survey results",
                 subtitle = "Source: FHI 2024",
                 caption  = "Tall om helse",
                 ylim     = c(0, 80))

# -- 4. Render — swap backend without touching spec or opts ----------------
hd_make(spec, "column", opts)                        # interactive HC
hd_make(spec, "column", opts, backend = "ggplot2")   # static ggplot2
hd_make(spec, "line",   opts, smooth = TRUE)         # HC spline
hd_make(spec, "line",   opts, backend = "ggplot2")   # gg line
hd_make(spec, "scatter")                             # HC scatter

# Donut chart
hd_make(pie_spec, "pie", pie_opts, inner_size = "50\%")

# -- 5. Arearange (confidence intervals) ----------------------------------
df2   <- cbind(df, lo = df$pct - 5, hi = df$pct + 5)
spec2 <- hd_spec(df2, "age", "pct", group = "sex")
hd_make(spec2, "arearange", opts, ymin = "lo", ymax = "hi")

# -- 6. Save to disk -------------------------------------------------------
hd_save(hd_make(spec, "column", opts),               "chart.html")
hd_save(hd_make(spec, "column", opts, backend="ggplot2"), "chart.png")

# -- 7. Session-wide styling -----------------------------------------------
hd_set_theme(hc_theme = "economist",
             colors   = c("#025169","#7C145C","#C68803"))

# -- 8. Launch the Shiny GUI -----------------------------------------------
hd_app()
}\if{html}{\out{</div>}}
}

\subsection{Key functions}{\tabular{ll}{
   Function \tab Purpose \cr
   \code{\link[=hd_spec]{hd_spec()}} \tab Describe data mapping (x, y, group, n, labels) \cr
   \code{\link[=hd_opts]{hd_opts()}} \tab Describe presentation (title, ylim, colours, theme) \cr
   \code{\link[=hd_make]{hd_make()}} \tab Render spec + opts → highchart or ggplot \cr
   \code{\link[=hd_save]{hd_save()}} \tab Export to HTML / JSON / PNG / SVG / PDF \cr
   \code{\link[=hd_set_theme]{hd_set_theme()}} \tab Session-wide colour / font / theme defaults \cr
   \code{\link[=hd_theme]{hd_theme()}} \tab Build a highcharter theme object directly \cr
   \code{\link[=hd_add_js]{hd_add_js()}} \tab Inject custom JS into a highchart \cr
   \code{\link[=register_geom]{register_geom()}} \tab Add a custom geometry \cr
   \code{\link[=register_backend]{register_backend()}} \tab Add a custom rendering backend \cr
   \code{\link[=register_palette]{register_palette()}} \tab Add a named colour palette \cr
   \code{\link[=list_geoms]{list_geoms()}} \tab Show available geometries \cr
   \code{\link[=list_backends]{list_backends()}} \tab Show available backends \cr
   \code{\link[=list_palettes]{list_palettes()}} \tab Show available palettes \cr
   \code{\link[=hd_app]{hd_app()}} \tab Launch the Shiny GUI \cr
}

}
}
\seealso{
Useful links:
\itemize{
  \item \url{https://folkehelsestats.github.io/highdir/}
  \item \url{https://github.com/folkehelsestats/highdir}
  \item Report bugs at \url{https://github.com/folkehelsestats/highdir/issues}
}

}
\author{
\strong{Maintainer}: Yusman Kamaleri \email{ybkamaleri@gmail.com} (\href{https://orcid.org/0000-0001-5014-3665}{ORCID})

Authors:
\itemize{
  \item Yusman Kamaleri \email{ybkamaleri@gmail.com} (\href{https://orcid.org/0000-0001-5014-3665}{ORCID})
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/registry.R
\name{register_geom}
\alias{register_geom}
\title{Register a Geometry}
\usage{
register_geom(
  name,
  ggplot_fun = NULL,
  highcharter_fun = NULL,
  required_args = list(),
  optional_args = list(),
  is_map_geom = FALSE
)
}
\arguments{
\item{name}{Character. Unique geometry identifier.}

\item{ggplot_fun}{Function or \code{NULL}.}

\item{highcharter_fun}{Function or \code{NULL}.}

\item{required_args}{Named list of \code{list(default, desc)}. Args that MUST be supplied via
\code{...} in \code{hd_make()}. Validation fails if any are missing.}

\item{optional_args}{Named list of \code{list(default, desc)}. Args that MAY
be supplied and have a sensible default when omitted.  These are purely
informational from the registry's perspective - the geom function applies
the defaults itself via \code{geom_params$key \%||\% default}.}

\item{is_map_geom}{Logical. Bypasses \code{base_fig()} in both engines.}
}
\value{
\code{name}, invisibly.
}
\description{
Adds a geometry to the registry.  The \code{optional_args} field is the key
mechanism for user discoverability: it is what \code{geom_args()} prints, what
the Shiny app reads to build dynamic UI, and what automated documentation
can harvest without parsing source code.
}
\details{
Structure of \code{optional_args}:
A named list where each element is itself a list with two fields:
\code{default} - the value used when the arg is not supplied (may be \code{NULL})
\code{desc}    - a short human-readable description (shown by \code{geom_args()})

Example:

\if{html}{\out{<div class="sourceCode r">}}\preformatted{optional_args = list(
  smooth   = list(default = TRUE,  desc = "Spline smoothing (logical)"),
  dot_size = list(default = 4,     desc = "Marker radius in px (numeric)")
)
}\if{html}{\out{</div>}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/palettes.R
\name{list_palettes}
\alias{list_palettes}
\title{List Registered Palettes}
\usage{
list_palettes()
}
\value{
Character vector of registered palette names.
}
\description{
List Registered Palettes
}
\examples{
list_palettes()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/geom-scatter.R
\name{hd_geom_scatter}
\alias{hd_geom_scatter}
\title{Scatter Geometry Layer for hd Objects}
\usage{
hd_geom_scatter(dot_size = 4, ...)
}
\arguments{
\item{dot_size}{Numeric. Size of the points in the scatter plot. Default is 4.}

\item{...}{Geometry-specific arguments forwarded to \code{\link[=hd_make]{hd_make()}}.
' @return An S3 object of class \code{"hd_geom"} for use with \code{+.hd}.}
}
\value{
An S3 object of class \code{"hd_geom"} for use with \code{+.hd}.
}
\description{
\code{hd_geom_scatter()} creates a scatter geometry layer that is added to an
\code{\link[=hd]{hd()}} object via \code{+}. Use \code{\link[=geom_args]{geom_args()}} to discover available arguments per
geometry, e.g. \code{geom_args("scatter")} lists \code{dot_size}.
}
\examples{
# Basic scatter plot - layered API
hd(mtcars, x = "wt", y = "mpg", backend = "ggplot2") +
 hd_geom_scatter() +
 hd_opts(title = "Scatter Plot of mtcars")

# Basic scatter plot - declarative API
car <- hd_spec(mtcars, x = "wt", y = "mpg")
opt <- hd_opts(title = "Scatter Plot of mtcars")
hd_make(car, type = "scatter")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hd-object.R
\name{hd}
\alias{hd}
\title{Initialise a Composable highdir Figure}
\usage{
hd(
  data,
  x = NULL,
  y = NULL,
  group = NULL,
  n = NULL,
  colour = NULL,
  backend = getOption("highdir.backend", "highcharter")
)
}
\arguments{
\item{data}{A \code{data.frame} \strong{or} an \code{\link[=hd_spec]{hd_spec()}} object.  When an
\code{hd_spec} is supplied every other mapping argument (\code{x}, \code{y}, …) is
ignored - the spec carries them already.}

\item{x}{Character. Column name for the x-axis variable.
Ignored when \code{data} is an \code{hd_spec}.}

\item{y}{Character. Column name for the y-axis variable.
Ignored when \code{data} is an \code{hd_spec}.}

\item{group}{Character or \code{NULL}. Column used to split data into multiple
series.  Ignored when \code{data} is an \code{hd_spec}.}

\item{n}{Character or \code{NULL}. Column of raw counts shown in
highcharter tooltips alongside the y value.
Ignored when \code{data} is an \code{hd_spec}.}

\item{colour}{Character or \code{NULL}. ggplot2 colour aesthetic column.
Defaults to \code{group} when \code{NULL} and \code{group} is set.
Ignored when \code{data} is an \code{hd_spec}.}

\item{backend}{Character. Rendering engine - \code{"highcharter"} (default,
interactive) or \code{"ggplot2"} (static), or any engine added with
\code{\link[=register_backend]{register_backend()}}.  Falls back to \code{getOption("highdir.backend", "highcharter")}.}
}
\value{
An S3 object of class \code{"hd"} with slots:
\describe{
\item{\verb{$spec}}{An \code{\link[=hd_spec]{hd_spec()}} object.}
\item{\verb{$geom}}{\code{NULL} until a \verb{+ hd_geom_*()} layer is added.}
\item{\verb{$opts}}{An \code{\link[=hd_opts]{hd_opts()}} object (defaults until overridden).}
\item{\verb{$backend}}{Character. The resolved engine name.}
}
}
\description{
Creates an \code{hd} object that accumulates geometry and presentation layers
via \code{+}, then renders when printed.  This mirrors the way \code{ggplot2} builds
plots: data mapping is declared first; visual decisions are added
incrementally; nothing is rendered until the object hits the console (or an
explicit \code{print()} / \code{knit} call).
}
\details{
\code{hd()} accepts either raw data columns \strong{or} an existing \code{\link[=hd_spec]{hd_spec()}} object.
Passing an \code{hd_spec} is the recommended bridge for code that already
constructs specs separately (e.g. in Shiny or a reporting pipeline).
}
\examples{
df <- data.frame(
  age = rep(c("18-24", "25-34", "35-44", "45-54"), each = 2),
  sex = rep(c("Male", "Female"), 4),
  pct = c(42, 38, 55, 61, 48, 52, 60, 57),
  n   = c(120, 115, 200, 210, 180, 175, 160, 155)
)

# Composable style
hd(df, x = "age", y = "pct", group = "sex") +
  hd_geom_column() +
  hd_opts(title = "Health survey", ylim = c(0, 80))

# Pass an existing hd_spec
spec <- hd_spec(df, x = "age", y = "pct", group = "sex", n = "n")
hd(spec) +
  hd_geom_line(smooth = TRUE) +
  hd_opts(title = "Trend")

# Switch backend per figure
hd(df, x = "age", y = "pct", backend = "ggplot2") +
  hd_geom_column() +
  hd_opts(title = "Static version")

}
\seealso{
\code{\link[=hd_geom_column]{hd_geom_column()}}, \code{\link[=hd_geom_line]{hd_geom_line()}}, \code{\link[=hd_geom_arearange]{hd_geom_arearange()}},
\code{\link[=hd_opts]{hd_opts()}}, \code{\link[=hd_make]{hd_make()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/geom-arearange.R
\name{gg_arearange}
\alias{gg_arearange}
\title{ggplot2 Arearange Geom Function}
\usage{
gg_arearange(spec, opts, geom_params)
}
\arguments{
\item{spec}{An \code{\link[=hd_spec]{hd_spec()}} object.}

\item{opts}{An \code{\link[=hd_opts]{hd_opts()}} object.}

\item{geom_params}{Named list of geom-specific arguments (see above).}
}
\value{
A list of \code{ggplot2} layer objects.
}
\description{
Returns a list of ggplot2 layers (ribbon + optional centre line + points)
that the ggplot2 engine adds to the base figure.  Not intended to be called
directly; use \code{\link[=hd_geom_arearange]{hd_geom_arearange()}} or \code{hd_make(..., type = "arearange")}.
}
\section{geom_params recognised}{

\describe{
\item{\code{ymin}}{Character. \strong{Required.} Column name for the lower bound.}
\item{\code{ymax}}{Character. \strong{Required.} Column name for the upper bound.}
\item{\code{show_line}}{Logical. Draw a centre line + open-circle points on top
of the ribbon.  Default \code{TRUE}.}
\item{\code{single_colour}}{Character or \code{NULL}.  Fixed hex colour injected by
the ggplot2 engine for single-series figures.  \code{NULL} for multi-series
figures (colour is handled by \code{\link[=apply_gg_colors]{apply_gg_colors()}}).}
}
}

\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{.hc_symbols}
\alias{.hc_symbols}
\title{Valid Highcharts marker symbol names}
\usage{
.hc_symbols
}
\description{
Valid Highcharts marker symbol names
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/geom-column.R
\name{hd_geom_column}
\alias{hd_geom_column}
\title{Column Geometry Layer for hd Objects}
\usage{
hd_geom_column(...)
}
\arguments{
\item{...}{Geometry-specific arguments forwarded to \code{\link[=hd_make]{hd_make()}}.}
}
\value{
An S3 object of class \code{"hd_geom"} for use with \code{+.hd}.
}
\description{
\code{hd_geom_column()} creates a column geometry layer that is added to an \code{\link[=hd]{hd()}}
object via \code{+}.  The layer records the geometry type and any geometry-specific
arguments; rendering only happens when the \code{hd} object is printed.
}
\examples{

survey <- data.frame(
  age_group = rep(c("18-24", "25-34", "35-44", "45-54", "55-64"), each = 2),
  kjonn       = rep(c("Male", "Female"), times = 5),
  pct       = c(42, 38, 55, 61, 48, 52, 60, 57, 65, 70),
  n         = c(120, 115, 200, 210, 180, 175, 160, 155, 140, 145)
)

spec_col <- hd_spec(survey,
                    x     = "age_group",
                    y     = "pct",
                    group = "kjonn",
                    n     = "n")

opts_col <- hd_opts(
  title    = "Alcohol use by age group and kjonn",
  subtitle = "Source: Norwegian Directorate of Health",
  ylim     = c(0, 100),
  yint     = 20,
  ylab     = "Percentage (\%)"
)

# Interactive (default)
hd_make(spec_col, "column", opts_col)

# Static ggplot2
hd_make(spec_col, "column", opts_col, backend = "ggplot2")

# Composable style
p <- hd(survey, x = "age_group", y = "pct", group = "kjonn")
p2 <- p + hd_geom_column()

# More options
p2 + hd_opts(title = "Health survey", ylim = c(0, 100))

# Pass an existing hd_spec
spec <- hd_spec(survey, x = "age_group", y = "pct", group = "kjonn", n = "n")

hd(spec, backend = "ggplot2") +
 hd_geom_column() +
 hd_opts(title = "Health survey", ylim = c(0, 80))

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{resolve_symbols}
\alias{resolve_symbols}
\title{Resolve and validate line symbols for n groups}
\usage{
resolve_symbols(n, symbols = NULL)
}
\arguments{
\item{n}{Integer. Number of groups.}

\item{symbols}{Character vector or \code{NULL} supplied by the user.}
}
\value{
Character vector of length \code{n}.
}
\description{
Resolve and validate line symbols for n groups
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dataset.R
\docType{data}
\name{alco2}
\alias{alco2}
\title{Alcohol Consumption by Gender}
\format{
A data frame with 28 rows and 10 variables:
}
\source{
Helsedirektoratet. \emph{Tall om alkohol}.
\url{https://www.helsedirektoratet.no/rapporter/tall-om-alkohol}
}
\description{
Annual estimates of alcohol consumption stratified by \enc{kjønn}{kjonn}
(gender), including adjusted means, standard errors, and 95\\% confidence
intervals. Also includes an alternative unit (\code{adj_enhet}) with uncertainty.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/theme.R
\name{apply_gg_colors}
\alias{apply_gg_colors}
\title{Apply brand colours to a ggplot object}
\usage{
apply_gg_colors(p, colors = NULL, n_groups = NULL, group_levels = NULL)
}
\arguments{
\item{p}{A ggplot object.}

\item{colors}{Character vector, palette name string, or NULL.}

\item{n_groups}{Integer. Number of groups in the data. Drives which
palette rule fires. Passed from \code{ggplot_engine()}.}

\item{group_levels}{Character vector or NULL. Group level names in data
order. Used to name the palette so ggplot2 matches colours to groups
by name rather than alphabetical sort.}
}
\value{
The modified ggplot object.
}
\description{
Uses the same \code{resolve_colors()} priority chain as the highcharter
engine so both backends always produce identical colour assignments:
}
\details{
\enumerate{
\item Explicit \code{colors} argument
\item \code{getOption("highdir.colors")} session default
\item Built-in rules: n==2 -> hdir2, n<=10 -> \code{hdir[1:n]}, n>10 -> viridis
}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hd-object.R
\name{+.hd}
\alias{+.hd}
\title{Add Layers to an hd Object}
\usage{
\method{+}{hd}(e1, e2)
}
\arguments{
\item{e1}{An \code{hd} object (left-hand side).}

\item{e2}{An \code{hd_geom} or \code{hd_opts} object (right-hand side).}
}
\value{
The updated \code{hd} object (invisibly, so the chain prints once).
}
\description{
Implements the \code{+} operator for \code{\link[=hd]{hd()}} objects, mirroring how \code{ggplot2}
builds plots incrementally.  Recognised right-hand sides:
}
\details{
\describe{
\item{\code{hd_geom} object}{Sets the geometry (from any \verb{hd_geom_*()} call).
Adding a second geom replaces the first where highdir renders one geometry
per figure.}
\item{\code{hd_opts} object}{Sets presentation options.  Adding a second
\code{hd_opts} replaces the first.}
}

Unknown right-hand sides produce an informative error rather than silently
being ignored.
}
\examples{
df <- data.frame(age = c("18-24", "25-34"), pct = c(42, 55))

# Chain layers
p <- hd(df, x = "age", y = "pct") +
  hd_geom_column() +
  hd_opts(title = "Demo")

# Reuse a partial object with different opts
base <- hd(df, x = "age", y = "pct") + hd_geom_column()
base + hd_opts(title = "English title")
base + hd_opts(title = "Norsk tittel")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/save.R
\name{hd_save}
\alias{hd_save}
\title{Save a Figure to Disk}
\usage{
hd_save(
  fig,
  file,
  type = "auto",
  width = 8,
  height = 5,
  dpi = 300,
  selfcontained = TRUE,
  ...
)
}
\arguments{
\item{fig}{A \code{highchart} or \code{ggplot} object (output of
\code{\link[=hd_make]{hd_make()}}).}

\item{file}{Character. Output path including extension.}

\item{type}{\code{"auto"} (infer from extension) or an explicit format
string. Default: \code{"auto"}.}

\item{width}{Numeric. Width in inches (ggplot2). Default: \code{8}.}

\item{height}{Numeric. Height in inches (ggplot2). Default: \code{5}.}

\item{dpi}{Numeric. Raster resolution for ggplot2. Default: \code{300}.}

\item{selfcontained}{Logical. Embed all JS/CSS in the HTML file.
Default: \code{TRUE}.}

\item{...}{Passed to \code{\link[ggplot2:ggsave]{ggplot2::ggsave()}} for ggplot2 figures.}
}
\value{
\code{file}, invisibly.
}
\description{
Exports a \code{highchart} or \code{ggplot} figure produced by \code{\link[=hd_make]{hd_make()}} to a
file. The output format is inferred from the file extension unless \code{type}
is supplied explicitly.
}
\details{
\tabular{ll}{
   Backend \tab Supported formats \cr
   highcharter \tab \code{html}, \code{json} \cr
   ggplot2 \tab \code{png}, \code{svg}, \code{pdf}, \code{jpeg}, \code{jpg}, \cr
    \tab \code{tiff}, \code{bmp}, \code{eps} \cr
}


To export a highcharter figure as an image, either save as \code{html} and
screenshot in a browser, or re-render with \code{backend = "ggplot2"} in
\code{\link[=hd_make]{hd_make()}} and save as \code{png}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/registry.R
\name{geom_args}
\alias{geom_args}
\title{Show Arguments for a Geometry}
\usage{
geom_args(type = NULL)
}
\arguments{
\item{type}{Character.  Geometry name, e.g. \code{"line"}, \code{"ranked_bar"}.
If \code{NULL} (default), prints a summary for every registered geometry.}
}
\value{
A data frame of argument metadata, invisibly.  The primary purpose
is the side-effect of printing.
}
\description{
Prints the required and optional \code{...} arguments accepted by a geometry
when used with \code{\link[=hd_make]{hd_make()}}.  This is the primary discoverability tool
for geometry-specific arguments that do not appear in \code{hd_make()}'s
signature.
}
\section{Why this exists}{

\code{hd_make()} uses \code{...} for all geometry-specific arguments so its own
signature stays clean regardless of how many geometries are registered.
The trade-off is that users cannot see available args from \code{hd_make()}
alone.  \code{geom_args()} solves that: it reads the \code{required_args} and
\code{optional_args} fields registered for each geometry and presents them
in a readable table.
}

\examples{
geom_args("line")
geom_args("ranked_bar")
geom_args("arearange")
geom_args()           # all registered geometries

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hd-object.R
\name{print.hd}
\alias{print.hd}
\title{Render an hd Object}
\usage{
\method{print}{hd}(x, ...)
}
\arguments{
\item{x}{An \code{hd} object.}

\item{...}{Ignored; present for S3 consistency.}
}
\value{
The rendered output (a \code{highchart} widget or a \code{ggplot} object),
invisibly.
}
\description{
Printing an \code{\link[=hd]{hd()}} object triggers rendering.  The geometry type and any
geometry-specific parameters are extracted from the stored \code{hd_geom} layer;
presentation options from the \code{hd_opts} layer; the backend from the
\verb{$backend} slot.  All of these are forwarded to \code{\link[=hd_make]{hd_make()}}, which performs
the actual rendering via the registered engine.
}
\details{
You rarely need to call \code{print.hd()} directly since R calls it automatically
when the object appears at the top level, in knitr/Quarto chunks, or in
Shiny \code{renderHighchart()} / \code{renderPlot()} blocks.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/registry.R
\name{list_backends}
\alias{list_backends}
\title{List Registered Backends}
\usage{
list_backends()
}
\value{
Character vector of registered backend names.
}
\description{
List Registered Backends
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/palettes.R
\name{resolve_colors}
\alias{resolve_colors}
\title{Resolve a Colour Vector for n Groups}
\usage{
resolve_colors(n, colors = NULL)
}
\arguments{
\item{n}{Integer. Number of colours required.}

\item{colors}{Character vector, palette name, or \code{NULL}.}
}
\value{
Character vector of exactly length \code{n}.
}
\description{
Returns exactly \code{n} colours. Priority order:
}
\details{
\enumerate{
\item Explicit \code{colors} argument - vector or palette name string.
\item \code{getOption("highdir.colors")} - set via \code{\link[=hd_set_theme]{hd_set_theme()}}.
\item Built-in hdir rules:
\itemize{
\item n == 2  -> \code{"hdir2"} two-colour teal/purple pair
\item n <= 10 -> \code{"hdir"} 10-colour brand palette
\item n > 10  -> viridis continuous scale
}
}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/validate.R
\name{validate_fig_inputs}
\alias{validate_fig_inputs}
\title{Validate inputs before rendering a figure}
\usage{
validate_fig_inputs(spec, opts, type, backend, extra_args)
}
\arguments{
\item{spec}{A \code{hd_spec} object.}

\item{opts}{A \code{hd_opts} object.}

\item{type}{Character. Geometry name.}

\item{backend}{Character. Backend name.}

\item{extra_args}{Named list of additional arguments (for required-arg
check).}
}
\value{
\code{invisible(NULL)} on success.
}
\description{
Called by \code{\link[=hd_make]{hd_make()}} immediately before dispatching to a backend engine.
Stops with an informative message if anything is wrong.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/geom-pie.R
\name{hd_geom_pie}
\alias{hd_geom_pie}
\title{Pie Geometry Layer for hd Objects}
\usage{
hd_geom_pie(inner_size = "0\%", ...)
}
\arguments{
\item{inner_size}{A string specifying the inner radius of the pie as a percentage
of the total radius.  For example, "50\%" creates a donut chart
with a hole in the middle.  The default "0\%" creates a standard pie chart.
This argument is only applicable to the Highcharts backend; it is ignored
by ggplot2 since it does not support donut charts.}

\item{...}{Geometry-specific arguments forwarded to \code{\link[=hd_make]{hd_make()}}.}
}
\value{
An S3 object of class \code{"hd_geom"} for use with \code{+.hd}.
}
\description{
\code{hd_geom_pie()} creates a pie geometry layer that is added to an \code{\link[=hd]{hd()}}
object via \code{+}.  The layer records the geometry type and any geometry-specific
arguments; rendering only happens when the \code{hd} object is printed.
}
\examples{
# Category share dataset (pie)
drinking_freq <- data.frame(
    category = c("Never", "Rarely", "Monthly", "Weekly", "Daily"),
    pct      = c(18, 25, 30, 20, 7)
)

spec_pie <- hd_spec(drinking_freq,
    x    = "category",
    y    = "pct"
)

opts_pie <- hd_opts(
    title = "Drinking frequency",
    subtitle = "Source: Norwegian Directorate of Health",
    ylab = "Share (\%)"
)

# Donut interactive
hd_make(spec_pie, "pie", opts_pie, inner_size = "50\%")

# Composable API style (ggplot2 ignores inner_size)
hd(drinking_freq, x = "category", y = "pct", backend = "ggplot2") +
    hd_geom_pie() +
    hd_opts(
        title = "Drinking frequency",
        subtitle = "Source: Norwegian Directorate of Health"
    )

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/base-figure.R
\name{base_fig}
\alias{base_fig}
\title{Build a Blank Backend Canvas}
\usage{
base_fig(spec, opts, backend)
}
\arguments{
\item{spec}{A \link{hd_spec} object.}

\item{opts}{A \link{hd_opts} object.}

\item{backend}{Character. Backend name.}
}
\value{
A \code{ggplot} or \code{highchart} object.
}
\description{
Constructs the empty backend object (ggplot or highchart) with all
chart-level options applied from \code{spec} and \code{opts}. Called by the
backend engines; you rarely need this directly.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/geom-stacked-column.R
\name{hd_geom_stacked_column}
\alias{hd_geom_stacked_column}
\title{Stacked Column Geometry Layer}
\usage{
hd_geom_stacked_column(stack, stacking = c("normal", "percent"), ...)
}
\arguments{
\item{stack}{Character. Column name for the stack variable.  Each unique value
in this column creates a separate stack (facet) containing all series with that
stack value.  Required.}

\item{stacking}{Character. Stacking mode for the column geometry.  One of
\code{"normal"} (default) or \code{"percent"}.  See Highcharts documentation for
details: https://api.highcharts.com/highcharts/plotOptions.column.stacking}

\item{...}{Additional optional arguments forwarded to the geom function
(e.g. \code{show_line = FALSE}, \code{single_colour = "#025169"}).
Run \code{geom_args("arearange")} for the full list.}
}
\value{
An S3 object of class \code{"hd_geom"} for use with \code{+.hd}.
}
\description{
Create a stacked column geometry layer for \code{hd} objects.  Each stack is a facet
(sub-panel) containing one or more series.  The \code{stack} argument specifies
the column in the data that defines the stacks.  The \code{group} aesthetic in
\code{hd_spec()} defines the series within each stack.  The \code{stacking}
argument controls how the stacks are rendered: \code{"normal"} (default) stacks values
on top of each other, while \code{"percent"} stacks values as percentages of the total
stack height.
}
\examples{

# Example data: medal counts for four countries across three medal types
olympics <- data.frame(
    Country   = rep(c("Norway", "Germany", "United States", "Canada"), each = 3),
    Continent = rep(c("Europe", "Europe", "North America", "North America"), each = 3),
    Medal     = rep(c("Gold", "Silver", "Bronze"), times = 4),
    Count     = c(148, 133, 124, 102, 98, 65, 113, 122, 95, 77, 72, 80)
)

# Define Specification and Options
spec_st <- hd_spec(olympics,
    x     = "Medal",
    y     = "Count",
    group = "Country"
)

opts_st <- hd_opts(
    title    = "Olympic Games all-time medal table, grouped by continent",
    subtitle = "Source: Olympics",
    ylab     = "Count medals"
)

# Interactive - stacks are separated by continent
hd_make(spec_st, "stacked_column", opts_st, stack = "Continent")

# Static ggplot2 - stacks are separated by continent
hd(spec_st, backend = "ggplot2") +
  hd_geom_stacked_column(stack = "Continent") +
  hd_opts(title = "Olympic Games all-time medal table, grouped by continent", ylab = "Count medals")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{.resolve_axis_label}
\alias{.resolve_axis_label}
\title{Resolve an axis label from opts and spec}
\usage{
.resolve_axis_label(opts_label, spec_col)
}
\arguments{
\item{opts_label}{The value from hd_opts()$ylab or $xlab.}

\item{spec_col}{The column name from hd_spec()$y or $x.}
}
\value{
Character string or NULL.
}
\description{
Three-way logic:
NULL   -> hide the axis label entirely
" "    -> use the column name from spec as the fallback
string -> use the string as-is
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/theme.R
\name{hd_add_js}
\alias{hd_add_js}
\title{Inject JavaScript into a Highcharts Widget}
\usage{
hd_add_js(
  hc,
  code = NULL,
  file = NULL,
  plugin = NULL,
  where = c("load", "render")
)
}
\arguments{
\item{hc}{A \code{highchart} object.}

\item{code}{Character or \code{NULL}. Raw JavaScript string.}

\item{file}{Character or \code{NULL}. Path to a \code{.js} file to read and inject.}

\item{plugin}{Character or \code{NULL}. Name of a plugin bundled in \verb{inst/js/}.}

\item{where}{\code{"load"} (default) or \code{"render"}.}
}
\value{
The \code{highchart} object with JS injected.
}
\description{
Appends custom JavaScript to a \code{highchart} object via
\verb{chart.events.<where>}.  Use this for hand-written callbacks and plugins.
For Highcharts built-in modules (accessibility, exporting, etc.) use
\code{\link[highcharter:hc_add_dependency]{highcharter::hc_add_dependency()}} instead.
}
\details{
Exactly one of \code{code}, \code{file}, or \code{plugin} must be supplied.
}
\examples{
if (interactive()) {
spec <- hd_spec(mtcars, "wt", "mpg")
fig  <- hd_make(spec, "scatter")
fig  <- hd_add_js(fig, code = "console.log('chart loaded');")
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/figure-spec.R
\name{hd_spec}
\alias{hd_spec}
\title{Create a Figure Data Specification}
\usage{
hd_spec(data, x, y, group = NULL, n = NULL, colour = NULL)
}
\arguments{
\item{data}{A \code{data.frame} containing all referenced columns.}

\item{x}{Character. Column name for the x-axis variable.}

\item{y}{Character. Column name for the y-axis variable (typically
a percentage or count).}

\item{group}{Character or \code{NULL}. Column used to split data into multiple
series.}

\item{n}{Character or \code{NULL}. Column of raw counts shown in
highcharter tooltips alongside the y value.  Ignored by ggplot2.}

\item{colour}{Character or \code{NULL}. ggplot2 colour aesthetic column.
Defaults to \code{group} when \code{NULL} and \code{group} is set.}
}
\value{
An S3 object of class \code{"hd_spec"}.
}
\description{
Defines the \strong{data mapping} for a figure - which columns map to x, y,
group, and count - independently of any visual presentation choices.
Pass the result to \code{\link[=hd_make]{hd_make()}} together with an optional \code{\link[=hd_opts]{hd_opts()}}
object.
}
\examples{
df <- data.frame(
  age = rep(c("18-24", "25-34", "35-44", "45-54"), each = 2),
  sex = rep(c("Male", "Female"), 4),
  pct = c(42, 38, 55, 61, 48, 52, 60, 57),
  n   = c(120, 115, 200, 210, 180, 175, 160, 155)
)

spec <- hd_spec(df, x = "age", y = "pct", group = "sex", n = "n")
spec

}
\seealso{
\code{\link[=hd_opts]{hd_opts()}}, \code{\link[=hd_make]{hd_make()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/make-figure.R
\name{hd_make}
\alias{hd_make}
\title{Build a Figure from a Specification}
\usage{
hd_make(
  spec,
  type = "column",
  opts = NULL,
  backend = "highcharter",
  use_js = TRUE,
  module = FALSE,
  ...
)
}
\arguments{
\item{spec}{A \link{hd_spec} object from \code{\link[=hd_spec]{hd_spec()}}.}

\item{type}{Character.  Geometry name - one of \code{\link[=list_geoms]{list_geoms()}}:
\code{"column"}, \code{"line"}, \code{"scatter"}, \code{"arearange"}, \code{"pie"}, or any
custom geometry added with \code{\link[=register_geom]{register_geom()}}.}

\item{opts}{A \link{hd_opts} object or \code{NULL} (uses all defaults).
Controls title, subtitle, caption, ylim, yint, flip, per-figure
colours, and highcharter theme.}

\item{backend}{Character.  Rendering engine - \code{"highcharter"} (default,
interactive) or \code{"ggplot2"} (static), or any engine added with
\code{\link[=register_backend]{register_backend()}}.}

\item{use_js}{Logical.  When \code{TRUE} (default) injects a hover-band
\code{htmlwidgets::JS()} callback via \code{point.events.mouseOver/Out}.
Tooltips, accessibility module, and all other Highcharts declarative
features are \strong{always} present.  Set \code{FALSE} for clean, no-custom-JS
widgets.  Ignored by the ggplot2 backend.}

\item{module}{Use available modules js from CDN \url{https://api.highcharts.com/highcharts/}}

\item{...}{Extra arguments forwarded to the geometry function.
Required arguments (e.g. \code{ymin}, \code{ymax} for \code{"arearange"}) \strong{must}
be supplied here.}
}
\value{
A \code{highchart} widget (highcharter backend) or \code{ggplot} object
(ggplot2 backend), invisibly wrapped so knitr/Shiny render it
automatically.
}
\description{
Renders a \link{hd_spec} and \link{hd_opts} pair using the selected backend and
geometry.  This is the central function of the package - everything else
feeds into or flows out of \code{hd_make()}.
}
\section{Workflow}{


\if{html}{\out{<div class="sourceCode r">}}\preformatted{spec <- hd_spec(df, x = "age", y = "pct", group = "sex", n = "n")
opts <- hd_opts(title = "Health survey", ylim = c(0, 80))

hd_make(spec, "column", opts)                       # highcharter (default)
hd_make(spec, "column", opts, backend = "ggplot2")  # static ggplot2
hd_make(spec, "line",   opts, smooth = TRUE)        # smooth spline
hd_make(spec, "pie",    opts)                       # pie / donut
}\if{html}{\out{</div>}}
}

\examples{
df <- data.frame(
  age = rep(c("18-24", "25-34", "35-44", "45-54"), each = 2),
  sex = rep(c("Male", "Female"), 4),
  pct = c(42, 38, 55, 61, 48, 52, 60, 57),
  n   = c(120, 115, 200, 210, 180, 175, 160, 155)
)

spec <- hd_spec(df, x = "age", y = "pct", group = "sex", n = "n")

opts <- hd_opts(title    = "Health survey results",
                 subtitle = "Source: FHI 2024",
                 ylim     = c(0, 80))

\donttest{
# -- Interactive charts (highcharter) --------------------------------------
hd_make(spec, "column", opts)
hd_make(spec, "line",   opts, smooth = TRUE)
hd_make(spec, "line",   opts, smooth = FALSE, dot_size = 6)
hd_make(spec, "scatter")

# Pie chart - group is ignored; x = label, y = value
pie_df   <- data.frame(category = c("A","B","C","D"),
                        value    = c(35, 25, 20, 20))
pie_spec <- hd_spec(pie_df, x = "category", y = "value")
pie_opts <- hd_opts(title = "Share by category")
hd_make(pie_spec, "pie", pie_opts)
hd_make(pie_spec, "pie", pie_opts, inner_size = "50\%")  # donut

# Arearange - requires ymin + ymax in ...
df2   <- cbind(df, lo = df$pct - 5, hi = df$pct + 5)
spec2 <- hd_spec(df2, "age", "pct", group = "sex")
hd_make(spec2, "arearange", opts, ymin = "lo", ymax = "hi")

# -- Disable JS hover band -------------------------------------------------
hd_make(spec, "column", opts, use_js = FALSE)

# -- Static ggplot2 versions -----------------------------------------------
hd_make(spec, "column",  opts, backend = "ggplot2")
hd_make(spec, "line",    opts, backend = "ggplot2")
hd_make(spec, "scatter", opts, backend = "ggplot2")
hd_make(pie_spec, "pie", pie_opts, backend = "ggplot2")

# -- Reuse spec with different presentation --------------------------------
opts_no <- hd_opts(title = "Helseundersøkelse", subtitle = "Alle aldre")
hd_make(spec, "column", opts_no)

# -- Save outputs ----------------------------------------------------------
hd_save(hd_make(spec, "column", opts),               "column.html")
hd_save(hd_make(spec, "column", opts, backend="ggplot2"), "column.png")
}

}
\seealso{
\code{\link[=hd_spec]{hd_spec()}}, \code{\link[=hd_opts]{hd_opts()}}, \code{\link[=hd_save]{hd_save()}}, \code{\link[=hd_set_theme]{hd_set_theme()}},
\code{\link[=list_geoms]{list_geoms()}}, \code{\link[=list_backends]{list_backends()}}, \code{\link[=hd_app]{hd_app()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{check_ylim}
\alias{check_ylim}
\title{Validate the ylim argument}
\usage{
check_ylim(ylim)
}
\description{
Validate the ylim argument
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{.hd_accessibility}
\alias{.hd_accessibility}
\title{Version-safe highcharter accessibility description setter}
\usage{
.hd_accessibility(chart, description)
}
\arguments{
\item{chart}{A highchart object.}

\item{description}{Character. The accessibility description string.}
}
\value{
The modified highchart object.
}
\description{
highcharter 0.9.4 does not export hc_accessibility().
This wrapper patches chart$x$hc_opts$accessibility directly, which works across
all highcharter versions because hc_opts is the raw Highcharts config.
}
\keyword{internal}
